@@ -39,7 +39,6 @@ extern "C" {
 
 /* ======================================================================== */
 /* [MACROS] */
-#define KNH_FLAG_MMF_AFFINE (KNH_FLAG_MMF_SIGNIFICANT|KNH_FLAG_MMF_SYNONYM|KNH_FLAG_MMF_TOTAL|KNH_FLAG_MMF_CONST)
 #define knh_Array_n(a,n)      (a)->list[(n)]
 #define knh_Array_size(a)     (a)->size
 #define knh_Bytes_size(o)      (o)->size
@@ -51,24 +50,8 @@ extern "C" {
 #define B2(c,n)   new_bytes__2(c,n)
 #define STEXT(c)  new_bytes__2(c,sizeof(c)-1)
 #define ISB(t,c) (t.len == (sizeof(c)-1) && knh_strncmp((char*)t.buf,c,t.len) == 0)
-#define CLASS_newid          ((knh_class_t)-1)
-#define CLASS_unknown        ((knh_class_t)-2)
-#define knh_Class_cid(c)     (knh_class_t)(c)->cid
-#define KNH_FLAG_CF2OF(f)        (f)
-#define knh_Class_isGenerics(cid)    (ctx->share->ClassTable[cid].p1 != CLASS_Nue)
-#define KNH_FLAG_SF_FIELD          KNH_FLAG_T1
-#define STRUCT_ISFIELD(sid)        ((sid & KNH_FLAG_SF_FIELD) == KNH_FLAG_SF_FIELD)
-#define BSIZE_TOSID(bsize)         (((knh_struct_t)bsize)|KNH_FLAG_SF_FIELD)
-#define STRUCT_FIELD(bsize)        (((knh_struct_t)bsize)|KNH_FLAG_SF_FIELD)
-#define STRUCT_FIELDSIZE(sid)      (sid & (~KNH_FLAG_SF_FIELD))
-#define STRUCT_UNMASK(sid)         (sid & (~KNH_FLAG_SF_FIELD))
-#define KNH_ASSERT_sid(sid)   KNH_ASSERT(((knh_struct_t)sid) < ctx->share->StructTableSize)
-#define KNH_ASSERT_cid(cid)    KNH_ASSERT(cid < KNH_TCLASS_SIZE)
-#define CLASSN(cid)   knh_ClassTable_CLASSN(ctx, cid)
-#define CLASSNo(o)    knh_ClassTable_CLASSN(ctx, knh_Object_cid(o))
-#define KNH_DEF(ctx, cid)  konoha_getClassDefaultValue(ctx, cid)
-#define KNH_DEF(ctx, cid)  konoha_getClassDefaultValue(ctx, cid)
-#define TYPEN(type)  knh_TYPEN(ctx,type)
+#define KNH_DEF(ctx, cid)  knh_getClassDefaultValue(ctx, cid)
+#define KNH_DEF(ctx, cid)  knh_getClassDefaultValue(ctx, cid)
 #define KNH_STDIN          (((Context*)ctx)->in)
 #define KNH_STDOUT         (((Context*)ctx)->out)
 #define KNH_STDERR         (((Context*)ctx)->err)
@@ -78,8 +61,13 @@ extern "C" {
 #define KNH_ASSERT_eid(eid)    KNH_ASSERT(eid < ctx->share->ExptTableSize + 1)
 #define EXPTN(eid)   knh_String_tochar(knh_ExptTable_name(ctx, eid))
 #define new_Exception__s(ctx, s)     new_Exception__b(ctx, B(s))
-#define KNH_PERRNO(ctx, emsg, func) { \
-		knh_perrno(ctx, emsg, func, __FILE__, __LINE__); \
+#define KNH_PERRNO(ctx, cwb, emsg, func, isThrowable) { \
+		knh_cwb_perrno(ctx, cwb, emsg, func, __FILE__, __LINE__, isThrowable); \
+	}\
+
+
+#define KNH_NOAPI(ctx, cwb, isThrowable) { \
+		knh_throw_Unsupported(ctx, cwb, __FUNCTION__, __FILE__, __LINE__, isThrowable); \
 	}\
 
 
@@ -88,10 +76,8 @@ extern "C" {
 	}\
 
 
-#define knh_Float_value(o)    (o)->value
-#define knh_Int_value(o)      (o)->value
-#define knh_tMapper_find(ctx, scid, tcid) knh_tMapper_find_(ctx, scid, tcid, 1)
-#define knh_Class_getMapper(ctx, scid, tcid)  knh_tMapper_find_(ctx, scid, tcid, 0)
+#define knh_findMapper(ctx, scid, tcid) knh_findMapper_(ctx, scid, tcid, 1)
+#define knh_Class_getMapper(ctx, scid, tcid)  knh_findMapper_(ctx, scid, tcid, 0)
 #define knh_Method_mn(mtd)    DP(mtd)->mn
 #define knh_Method_mf(mtd)    DP(mtd)->mf
 #define knh_MethodField_rztype(mf)   ((mf)->rtype)
@@ -103,35 +89,18 @@ extern "C" {
 #define knh_Class_findMethod(ctx, c, mn)   knh_Class_getMethod__(ctx, c, mn, 1)
 #define CTXCLASSN(cid)     knh_Context_CLASSN(ctx,cid)
 #define CTXTYPEQN(cid)     knh_Context_CLASSN(ctx,CLASS_type(cid)), TYPEQ(cid)
-#define knh_Object_cid(o)           (o)->h.cid
-#define knh_Object_bcid(o)          (o)->h.bcid
-#define IS_NULL(o)          (((Object*)o)->h.cid == CLASS_Nue)
-#define IS_NOTNULL(o)       (((Object*)o)->h.cid != CLASS_Nue)
-#define IS_TRUE(o)         ((o)->h.bcid == CLASS_Boolean && ((Int*)o)->n.bvalue)
-#define IS_FALSE(o)        ((o)->h.bcid == CLASS_Boolean && (((Int*)o)->n.bvalue == 0))
-#define new_Boolean(ctx, c)    (c) ? KNH_TRUE : KNH_FALSE
-#define BOOL_ISTRUE(o)         (o == KNH_TRUE)
-#define BOOL_ISFALSE(o)        (o == KNH_FALSE)
-#define new_Nue__b(ctx, txt)   new_Nue(ctx, new_String(ctx, txt, NULL));
-#define new_Nue__s(ctx, txt)   new_Nue(ctx, new_String(ctx, B(txt), NULL));
-#define knh_write_delim(ctx, w)    knh_write(ctx, w, STEXT(", "))
-#define knh_write_dots(ctx, w)     knh_write(ctx, w, STEXT("..."))
-#define knh_write__i(ctx, w, n)   knh_write__ifmt(ctx, w, KNH_INTPTR_FMT, n)
-#define knh_write__u(ctx, w, n)   knh_write__ifmt(ctx, w, KNH_UINT_FMT, n)
-#define knh_write__x(ctx, w, n)   knh_write__ifmt(ctx, w, KNH_INTPTR_FMTX, n)
-#define knh_write__f(ctx, w, f)  knh_write__ffmt(ctx, w, KNH_FLOAT_FMT, f)
-#define knh_write__e(ctx, w, f)  knh_write__ffmt(ctx, w, KNH_FLOAT_FMTE, f)
-#define knh_write_fn(ctx, w, fn)   knh_write__s(ctx, w, FIELDN(fn))
-#define knh_write__O(ctx, w, o)    knh_format(ctx, w, METHODN__k, o, KNH_NULL)
+#define KNH_FLAG_MMF_AFFINE (KNH_FLAG_MMF_ICAST|KNH_FLAG_MMF_SYNONYM|KNH_FLAG_MMF_TOTAL|KNH_FLAG_MMF_CONST)
 #define knh_String_value(o)   ((o)->str)
 #define knh_String_tochar(o)  (char*)((o)->str)
 #define knh_String_strlen(o)  (o)->size
-#define new_String__T(ctx, text)    new_StringX__T(ctx, CLASS_String, text)
 #define NSN_main           0
 #define FILEN_unknown      0
+#define TYPEN(type)  knh_TYPEN(ctx,type)
+#define knh_write_delim(ctx, w)    knh_write(ctx, w, STEXT(", "))
+#define knh_write_dots(ctx, w)     knh_write(ctx, w, STEXT("..."))
+#define knh_write_fn(ctx, w, fn)   knh_write(ctx, w, B(FIELDN(fn)))
+#define knh_write__O(ctx, w, o)    knh_format(ctx, w, METHODN__k, o, KNH_NULL)
 #define KNH_ASM_JMP(ctx, abr, l)  KNH_ASM_JMP_(ctx, abr, l);
-#define KNH_PERRATA_(ctx, f, l, msgo, msgn)    knh_perrata(ctx, f, l, msgo, msgn)
-#define KNH_PERRATA(ctx, tk, msg)  knh_perrata(ctx, (tk)->fileid, (tk)->line, sToken(tk), msg)
 #define sToken(o)  knh_Token_tochar(ctx, o)
 #define new_TermCONST(ctx, fln, d)   TM(new_TokenCONST(ctx, fln, d))
 #define TERMs_getcid(stmt, n)    CLASS_type(TERMs_gettype(stmt, n))
@@ -144,36 +113,36 @@ extern "C" {
 #define KNH_FLAG_PF_FUNC        KNH_FLAG_T3
 #define KNH_FLAG_PF_NAME        KNH_FLAG_T4
 #define KNH_FLAG_PF_BOL         KNH_FLAG_T7
-#define KNH_THROW(ctx, e) knh_throw(ctx, UP(e), KNH_SAFEFILE(__FILE__), __LINE__)
+#define KNH_THROW(ctx, e) knh_throw(ctx, UP(e), knh_safefile(__FILE__), __LINE__)
 #define KNH_THROWs(ctx, s) \
-	knh_throwException(ctx, new_Exception__b(ctx, B(s)), KNH_SAFEFILE(__FILE__), __LINE__); \
+	knh_throwException(ctx, new_Exception__b(ctx, B(s)), knh_safefile(__FILE__), __LINE__); \
 
 
 #define KNH_THROWf(ctx, fmt, ...) \
 	char throcwb_[256]; \
 	knh_snprintf(throcwb_, sizeof(throcwb_), fmt, ## __VA_ARGS__); \
-	knh_throwException(ctx, new_Exception__b(ctx, B(throcwb_)), KNH_SAFEFILE(__FILE__), __LINE__); \
+	knh_throwException(ctx, new_Exception__b(ctx, B(throcwb_)), knh_safefile(__FILE__), __LINE__); \
 
 
 #define TODO_THROW(ctx) knh_throw_TODO(ctx, (char*)__FILE__, __LINE__, (char*)__FUNCTION__)
-#define KNH_SYS(ctx, n)    konoha_getSystemConst(ctx, n)
+#define KNH_SYS(ctx, n)    knh_getSystemConst(ctx, n)
 #define KNH_SYS_CTX    0
 #define KNH_SYS_STDIN  1
 #define KNH_SYS_STDOUT 2
 #define KNH_SYS_STDERR 3
 #define KNH_SYS_OS     4
 #define KNH_SYS_SCRIPT 5
-#define FIELDN(fn) knh_String_tochar(konoha_getFieldName(ctx, fn))
-#define FILEIDN(fileid) knh_String_tochar(konoha_getFileName(ctx, fileid))
+#define FIELDN(fn) knh_String_tochar(knh_getFieldName(ctx, fn))
+#define URIDN(uri) knh_String_tochar(knh_getResourceName(ctx, uri))
 
 /* ======================================================================== */
 /* [PROTOTYPE] */
-KNHAPI(void) konoha_addAffineMapper(Ctx *ctx, knh_class_t scid, char *text, knh_float_t scale, knh_float_t shift);
 KNHAPI(knh_Array_t*) new_Array(Ctx *ctx, knh_class_t p1, size_t capacity);
 KNHAPI(void) knh_Array_add(Ctx *ctx, Array *o, Any *value);
 KNHAPI(Bytes*) new_Bytes(Ctx *ctx, size_t capacity);
 KNHAPI(knh_bytes_t) knh_Bytes_tobytes(Bytes *o);
 KNHAPI(void) knh_Bytes_clear(Bytes *o, size_t pos);
+KNHAPI(void) knh_Bytes_ensureSize(Ctx *ctx, Bytes *o, size_t len);
 KNHAPI(void) knh_Bytes_putc(Ctx *ctx, Bytes *o, int ch);
 KNHAPI(void) knh_Bytes_write(Ctx *ctx, Bytes *o, knh_bytes_t v);
 KNHAPI(knh_bytes_t) new_bytes(char *c);
@@ -187,44 +156,62 @@ KNHAPI(knh_bool_t) knh_bytes_startsWith(knh_bytes_t v1, knh_bytes_t v2);
 KNHAPI(knh_bool_t) knh_bytes_endsWith(knh_bytes_t v1, knh_bytes_t v2);
 KNHAPI(knh_index_t) knh_bytes_index(knh_bytes_t v, knh_intptr_t ch);
 KNHAPI(knh_index_t) knh_bytes_rindex(knh_bytes_t v, knh_intptr_t ch);
+KNHAPI(knh_bytes_t) knh_bytes_mod(knh_bytes_t t, int ch);
+KNHAPI(knh_bytes_t) knh_bytes_rmod(knh_bytes_t t, int ch);
 KNHAPI(knh_bytes_t) knh_bytes_first(knh_bytes_t t, knh_intptr_t loc);
 KNHAPI(knh_bytes_t) knh_bytes_last(knh_bytes_t t, knh_intptr_t loc);
 KNHAPI(knh_bytes_t) knh_bytes_skipscheme(knh_bytes_t t);
 KNHAPI(char*) knh_format_bytes(char *buf, size_t bufsiz, knh_bytes_t t);
+KNHAPI(char*) knh_format_join2(char *buf, size_t bufsiz, knh_bytes_t t, knh_bytes_t t2);
 KNHAPI(knh_intptr_t) knh_bytes_toint(knh_bytes_t t);
 KNHAPI(knh_float_t) knh_bytes_tofloat(knh_bytes_t t);
 KNHAPI(knh_int64_t) knh_bytes_toint64(knh_bytes_t t);
-KNHAPI(String*) new_String__fbcnv(Ctx *ctx, String *s, knh_fbcnv fbcnv, BytesConv *bc);
+KNHAPI(int) knh_bytes_parseURLscheme(knh_bytes_t url, char *buf, size_t bufsiz);
+KNHAPI(int) knh_bytes_parseURLuname(knh_bytes_t url, char *buf, size_t bufsiz);
+KNHAPI(int) knh_bytes_parseURL(knh_bytes_t url, char *buf, size_t bufsiz);
+KNHAPI(knh_bytes_t) knh_bytes_substringURLpath(knh_bytes_t url);
+KNHAPI(int) knh_bytes_parseURLpath(knh_bytes_t url, char *buf, size_t bufsiz);
+KNHAPI(int) knh_bytes_parseURLhost(knh_bytes_t url, char *buf, size_t bufsiz);
+KNHAPI(int) knh_bytes_parseURLport(knh_bytes_t url, int *port);
+KNHAPI(String*) new_String__fbcnv(Ctx *ctx, String *s, knh_fbyteconv fbcnv, BytesConv *bc);
+KNHAPI(ClassSpec*) new_Enum(Ctx *ctx, char *tag, knh_bytes_t urn, knh_int_t min, knh_int_t max);
+KNHAPI(ClassSpec*) new_Unit(Ctx *ctx, char *tag, knh_bytes_t urn, knh_float_t min, knh_float_t max, knh_float_t step);
+KNHAPI(ClassSpec*) new_Vocab(Ctx *ctx, char *tag, knh_bytes_t urn, int base, char **terms);
+KNHAPI(void) knh_loadURNAliasData(Ctx *ctx, knh_StringConstData_t *data);
+KNHAPI(void) knh_loadClassSpecFuncData(Ctx *ctx, knh_NamedPointerData_t *data);
 KNHAPI(void) knh_putsfp(Ctx *ctx, knh_sfp_t *lsfp, int n, Object *obj);
-KNHAPI(void) knh_Closure_invokesfp(Ctx *ctx, Closure *c, knh_sfp_t *lsfp, int argc);
+KNHAPI(void) knh_Closure_invokesfp(Ctx *ctx, Closure *cc, knh_sfp_t *lsfp, int argc);
 KNHAPI(knh_sfp_t*) knh_Closure_invokef(Ctx *ctx, Closure *c, const char *fmt, ...);
+KNHAPI(void) knh_Closure_invoke(Ctx *ctx, Closure *c, const char *fmt, ...);
 KNHAPI(Exception*) new_Exception(Ctx *ctx, String *msg);
 KNHAPI(Exception*) new_Exception__b(Ctx *ctx, knh_bytes_t msg);
-KNHAPI(void) knh_perrno(Ctx *ctx, char *emsg, char *func, char *file, int line);
+KNHAPI(void) knh_cwb_perrno(Ctx *ctx, knh_cwb_t *cwb, char *emsg, char *func, char *file, int line, int isThrowable);
+KNHAPI(void) knh_throw_Unsupported(Ctx *ctx, knh_cwb_t *cwb, const char *func, char *file, int line, int isThrowable);
 KNHAPI(void) knh_throw_OutOfIndex(Ctx *ctx, knh_int_t n, size_t max, char *file, int line);
-KNHAPI(ClassSpec*) new_Unit(Ctx *ctx, char *tag, knh_float_t min, knh_float_t max, knh_float_t defv, knh_float_t step);
 KNHAPI(InputStream*) new_InputStream__io(Ctx *ctx, String *urn, knh_io_t fd, knh_iodrv_t *drv);
 KNHAPI(InputStream*) new_InputStream__FILE(Ctx *ctx, String *urn, FILE *fp, knh_iodrv_t *drv);
-KNHAPI(InputStream*) new_FileInputStream(Ctx *ctx, knh_bytes_t file);
+KNHAPI(InputStream*) new_FileInputStream(Ctx *ctx, knh_bytes_t file, int isThrowable);
 KNHAPI(InputStream*) new_BytesInputStream(Ctx *ctx, Bytes *ba, size_t s, size_t e);
 KNHAPI(InputStream*) new_StringInputStream(Ctx *ctx, String *str, size_t s, size_t e);
+KNHAPI(void) knh_Iterator_close(Ctx *ctx, Iterator *it);
+KNHAPI(Iterator*) new_Iterator(Ctx *ctx, knh_class_t p1, Any *source, knh_fitrnext fnext);
+KNHAPI(Iterator*) new_GlueIterator(Ctx *ctx, knh_class_t p1, void *ref, knh_fitrnext fnext, knh_ffree ffree);
+KNHAPI(Mapper*) new_Mapper(Ctx *ctx, knh_flag_t flag, knh_class_t scid, knh_class_t tcid, knh_fmapper fmap, Object *mapdata);
+KNHAPI(void) knh_addMapper(Ctx *ctx, Mapper *mpr);
+KNHAPI(knh_class_t) knh_findcid(Ctx *ctx, knh_bytes_t lname);
+KNHAPI(Int*) new_Int(Ctx *ctx, knh_int_t value);
+KNHAPI(Int*) new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_int_t value);
+KNHAPI(Int*) new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value);
+KNHAPI(Float*) new_Float(Ctx *ctx, knh_float_t value);
+KNHAPI(Float*) new_FloatX__fast(Ctx *ctx, knh_class_t cid, knh_float_t value);
+KNHAPI(Float*) new_FloatX(Ctx *ctx, knh_class_t cid, knh_float_t value);
+KNHAPI(void) knh_addAffineMapper(Ctx *ctx, knh_class_t scid, char *text, knh_float_t scale, knh_float_t shift);
 KNHAPI(void) knh_Glue_init(Ctx *ctx, knh_Glue_t *g, void *ptr, knh_fgfree gfree);
 KNHAPI(Object*) new_Glue(Ctx *ctx, char *lname, void *ptr, knh_fgfree gfree);
 KNHAPI(OutputStream*) new_OutputStream__io(Ctx *ctx, String *urn, knh_io_t fd, knh_iodrv_t *drv);
 KNHAPI(OutputStream*) new_OutputStream__FILE(Ctx *ctx, String *urn, FILE *fp, knh_iodrv_t *drv);
-KNHAPI(OutputStream*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, char *mode);
+KNHAPI(OutputStream*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, char *mode, int isThrowable);
 KNHAPI(OutputStream*) new_BytesOutputStream(Ctx *ctx, Bytes *ba);
-KNHAPI(void) knh_putc(Ctx *ctx, OutputStream *w, int ch);
-KNHAPI(void) knh_write(Ctx *ctx, OutputStream *w, knh_bytes_t s);
-KNHAPI(void) knh_flush(Ctx *ctx, OutputStream *w);
-KNHAPI(void) knh_print(Ctx *ctx, OutputStream *w, knh_bytes_t s);
-KNHAPI(void) knh_println(Ctx *ctx, OutputStream *w, knh_bytes_t s);
-KNHAPI(void) knh_write_EOL(Ctx *ctx, OutputStream *w);
-KNHAPI(void) knh_write_TAB(Ctx *ctx, OutputStream *w);
-KNHAPI(void) knh_write_BOL(Ctx *ctx, OutputStream *w);
-KNHAPI(void) knh_format(Ctx *ctx, OutputStream *w, knh_methodn_t mn, Object *o, Any *m);
-KNHAPI(void) knh_printf(Ctx *ctx, OutputStream *w, char *fmt, ...);
-KNHAPI(void) konoha_says(Ctx *ctx, int type, char *fmt, ...);
 KNHAPI(void) knh_ResultSet_initColumn(Ctx *ctx, ResultSet *o, size_t column_size);
 KNHAPI(void) knh_ResultSet_setName(Ctx *ctx, ResultSet *o, size_t n, String *name);
 KNHAPI(void) knh_ResultSet_initData(Ctx *ctx, ResultSet *o);
@@ -235,38 +222,61 @@ KNHAPI(void) knh_ResultSet_setBlob(Ctx *ctx, ResultSet *o, size_t n, knh_bytes_t
 KNHAPI(void) knh_ResultSet_setNULL(Ctx *ctx, ResultSet *o, size_t n);
 KNHAPI(String*) new_String(Ctx *ctx, knh_bytes_t t, String *orign);
 KNHAPI(knh_bytes_t) knh_String_tobytes(String *o);
-KNHAPI(knh_bytes_t) knh_StringNULL_tobytes(String *o, knh_bytes_t def);
-KNHAPI(void) konoha_addParserDriver(Ctx *ctx, char *alias, knh_parser_drvapi_t *d);
-KNHAPI(ClassSpec*) new_Vocabulary(Ctx *ctx, char *lang, char *defval, ...);
-KNHAPI(void) konoha_addVocabularyMapper(Ctx *ctx, knh_class_t scid, char *text);
-KNHAPI(void) konoha_addAliasURN(Ctx *ctx, char *alias, char *urn);
-KNHAPI(FILE*) knh_fopen(Ctx *ctx, char *filename, char *mode);
-KNHAPI(size_t) knh_fgetc(Ctx *ctx, FILE *fp);
+KNHAPI(char*) knh_String_text(Ctx *ctx, String *s);
+KNHAPI(void) knh_addParserDriver(Ctx *ctx, char *alias, knh_parser_drvapi_t *d);
+KNHAPI(void) knh_setSystemPropertyText(Ctx *ctx, char *key, char *value);
+KNHAPI(void) knh_putc(Ctx *ctx, OutputStream *w, int ch);
+KNHAPI(void) knh_write(Ctx *ctx, OutputStream *w, knh_bytes_t s);
+KNHAPI(void) knh_flush(Ctx *ctx, OutputStream *w);
+KNHAPI(void) knh_print(Ctx *ctx, OutputStream *w, knh_bytes_t s);
+KNHAPI(void) knh_println(Ctx *ctx, OutputStream *w, knh_bytes_t s);
+KNHAPI(void) knh_write_EOL(Ctx *ctx, OutputStream *w);
+KNHAPI(void) knh_write_TAB(Ctx *ctx, OutputStream *w);
+KNHAPI(void) knh_write_BOL(Ctx *ctx, OutputStream *w);
+KNHAPI(void) knh_format(Ctx *ctx, OutputStream *w, knh_methodn_t mn, Object *o, Any *m);
+KNHAPI(void) knh_printf(Ctx *ctx, OutputStream *w, char *fmt, ...);
+KNHAPI(void) knh_says(Ctx *ctx, int type, char *fmt, ...);
+KNHAPI(FILE*) knh_fopen(Ctx *ctx, char *filename, char *mode, int isThrowable);
+KNHAPI(int) knh_fgetc(Ctx *ctx, FILE *fp);
 KNHAPI(size_t) knh_fread(Ctx *ctx, void *ptr, size_t size, FILE *fp);
 KNHAPI(size_t) knh_fwrite(Ctx *ctx, void *ptr, size_t size, FILE *fp);
 KNHAPI(int) knh_fflush(Ctx *ctx, FILE *fp);
 KNHAPI(int) knh_fclose(Ctx *ctx, FILE *fp);
-KNHAPI(void) konoha_addIODriver(Ctx *ctx, char *alias, knh_iodrv_t *d);
-KNHAPI(void) konoha_addRegexDriver(Ctx *ctx, char *alias, knh_regex_drvapi_t *d);
-KNHAPI(knh_intptr_t) knh_socket_open(Ctx *ctx, char *ip_or_host, int port);
+KNHAPI(void) knh_addIODriver(Ctx *ctx, char *alias, knh_iodrv_t *d);
+KNHAPI(void) knh_addRegexDriver(Ctx *ctx, char *alias, knh_regex_drvapi_t *d);
+KNHAPI(knh_intptr_t) knh_socket_open(Ctx *ctx, char *ip_or_host, int port, int isThrowable);
 KNHAPI(int) knh_socket_send(Ctx *ctx, knh_intptr_t sd, char *buf, size_t bufsiz, int flags);
 KNHAPI(int) knh_socket_recv(Ctx *ctx, knh_intptr_t sd, char *buf, size_t bufsiz, int flags);
 KNHAPI(int) knh_socket_close(Ctx *ctx, knh_intptr_t sd);
-KNHAPI(void) konoha_addDBDriver(Ctx *ctx, char *alias, knh_db_drvapi_t *d);
-KNHAPI(char*) knh_format_ospath(Ctx *ctx, char *buf, size_t bufsiz, knh_bytes_t path);
-KNHAPI(Ctx*) konoha_getCurrentContext(void);
-KNHAPI(Ctx*) konoha_getThreadContext(Ctx *ctx);
-KNHAPI(int) konoha_isDebugMode();
-KNHAPI(int) konoha_parseopt(konoha_t konoha, int argc, char **argv);
-KNHAPI(char*) konoha_eval(konoha_t konoha, char *script);
-KNHAPI(void) konoha_readFile(Ctx *ctx, char *fpath);
-KNHAPI(void) konoha_read(konoha_t konoha, char *fpath);
-KNHAPI(int) konoha_main(konoha_t konoha, int argc, char **argv);
-KNHAPI(void) konoha_shell(konoha_t konoha);
+KNHAPI(void) knh_addDBDriver(Ctx *ctx, char *alias, knh_db_drvapi_t *d);
 KNHAPI(void) konoha_SETv(Ctx *ctx, Object **v, Object *o);
 KNHAPI(void) konoha_FINALv(Ctx *ctx, Object **v);
-KNHAPI(char*) KNH_SAFEFILE(char *file);
+KNHAPI(Ctx*) knh_beginContext(Ctx *ctx);
+KNHAPI(void) knh_endContext(Ctx *ctx);
+KNHAPI(Ctx*) knh_getCurrentContext(void);
+KNHAPI(int) konoha_debugLevel(void);
+KNHAPI(int) konoha_isSystemDump2(void);
+KNHAPI(int) konoha_parseopt(konoha_t konoha, int argc, char **argv);
+KNHAPI(int) konoha_hasRuntimeError(konoha_t konoha);
+KNHAPI(char*) konoha_getRuntimeError(konoha_t konoha);
+KNHAPI(void) konoha_setOutputStreamBuffer(konoha_t konoha, size_t osize, size_t esize);
+KNHAPI(char*) konoha_getStdOutBufferText(konoha_t konoha);
+KNHAPI(char*) konoha_getStdErrBufferText(konoha_t konoha);
+KNHAPI(void) konoha_evalScript(konoha_t konoha, char *script);
+KNHAPI(int) konoha_loadScript(konoha_t konoha, char *fpath);
+KNHAPI(int) konoha_runMain(konoha_t konoha, int argc, char **argv);
+KNHAPI(int) konoha_hasScriptFunc(konoha_t konoha, char *fmt);
+KNHAPI(void) konoha_invokeScriptFunc(konoha_t konoha, char *fmt, ...);
+KNHAPI(knh_int_t) konoha_invokeIntFunc(konoha_t konoha, char *fmt, ...);
+KNHAPI(knh_float_t) konoha_invokeFloatFunc(konoha_t konoha, char *fmt, ...);
+KNHAPI(int) konoha_invokeBooleanFunc(konoha_t konoha, char *fmt, ...);
+KNHAPI(char*) konoha_invokeStringFunc(konoha_t konoha, char *fmt, ...);
+KNHAPI(int) konoha_setMethodFunc(konoha_t konoha, char *name, knh_fmethod func);
+KNHAPI(void) konoha_shell(konoha_t konoha);
+KNHAPI(char*) knh_safefile(char *file);
 KNHAPI(Object*) new_Object_boxing(Ctx *ctx, knh_class_t cid, knh_sfp_t *sfp);
+KNHAPI(char*) knh_getPassword(Ctx *ctx, knh_bytes_t url);
+KNHAPI(void) knh_stack_checkSecurityManager(Ctx *ctx, knh_sfp_t *sfp);
 KNHAPI(void) knh_sfp_boxing(Ctx *ctx, knh_sfp_t *sfp);
 KNHAPI(void) knh_sfp_unboxing(Ctx *ctx, knh_sfp_t *sfp);
 KNHAPI(void) knh_esp1_sformat(Ctx *ctx, Method *mtd, OutputStream *w, Any *m);
@@ -276,11 +286,11 @@ KNHAPI(void) knh_throw_bugstop(Ctx *ctx, char *file, int line, char *func);
 KNHAPI(void) knh_throwException(Ctx *ctx, Exception *e, char *file, int line);
 KNHAPI(void) knh_throw__s(Ctx *ctx, char *msg, char *file, int line);
 KNHAPI(void) knh_throw(Ctx *ctx, Object *e, char *file, int line);
+KNHAPI(void) knh_throwStupidException(Ctx *ctx);
 KNHAPI(void) konoha_throwSecurityException(void);
-KNHAPI(void) konoha_loadIntConstData(Ctx *ctx, knh_IntConstData_t *data);
-KNHAPI(void) konoha_loadFloatConstData(Ctx *ctx, knh_FloatConstData_t *data);
-KNHAPI(void) konoha_loadStringConstData(Ctx *ctx, knh_StringConstData_t *data);
-KNHAPI(void) konoha_addClassSpecFunc(Ctx *ctx, char *urn, knh_fspec func);
+KNHAPI(void) knh_loadIntConstData(Ctx *ctx, knh_IntConstData_t *data);
+KNHAPI(void) knh_loadFloatConstData(Ctx *ctx, knh_FloatConstData_t *data);
+KNHAPI(void) knh_loadStringConstData(Ctx *ctx, knh_StringConstData_t *data);
 KNHAPI(konoha_t) konoha_open(size_t stacksize);
 KNHAPI(void) konoha_close(konoha_t konoha);
 
