@@ -74,16 +74,18 @@ void knh_OutputStream_setEncoding(Ctx *ctx, OutputStream *o, String *enc);
 /* ../src/api/stringapi.c */
 size_t bconv__toLower(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba);
 size_t bconv__toUpper(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba);
+Array *knh_Regex_split(Ctx *ctx, Regex *o, String *s);
+void knh_Regex__s(Ctx *ctx, Regex *o, OutputStream *w, String *m);
 /* ../src/api/systemapi.c */
+METHOD knh__System_setRandomSeed(Ctx *ctx, knh_sfp_t *sfp);
 Object* knh_Context_getProperty(Ctx *ctx, Context *b, knh_bytes_t key);
 void knh_Context_setProperty(Ctx *ctx, Context *b, String *key, Any *value);
 void knh_System__k(Ctx *ctx, System *o, OutputStream *w, String *m);
 void knh_System__dump(Ctx *ctx, System *o, OutputStream *w, String *m);
 void knh_Context_setEncoding(Ctx *ctx, knh_Context_t *o, String *enc);
-/* ../src/class/knh_AffineConv.c */
-AffineConv *new_AffineConv(Ctx *ctx, knh_float_t fa, knh_float_t fb);
 /* ../src/class/knh_Array.c */
 size_t knh_array_newsize(size_t newsize, size_t usize);
+size_t knh_array_index0(Ctx *ctx, knh_intptr_t n, size_t size);
 size_t knh_array_index(Ctx *ctx, knh_intptr_t n, size_t size);
 void knh_Array_grow(Ctx *ctx, Array *a, size_t newsize, Object *v);
 knh_Array_t* new_Array0(Ctx *ctx, size_t capacity);
@@ -101,19 +103,19 @@ void knh_Bytes_unputc(Bytes *o);
 size_t knh_size(size_t s);
 knh_bytes_t knh_bytes_subset(knh_bytes_t t, size_t s, size_t e);
 knh_bytes_t knh_bytes_offlen(knh_bytes_t t, size_t off, size_t len);
-knh_bytes_t knh_bytes_trim(knh_bytes_t t, knh_intptr_t ch);
-int knh_bytes_parseinteger(knh_bytes_t t, knh_int_t *value);
+knh_bytes_t knh_bytes_trim(knh_bytes_t t /*, knh_intptr_t ch*/);
+int knh_bytes_parseint(knh_bytes_t t, knh_int_t *value);
 int knh_bytes_parsefloat(knh_bytes_t t, knh_float_t *value);
 /* ../src/class/knh_BytesConv.c */
 size_t knh_BytesConv_conv(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba);
-BytesConv* new_BytesConv(Ctx *ctx, String *name, knh_fbcnv fbconv);
+iconv_t knh_iconv_open(Ctx *ctx, char *to, char *from);
+void knh_iconv_close(Ctx *ctx, iconv_t cd);
 BytesConv* new_BytesConv__iconv(Ctx *ctx, char *from, char *to);
 BytesConv* new_BytesConv__in(Ctx *ctx, char *from);
 BytesConv* new_BytesConv__out(Ctx *ctx, char *to);
 String *new_String__bconv(Ctx *ctx, knh_bytes_t t, BytesConv *bc);
 String *new_String__cwbconv(Ctx *ctx, knh_cwb_t cwb, BytesConv *bc);
 void knh_OutputStream_write__bconv(Ctx *ctx, OutputStream *w, knh_bytes_t t);
-void knh_BytesConv__k(Ctx *ctx, BytesConv *o, OutputStream *w, String *m);
 /* ../src/class/knh_Class.c */
 knh_struct_t knh_StructTable_newId(Ctx *ctx);
 knh_class_t knh_ClassTable_newId(Ctx *ctx);
@@ -124,6 +126,8 @@ Object *konoha_getClassDefaultValue(Ctx *ctx, knh_class_t cid);
 Object *konoha_getDefaultValue(Ctx *ctx, knh_type_t type);
 void konoha_setClassParam(Ctx *ctx, knh_class_t cid, knh_class_t p1, knh_class_t p2);
 knh_class_t konoha_addGenericsClass(Ctx *ctx, knh_class_t cid, String *name, knh_class_t bcid, knh_class_t p1, knh_class_t p2);
+void KNH_ACLASS(Ctx *ctx, knh_class_t cid, knh_class_t p1);
+void KNH_ICLASS(Ctx *ctx, knh_class_t cid, knh_class_t p1);
 ClassStruct* new_ClassStruct0(Ctx *ctx, int field_size, int method_size);
 knh_index_t knh_Class_indexOfField(Ctx *ctx, knh_class_t cid, knh_fieldn_t fn);
 knh_index_t knh_Class_queryField(Ctx *ctx, knh_class_t cid, knh_fieldn_t fnq);
@@ -138,15 +142,13 @@ void knh_ClassMap_sort(Ctx *ctx, ClassMap *o);
 void knh_ClassMap_add(Ctx *ctx, ClassMap *o, Mapper *map);
 void knh_ClassMap__dump(Ctx *ctx, ClassMap *o, OutputStream *w, String *m);
 void knh_ClassMap__man(Ctx *ctx, ClassMap *o, OutputStream *w, knh_class_t cid);
-/* ../src/class/knh_Class_type_t.c */
-knh_class_t knh_pmztype_toclass(Ctx *ctx, knh_type_t t, knh_class_t this_cid);
-knh_type_t knh_pmztype_totype(Ctx *ctx, knh_type_t t, knh_class_t this_cid);
-knh_class_t knh_class_Array(Ctx *ctx, knh_class_t p1);
-knh_class_t knh_class_Iterator(Ctx *ctx, knh_class_t p1);
-knh_class_t knh_class_Generics(Ctx *ctx, knh_class_t bcid, knh_class_t p1, knh_class_t p2);
-char* knh_format_type(Ctx *ctx, char *buf, size_t bufsiz, knh_type_t type);
-char *TYPEQ(knh_type_t type);
-char *knh_TYPEN(Ctx *ctx, knh_type_t type);
+/* ../src/class/knh_ClassSpec.c */
+void knh_write_intx(Ctx *ctx, OutputStream *w, ClassSpec *u, knh_int_t v);
+void knh_write_floatx(Ctx *ctx, OutputStream *w, ClassSpec *u, knh_float_t v);
+void knh_ClassSpec_reuse(Ctx *ctx, ClassSpec *u, knh_class_t cid);
+knh_bytes_t konoha_getURNAlias(Ctx *ctx, knh_bytes_t aurn);
+ClassSpec *konoha_getClassSpec(Ctx *ctx, knh_class_t cid);
+knh_class_t konoha_findcidx(Ctx *ctx, knh_bytes_t lname);
 /* ../src/class/knh_Closure.c */
 METHOD knh__Closure_new(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Closure_invoke(Ctx *ctx, knh_sfp_t *sfp);
@@ -222,15 +224,9 @@ knh_expt_t konoha_addException(Ctx *ctx, knh_flag_t flag, knh_class_t eid, Strin
 Exception* knh_Exception_new__init(Ctx *ctx, Exception *o, String *e, String *msg, Object *bag);
 int knh_Exception_isa(Ctx *ctx, Exception *o, String *msg);
 Exception* new_Exception__Nue(Ctx *ctx, Object *o);
-/* ../src/class/knh_ExceptionHandler.c */
 ExceptionHandler* new_ExceptionHandler(Ctx *ctx);
 void knh_ExceptionHandler_longjmp(Ctx *ctx, ExceptionHandler *o, Exception *e);
 Exception* knh_ExceptionHandler_getCaughtException(ExceptionHandler *o);
-/* ../src/class/knh_Float.c */
-FloatUnit *new_FloatUnit(Ctx *ctx, knh_flag_t flag, knh_class_t cid, String *tag, knh_float_t min, knh_float_t max, knh_float_t defvalue, knh_float_t step);
-Float* new_Float(Ctx *ctx, knh_float_t value);
-Float* new_FloatX__fast(Ctx *ctx, knh_class_t cid, knh_float_t value);
-Float* new_FloatX(Ctx *ctx, knh_class_t cid, knh_float_t value);
 /* ../src/class/knh_Hash.c */
 knh_hashentry_t *new_hashentry(Ctx *ctx, knh_Hash_t *o);
 void knh_hashentry_collect(Ctx *ctx, knh_Hash_t *o, knh_hashentry_t *e);
@@ -244,43 +240,32 @@ void knh_Hash_rehash(Ctx *ctx, knh_Hash_t *o);
 void knh_HashMap_set__hcode(Ctx *ctx, HashMap *o, knh_hcode_t hcode, Any *value);
 void knh_HashSet_set__hcode(Ctx *ctx, HashSet *o, knh_hcode_t hcode, knh_uintptr_t uvalue);
 /* ../src/class/knh_InputStream.c */
-Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn);
+Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn, String *m);
 int knh_InputStream_getc(Ctx *ctx, InputStream *o);
 size_t knh_InputStream_read(Ctx *ctx, InputStream *o, char *buf, size_t bufsiz);
 String* knh_InputStream_readLine(Ctx *ctx, InputStream *o);
 void knh_InputStream_close(Ctx *ctx, InputStream *o);
 int knh_InputStream_isClosed(InputStream *o);
 MAPPER knh_Bytes_InputStream(Ctx *ctx, knh_sfp_t *sfp);
-/* ../src/class/knh_Int.c */
-Int* new_Int(Ctx *ctx, knh_int_t value);
-Int* new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_int_t value);
-Int *new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value);
-knh_int_t knh_IntNULL_toint(Int *v);
-/* ../src/class/knh_IntUnit.c */
-void knh_IntUnit_initMinMax(Ctx *ctx, IntUnit *b, knh_int_t min, knh_int_t max);
-void knh_IntUnit_initDefaultValue(Ctx *ctx, IntUnit *b, knh_int_t v);
-IntUnit *new_IntUnit(Ctx *ctx, knh_flag_t flag, knh_class_t cid, String *urn, String *tag, knh_int_t min, knh_int_t max, knh_int_t defvalue);
-void knh_IntUnit_initId(Ctx *ctx, IntUnit *b, knh_flag_t flag, knh_class_t cid, String *urn, String *tag);
 /* ../src/class/knh_Iterator.c */
 void knh_Iterator_close(Ctx *ctx, Iterator *it);
 Iterator* new_Iterator(Ctx *ctx, knh_class_t p1, Any *source, knh_fitrnext fnext);
 /* ../src/class/knh_Mapper.c */
-Mapper* new_Mapper(Ctx *ctx, knh_flag_t flag, knh_class_t scid, knh_class_t tcid, knh_fmapper fmap, Object *mapdata);
 Mapper* new_MapMap(Ctx *ctx, Mapper *m1, Mapper *m2);
 void knh_Mapper__k(Ctx *ctx, Mapper *o, OutputStream *w, String *m);
-void konoha_addMapper(Ctx *ctx, knh_flag_t flag, knh_type_t stype, knh_type_t ttype, knh_fmapper fmap, Object *mapdata);
-MAPPER knh_Mapper_fInterface(Ctx *ctx, knh_sfp_t *sfp);
+void konoha_addMapperFunc(Ctx *ctx, knh_flag_t flag, knh_type_t stype, knh_type_t ttype, knh_fmapper fmap, Object *mapdata);
+MAPPER knh_Mapper_fasis(Ctx *ctx, knh_sfp_t *sfp);
 MAPPER knh_fmapper_null(Ctx *ctx, knh_sfp_t *sfp);
 knh_bool_t knh_Mapper_isNoSuchMapping(Mapper *mpr);
-Mapper *knh_tMapper_find_(Ctx *ctx, knh_class_t scid, knh_class_t tcid, int isgen);
+Mapper *konoha_findMapper_(Ctx *ctx, knh_class_t scid, knh_class_t tcid, int isgen);
 /* ../src/class/knh_Method.c */
 MethodField* new_MethodField(Ctx *ctx, knh_type_t rtype, size_t psize);
 knh_hcode_t knh_mparam_hcode(knh_hcode_t h, knh_type_t type, knh_fieldn_t fn);
 void knh_MethodField_set(MethodField *o, size_t n, knh_type_t type, knh_fieldn_t fn);
 knh_type_t knh_MethodField_pztype(MethodField *o, size_t n);
 knh_mparam_t knh_MethodField_param(MethodField *o, size_t n);
-knh_type_t knh_Method_rtype(Ctx *ctx, knh_class_t cid, Method *mtd);
-knh_type_t knh_Method_ptype(Ctx *ctx, knh_class_t cid, Method *o, size_t n);
+knh_type_t knh_Method_rtype(Ctx *ctx, Method *mtd, knh_class_t cid);
+knh_type_t knh_Method_ptype(Ctx *ctx, Method *o, knh_class_t cid, size_t n);
 knh_mparam_t knh_Method_param(Method *o, size_t n);
 int knh_MethodField_equalsType(MethodField *o, MethodField *o2);
 METHOD knh_fmethod_abstract(Ctx *ctx, knh_sfp_t *sfp);
@@ -309,7 +294,6 @@ NameSpace* new_NameSpace(Ctx *ctx, String *nsname);
 void knh_NameSpace_importNameSpace(Ctx *ctx, NameSpace *ns, NameSpace *other);
 Script *knh_NameSpace_getScript(Ctx *ctx, NameSpace *ns);
 knh_class_t konoha_getcid(Ctx *ctx, knh_bytes_t lname);
-knh_class_t konoha_findcid(Ctx *ctx, knh_bytes_t lname);
 void knh_NameSpace_setcid(Ctx *ctx, NameSpace *o, String *name, knh_class_t c);
 void knh_NameSpace_setLocalName(Ctx *ctx, NameSpace *o, knh_class_t cid);
 knh_class_t knh_NameSpace_getcid(Ctx *ctx, NameSpace *o, knh_bytes_t name);
@@ -320,12 +304,11 @@ Object * knh_NameSpace_getConstNULL(Ctx *ctx, NameSpace *ns, knh_bytes_t name);
 void knh_NameSpace_addConst(Ctx *ctx, NameSpace *ns, String *name, Object *value);
 void knh_NameSpace_setFuncClass(Ctx *ctx, NameSpace *o, knh_methodn_t mn, knh_class_t c);
 knh_class_t knh_NameSpace_getFuncClass(Ctx *ctx, NameSpace *o, knh_bytes_t funcname);
-knh_class_t knh_tclass_classURN(Ctx *ctx, knh_class_t bcid, knh_bytes_t urn);
-knh_class_t knh_tclass_loadURN(Ctx *ctx, knh_class_t bcid, knh_bytes_t urn);
-knh_type_t knh_NameSpace_tagcid(Ctx *ctx, NameSpace *o, knh_class_t bcid, knh_bytes_t tag);
+knh_type_t knh_NameSpace_tagcid(Ctx *ctx, NameSpace *o, knh_class_t cid, knh_bytes_t tag);
 /* ../src/class/knh_Number.c */
 knh_int_t knh_Number_tointeger(Any *o);
 knh_float_t knh_Number_tofloat(Any *o);
+knh_int_t knh_IntNULL_toint(Int *v);
 /* ../src/class/knh_Object.c */
 Object *new_Nue(Ctx *ctx, String *msg);
 METHOD knh__Object_new(Ctx *ctx, knh_sfp_t *sfp);
@@ -338,7 +321,7 @@ int knh_Object_cmp(Object *o1, Object *o2);
 knh_bool_t knh_Object_equals(Ctx *ctx, Object *o1, Object *o2);
 knh_bytes_t knh_Object_tobytes(Ctx *ctx, Object *o);
 /* ../src/class/knh_OutputStream.c */
-Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *mode);
+Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *m);
 void knh_OutputStream_putc(Ctx *ctx, OutputStream *o, knh_int_t ch);
 void knh_OutputStream_write(Ctx *ctx, OutputStream *o, knh_bytes_t buf);
 void knh_OutputStream_flush(Ctx *ctx, OutputStream *o);
@@ -349,29 +332,6 @@ void knh_OutputStream_indent_inc(Ctx *ctx, OutputStream *o);
 void knh_OutputStream_indent_dec(Ctx *ctx, OutputStream *o);
 void knh_OutputStream_write_indent(Ctx *ctx, OutputStream *o);
 void knh_OutputStream_print_(Ctx *ctx, OutputStream *o, knh_bytes_t str, knh_bool_t isnl);
-/* ../src/class/knh_OutputStream_write.c */
-void knh_write__s(Ctx *ctx, OutputStream *w, char *s);
-void knh_write__p(Ctx *ctx, OutputStream *w, void *ptr);
-void knh_write__ifmt(Ctx *ctx, OutputStream *w, char *fmt, knh_intptr_t n);
-void knh_write__ffmt(Ctx *ctx, OutputStream *w, char *fmt, knh_float_t n);
-void knh_write_integerfmt(Ctx *ctx, OutputStream *w, char *fmt, knh_int_t n);
-void knh_write__flag(Ctx *ctx, OutputStream *w, knh_flag_t flag);
-void knh_write_cid(Ctx *ctx, OutputStream *w, knh_class_t cid);
-void knh_write_mn(Ctx *ctx, OutputStream *w, knh_methodn_t mn);
-void knh_write_cidmn(Ctx *ctx, OutputStream *w, knh_class_t cid, knh_methodn_t mn);
-void knh_write_type(Ctx *ctx, OutputStream *w, knh_type_t type);
-void knh_write_fline(Ctx *ctx, OutputStream *w, char *file, int line);
-void konoha_setverbose(int v);
-METHOD knh_fmethod_movableText(Ctx *ctx, knh_sfp_t *sfp);
-/* ../src/class/knh_Range.c */
-Range *new_Range(Ctx *ctx, Object *start, Object *end);
-/* ../src/class/knh_Regex.c */
-Regex *new_Regex(Ctx *ctx, String *pattern);
-int knh_String_opMatch(Ctx *ctx, String *o, Regex *re);
-METHOD knh__String_opMatch(Ctx *ctx, knh_sfp_t *sfp);
-METHOD knh__Regex_opMatch(Ctx *ctx, knh_sfp_t *sfp);
-Array *knh_Regex_split(Ctx *ctx, Regex *o, String *s);
-void knh_Regex__s(Ctx *ctx, Regex *o, OutputStream *w, String *m);
 /* ../src/class/knh_ResultSet.c */
 ResultSet* new_ResultSet(Ctx *ctx);
 knh_boolean_t knh_ResultSet_next(Ctx *ctx, ResultSet *o);
@@ -401,23 +361,42 @@ knh_bool_t knh_String_equals(String *o, knh_bytes_t s);
 knh_bool_t knh_String_startsWith(String *b, knh_bytes_t s);
 knh_bool_t knh_String_endsWith(String *b, knh_bytes_t s);
 Object* new_String_parseOf(Ctx *ctx, String *p);
+int knh_bytes_splitTag(knh_bytes_t t, knh_bytes_t *tag, knh_bytes_t *body);
 Object *new_Object_parseOf(Ctx *ctx, String *s);
 /* ../src/class/knh_String_mbytes.c */
 knh_bool_t knh_bytes_checkENCODING(knh_bytes_t s);
 size_t knh_bytes_mlen(knh_bytes_t s);
 knh_bytes_t knh_bytes_mofflen(knh_bytes_t m, size_t moff, size_t mlen);
-/* ../src/class/knh_StringUnit.c */
-StringUnit *new_StringUnit(Ctx *ctx, knh_flag_t flag, knh_class_t cid, String *tag);
-String *new_StringX__FormatException(Ctx *ctx, knh_class_t cid, knh_bytes_t t, String *orign);
+knh_int_t knh_uchar_toucs4(knh_uchar_t *utf8);
+char *knh_format_utf8(char *buf, size_t bufsiz, knh_int_t ucs4);
 /* ../src/class/knh_System.c */
 char * knh_bytes_nzenvkey(knh_bytes_t t, char *buf, size_t bufsiz);
 Any* knh_System_getProperty(Ctx *ctx, System *o, knh_bytes_t key);
 void knh_System_setProperty(Ctx *ctx, System *o, String *key, Any *value);
 knh_bool_t konoha_initcheck(void);
-knh_bytes_t knh_Runtime_aliasURN(Ctx *ctx, knh_bytes_t aurn);
-void knh_Runtime_setAliasURN(Ctx *ctx, String *alias, String *urn);
-/* ../src/class/knh_Tuple2.c */
-Tuple2 *new_Tuple2(Ctx *ctx, Object *first, Object *second);
+/* ../src/class/knh_type_t.c */
+knh_class_t knh_pmztype_toclass(Ctx *ctx, knh_type_t t, knh_class_t this_cid);
+knh_type_t knh_pmztype_totype(Ctx *ctx, knh_type_t t, knh_class_t this_cid);
+knh_class_t knh_class_Array(Ctx *ctx, knh_class_t p1);
+knh_class_t knh_class_Iterator(Ctx *ctx, knh_class_t p1);
+knh_class_t knh_class_Generics(Ctx *ctx, knh_class_t bcid, knh_class_t p1, knh_class_t p2);
+char* knh_format_type(Ctx *ctx, char *buf, size_t bufsiz, knh_type_t type);
+char *TYPEQ(knh_type_t type);
+char *knh_TYPEN(Ctx *ctx, knh_type_t type);
+/* ../src/class/knh_write.c */
+void knh_write__s(Ctx *ctx, OutputStream *w, char *s);
+void knh_write__p(Ctx *ctx, OutputStream *w, void *ptr);
+void knh_write__ifmt(Ctx *ctx, OutputStream *w, char *fmt, knh_intptr_t n);
+void knh_write__ffmt(Ctx *ctx, OutputStream *w, char *fmt, knh_float_t n);
+void knh_write_integerfmt(Ctx *ctx, OutputStream *w, char *fmt, knh_int_t n);
+void knh_write__flag(Ctx *ctx, OutputStream *w, knh_flag_t flag);
+void knh_write_cid(Ctx *ctx, OutputStream *w, knh_class_t cid);
+void knh_write_mn(Ctx *ctx, OutputStream *w, knh_methodn_t mn);
+void knh_write_type(Ctx *ctx, OutputStream *w, knh_type_t type);
+void knh_write_fline(Ctx *ctx, OutputStream *w, char *file, int line);
+void knh_vprintf(Ctx *ctx, OutputStream *w, char *fmt, va_list ap);
+void konoha_setverbose(int v);
+METHOD knh_fmethod_movableText(Ctx *ctx, knh_sfp_t *sfp);
 /* ../src/compiler/asm.c */
 Asm* knh_Context_getAsm(Ctx *ctx);
 NameSpace *knh_Context_getNameSpace(Ctx *ctx);
@@ -436,6 +415,7 @@ void knh_StmtTRI_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx
 void knh_StmtLET_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx);
 void knh_StmtEXPR_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx);
 void knh_StmtPRINT_asm(Ctx *ctx, Stmt *stmt, Asm *abr);
+void knh_StmtASSERT_asm(Ctx *ctx, Stmt *stmt, Asm *abr);
 void knh_Stmt_asmBLOCK(Ctx *ctx, Stmt *stmt, Asm *abr, int isIteration);
 void KNH_ASM_METHOD(Ctx *ctx, Asm *abr, Method *mtd, Stmt *params, Stmt *body, int isIteration);
 void knh_StmtMETHOD_asm(Ctx *ctx, Stmt *stmt, Asm *abr);
@@ -457,8 +437,8 @@ void knh_Asm_genMethod(Ctx *ctx, Asm *abr, OutputStream *w);
 void knh_Asm_loadCompiledMethod(Ctx *ctx, Asm *abr);
 knh_fmethod knh_Asm_getCompiledMethod(Ctx *ctx, Asm *abr, knh_bytes_t cname, knh_bytes_t mname);
 /* ../src/compiler/konohac.c */
-int knh_Stmt_complie(Ctx *ctx, Stmt *stmt, String *nsname, int isEval);
-Stmt *knh_bytes_parseStmt(Ctx *ctx, knh_bytes_t kscript);
+int knh_Stmt_compile(Ctx *ctx, Stmt *stmt, String *nsname, int isEval);
+Stmt *knh_bytes_parseStmt(Ctx *ctx, knh_bytes_t kscript, int fileid, int line);
 void konohac_eval(Ctx *ctx, String *nsname, InputStream *in);
 void knh_Asm_openlib(Ctx *ctx, Asm *abr, knh_bytes_t fpath);
 int knh_NameSpace_loadScript(Ctx *ctx, NameSpace *ns, knh_bytes_t fpath, int isEval);
@@ -467,17 +447,12 @@ void konoha_compile(Ctx *ctx, String *nsname, knh_bytes_t fpath);
 Stmt *new_StmtINSTMT(Ctx *ctx, Token *tk);
 void knh_Stmt_add_PEACH(Ctx *ctx, Stmt *o, knh_tokens_t *tc);
 /* ../src/compiler/perror.c */
-void knh_perrata(Ctx *ctx, knh_fileid_t fileid, int line, char *oldt, char *newt);
-void knh_Token_perrata(Ctx *ctx, Token *o, char *newtoken);
-void knh_perror(Ctx *ctx, knh_fileid_t fileid, int line, int pe, char *msg);
-void knh_perror__s(Ctx *ctx, knh_fileid_t fileid, int line, int pe, char *msg);
-void knh_Token_perror(Ctx *ctx, Token *o, int pe);
-void knh_Stmt_perror(Ctx *ctx, Stmt *o, int pe, Token *tk);
-void knh_Asm_perror(Ctx *ctx, Asm *abr, int pe, char *msg);
-void knh_Asm_assert(Ctx *ctx, Asm *abr, int c);
+void knh_vperror(Ctx *ctx, knh_fileid_t fileid, int line, int pe, char *fmt, va_list ap);
+void knh_perror(Ctx *ctx, knh_fileid_t fileid, int line, int pe, char *fmt, ...);
+void knh_Token_perror(Ctx *ctx, Token *tk, int pe, char *fmt, ...);
+void knh_Asm_perror(Ctx *ctx, Asm *abr, int pe, char *fmt, ...);
 /* ../src/compiler/stmt.c */
 Stmt* new_Stmt(Ctx *ctx, knh_flag_t flag, knh_stmt_t stt);
-Stmt* new_StmtDONE(Ctx *ctx);
 void knh_Stmt_toERR(Ctx *ctx, Stmt *stmt, Term *tm);
 void knh_Stmt_add(Ctx *ctx, Stmt *o, Term *tm);
 Stmt *knh_Stmt_tail(Stmt *o);
@@ -487,6 +462,7 @@ knh_flag_t knh_StmtMETHOD_flag(Ctx *ctx, Stmt *o);
 knh_flag_t knh_Stmt_metaflag__field(Ctx *ctx, Stmt *b);
 knh_flag_t knh_StmtPRINT_flag(Ctx *ctx, Stmt *o);
 knh_bool_t knh_StmtMETA_isOverride(Ctx *ctx, Stmt *o);
+int knh_StmtMETA_is(Ctx *ctx, Stmt *stmt, knh_bytes_t name);
 void knh_Stmt__s(Ctx *ctx, Stmt *o, OutputStream *w, String *m);
 void knh_Stmt__dump(Ctx *ctx, Stmt *o, OutputStream *w, String *m);
 /* ../src/compiler/token.c */
@@ -511,17 +487,17 @@ Token *new_Token__NAME(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t t
 knh_token_t knh_char_totoken(int ch);
 void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in);
 /* ../src/compiler/typing.c */
-void knh_Asm_perrorMTD(Ctx *ctx, Asm *abr, int pe, knh_class_t cid, knh_methodn_t mn);
-void knh_Asm_perrorMPR(Ctx *ctx, Asm *abr, int pe, knh_class_t scid, knh_class_t tcid);
 Token* new_TokenCONST(Ctx *ctx, Any *fln, Any *data);
 void knh_Token_setCONST(Ctx *ctx, Token *o, Any *data);
 Token* knh_Token_toCONST(Token *o);
 Token* new_TokenNULL(Ctx *ctx, Any *fln, knh_type_t type);
 knh_index_t knh_Asm_indexOfVariable(Asm *abr, knh_fieldn_t fnq);
+int knh_ismtchar(int c);
+int knh_bytes_findMT(Ctx *ctx, knh_bytes_t text, knh_bytes_t *mt, knh_bytes_t *expr, knh_bytes_t *next);
+int TERMs_isCONST(Stmt *stmt, size_t n);
 int TERMs_isTRUE(Stmt *stmt, size_t n);
 int TERMs_isFALSE(Stmt *stmt, size_t n);
 knh_type_t TERMs_gettype(Stmt *stmt, size_t n);
-char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int n);
 void knh_Stmt_setType(Ctx *ctx, Stmt *stmt, knh_type_t type);
 Term * knh_StmtDECL_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns);
 Term *knh_StmtLET_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_type_t reqt);
@@ -538,16 +514,19 @@ void *knh_dlopen(Ctx *ctx, const char* path, int mode);
 void *knh_dlsym(Ctx *ctx, void* hdr, const char* symbol);
 const char *knh_dlerror(Ctx *ctx);
 int knh_dlclose(Ctx *ctx, void* hdr);
-/* ../src/deps/file.c */
-int knh_isfile(Ctx *ctx, knh_bytes_t path);
-int knh_isdir(Ctx *ctx, knh_bytes_t path);
-char * knh_format_homepath(char *buf, size_t bufsiz);
-/* ../src/deps/io.c */
+/* ../src/deps/fileio.c */
 knh_iodrv_t *konoha_getIODriver(Ctx *ctx, knh_bytes_t name);
 knh_iodrv_t *konoha_getDefaultIODriver();
 void  init_IO(Ctx *ctx);
 InputStream *new_InputStream__stdio(Ctx *ctx, FILE *fp, String *enc);
 OutputStream *new_OutputStream__stdio(Ctx *ctx, FILE *fp, String *enc);
+/* ../src/deps/filesystem.c */
+int knh_isfile(Ctx *ctx, knh_bytes_t path);
+int knh_isdir(Ctx *ctx, knh_bytes_t path);
+char * knh_format_homepath(char *buf, size_t bufsiz);
+/* ../src/deps/konoha_locale.c */
+char *konoha_encoding();
+char *knh_format_lang(char *buf, size_t bufsiz);
 /* ../src/deps/regex.c */
 void knh_write_USING_REGEX(Ctx *ctx, OutputStream *w);
 knh_regex_drvapi_t *knh_System_getRegexDriver(Ctx *ctx, knh_bytes_t name);
@@ -576,17 +555,23 @@ int main(int argc, char **argv);
 /* ../src/labs/b4.c */
 METHOD knh__Script_changeChannel(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Script_hook(Ctx *ctx, knh_sfp_t *sfp);
-/* ../src/main/filesystem.c */
+int package_init(Ctx *ctx);
+/* ../src/main/classapi.c */
+knh_hcode_t knh_MethodField_hachCode(Ctx *ctx, MethodField *o);
+size_t knh_fbyteconv_nop(Ctx *ctx, BytesConv *bc, knh_bytes_t t, knh_Bytes_t *ba);
+void knh_fbyteconvfree_nop(Ctx *ctx, BytesConv *bc);
+void knh_OutputStream_init(Ctx *ctx, OutputStream *out, int init);
+void knh_ExceptionHandler_traverse(Ctx *ctx, ExceptionHandler *hdr, knh_ftraverse gc);
+void knh_Context_init(Ctx *ctx, knh_Context_t *o, int init);
+void knh_Stmt_done(Ctx *ctx, Stmt *o);
+/* ../src/main/exports.c */
+/* ../src/main/fileutils.c */
 char * knh_format_parentpath(char *buf, size_t bufsiz, knh_bytes_t path, int n);
 char * knh_format_nzpath(char *buf, size_t bufsiz, knh_bytes_t path);
 char * knh_format_catpath(char *buf, size_t bufsiz, knh_bytes_t path, knh_bytes_t file);
 /* ../src/main/glue.c */
 int knh_fcallback_cmpr(Object *obj, Object *obj2);
 void *konoha_generateCallBackFunc(Ctx *ctx, void *func, Closure *c);
-/* ../src/main/knh_tClass.c */
-void KNH_ACLASS(Ctx *ctx, knh_class_t cid, knh_class_t p1);
-void KNH_ICLASS(Ctx *ctx, knh_class_t cid, knh_class_t p1);
-knh_class_t KNH_XCLASS(Ctx *ctx, knh_class_t cid, knh_class_t bcid, ClassSpec *cs);
 /* ../src/main/konoha_api.c */
 void konoha_init(void);
 int knh_readline_askYesNo(char *prompt, int def);
@@ -594,22 +579,10 @@ METHOD knh__Script_eval(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Script_isStatement(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Script_readLine(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Script_addHistory(Ctx *ctx, knh_sfp_t *sfp);
-/* ../src/main/konoha_data.c */
-knh_hcode_t knh_MethodField_hachCode(Ctx *ctx, MethodField *o);
-size_t f_bconv__NOP(Ctx *ctx, BytesConv *bc, knh_bytes_t t, knh_Bytes_t *ba);
-void knh_OutputStream_init(Ctx *ctx, OutputStream *out, int init);
-void knh_ExceptionHandler_traverse(Ctx *ctx, ExceptionHandler *hdr, knh_ftraverse gc);
-void knh_Context_init(Ctx *ctx, knh_Context_t *o, int init);
-void knh_Stmt_done(Ctx *ctx, Stmt *o);
 /* ../src/main/konoha_ext.c */
 void knh_srand(knh_uint_t seed);
-METHOD knh__System_setRandomSeed(Ctx *ctx, knh_sfp_t *sfp);
 knh_uint_t knh_rand();
 knh_float_t knh_float_rand();
-/* ../src/main/konoha_libknh.c */
-/* ../src/main/konoha_locale.c */
-char *konoha_encoding();
-char *knh_format_lang(char *buf, size_t bufsiz);
 /* ../src/main/logging.c */
 void knh_stack_pmsg(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, String *s);
 void knh_stack_p(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, knh_methodn_t mn, int sfpidx);
@@ -635,7 +608,7 @@ knh_ftraverse konoha_getDefaultSweepFunc();
 /* ../src/main/stack.c */
 int knh_sfp_argc(Ctx *ctx, knh_sfp_t *v);
 METHOD knh__System_stackdump(Ctx *ctx, knh_sfp_t *sfp);
-char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int n); void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, knh_code_t *pc);
+void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, knh_code_t *pc);
 /* ../src/main/system.c */
 int konoha_addClassConst(Ctx *ctx, knh_class_t cid, String* name, Object *value);
 Object *konoha_getClassConstNULL(Ctx *ctx, knh_class_t cid, knh_bytes_t name);
@@ -647,7 +620,7 @@ char *knh_format_newFMT(char *buf, size_t bufsiz, knh_bytes_t t, int dot, char *
 String *konoha_getFieldName(Ctx *ctx, knh_fieldn_t fn);
 knh_fieldn_t knh_tName_get_fnq(Ctx *ctx, knh_bytes_t tname, knh_fieldn_t def);
 String *new_String__mn(Ctx *ctx, knh_methodn_t mn);
-knh_methodn_t knh_tName_getMethodn(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def);
+knh_methodn_t konoha_getMethodName(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def);
 char *knh_format_methodn(Ctx *ctx, char *buf, size_t bufsiz, knh_methodn_t mn);
 char * knh_format_cmethodn(Ctx *ctx, char *buf, size_t bufsiz, knh_class_t cid, knh_methodn_t mn);
 knh_fileid_t konoha_getFileId(Ctx *ctx, knh_bytes_t t);
@@ -656,7 +629,6 @@ void konoha_addDriverAPI(Ctx *ctx, char *alias, knh_drvapi_t* p);
 knh_drvapi_t *konoha_getDriverAPI(Ctx *ctx, int type, knh_bytes_t name);
 NameSpace *knh_System_loadPackage(Ctx *ctx, knh_bytes_t pkgname);
 NameSpace *knh_System_getNameSpace(Ctx *ctx, knh_bytes_t name);
-ClassSpec *konoha_getClassSpec(Ctx *ctx, knh_class_t bcid, knh_bytes_t urn);
 /* ../src/main/systemtable.c */
 Ctx *new_Context(Ctx *parent);
 void knh_Context_traverse(Ctx *ctx, knh_Context_t *o, knh_ftraverse ftr);
