@@ -97,6 +97,7 @@ knh_code_t* KNH_ASM_MAPnc_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_class_t a2);
 knh_code_t* KNH_ASM_AMAP_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_class_t a2);
 knh_code_t* KNH_ASM_NNMAP_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_class_t a2);
 knh_code_t* KNH_ASM_JMP_(Ctx *ctx, Asm *o,knh_labelid_t a1);
+knh_code_t* KNH_ASM_SKIP_(Ctx *ctx, Asm *o,knh_labelid_t a1);
 knh_code_t* KNH_ASM_bJIFT_(Ctx *ctx, Asm *o,knh_labelid_t a1,knh_sfi_t a2);
 knh_code_t* KNH_ASM_bJIFF_(Ctx *ctx, Asm *o,knh_labelid_t a1,knh_sfi_t a2);
 knh_code_t* KNH_ASM_bJIFF_LOOP_(Ctx *ctx, Asm *o,knh_labelid_t a1,knh_sfi_t a2);
@@ -163,6 +164,18 @@ knh_code_t* KNH_ASM_fGT_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_sfi_t a3
 knh_code_t* KNH_ASM_fGTn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_float_t a3);
 knh_code_t* KNH_ASM_fGTE_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_sfi_t a3);
 knh_code_t* KNH_ASM_fGTEn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_float_t a3);
+knh_code_t* KNH_ASM_AGET_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_sfi_t a3);
+knh_code_t* KNH_ASM_AGETn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_intptr_t a3);
+knh_code_t* KNH_ASM_IAGET_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_sfi_t a3);
+knh_code_t* KNH_ASM_IAGETn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_intptr_t a3);
+knh_code_t* KNH_ASM_FAGET_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_sfi_t a3);
+knh_code_t* KNH_ASM_FAGETn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_intptr_t a3);
+knh_code_t* KNH_ASM_ASET_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_sfi_t a3);
+knh_code_t* KNH_ASM_ASETn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_intptr_t a3);
+knh_code_t* KNH_ASM_IASET_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_sfi_t a3);
+knh_code_t* KNH_ASM_IASETn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_intptr_t a3);
+knh_code_t* KNH_ASM_FASET_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_sfi_t a3);
+knh_code_t* KNH_ASM_FASETn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2,knh_intptr_t a3);
 knh_code_t* KNH_ASM_INITCODE_(Ctx *ctx, Asm *o,knh_sfi_t a1);
 knh_code_t* KNH_ASM_NOP_(Ctx *ctx, Asm *o);
 void knh_Asm_remapAddress(Ctx *ctx, Asm *o);
@@ -971,7 +984,25 @@ typedef struct {
 
 #define OPSIZE_JMP  (sizeof(klr_jmp_t)-sizeof(void*))
 
-#define OPCODE_BJIFT ((knh_opcode_t)61)
+#define OPCODE_SKIP ((knh_opcode_t)61)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+#ifdef KNH_USING_THREADEDCODE
+	void *jumpaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	union {
+		knh_labelid_t a1;  /* labelid */
+		knh_code_t *jumppc;
+	};
+	void *nextaddr;
+} klr_skip_t;
+
+#define OPSIZE_SKIP  (sizeof(klr_skip_t)-sizeof(void*))
+
+#define OPCODE_BJIFT ((knh_opcode_t)62)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -990,7 +1021,7 @@ typedef struct {
 
 #define OPSIZE_BJIFT  (sizeof(klr_bjift_t)-sizeof(void*))
 
-#define OPCODE_BJIFF ((knh_opcode_t)62)
+#define OPCODE_BJIFF ((knh_opcode_t)63)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1009,7 +1040,7 @@ typedef struct {
 
 #define OPSIZE_BJIFF  (sizeof(klr_bjiff_t)-sizeof(void*))
 
-#define OPCODE_BJIFF_LOOP ((knh_opcode_t)63)
+#define OPCODE_BJIFF_LOOP ((knh_opcode_t)64)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1028,7 +1059,7 @@ typedef struct {
 
 #define OPSIZE_BJIFF_LOOP  (sizeof(klr_bjiff_loop_t)-sizeof(void*))
 
-#define OPCODE_JIFNUL ((knh_opcode_t)64)
+#define OPCODE_JIFNUL ((knh_opcode_t)65)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1047,7 +1078,7 @@ typedef struct {
 
 #define OPSIZE_JIFNUL  (sizeof(klr_jifnul_t)-sizeof(void*))
 
-#define OPCODE_JIFNN ((knh_opcode_t)65)
+#define OPCODE_JIFNN ((knh_opcode_t)66)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1066,7 +1097,7 @@ typedef struct {
 
 #define OPSIZE_JIFNN  (sizeof(klr_jifnn_t)-sizeof(void*))
 
-#define OPCODE_NEXT ((knh_opcode_t)66)
+#define OPCODE_NEXT ((knh_opcode_t)67)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1087,7 +1118,7 @@ typedef struct {
 
 #define OPSIZE_NEXT  (sizeof(klr_next_t)-sizeof(void*))
 
-#define OPCODE_INEXT ((knh_opcode_t)67)
+#define OPCODE_INEXT ((knh_opcode_t)68)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1109,7 +1140,7 @@ typedef struct {
 
 #define OPSIZE_INEXT  (sizeof(klr_inext_t)-sizeof(void*))
 
-#define OPCODE_MAPNEXT ((knh_opcode_t)68)
+#define OPCODE_MAPNEXT ((knh_opcode_t)69)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1131,7 +1162,7 @@ typedef struct {
 
 #define OPSIZE_MAPNEXT  (sizeof(klr_mapnext_t)-sizeof(void*))
 
-#define OPCODE_SMAPNEXT ((knh_opcode_t)69)
+#define OPCODE_SMAPNEXT ((knh_opcode_t)70)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1153,7 +1184,7 @@ typedef struct {
 
 #define OPSIZE_SMAPNEXT  (sizeof(klr_smapnext_t)-sizeof(void*))
 
-#define OPCODE_TRY ((knh_opcode_t)70)
+#define OPCODE_TRY ((knh_opcode_t)71)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1172,7 +1203,7 @@ typedef struct {
 
 #define OPSIZE_TRY  (sizeof(klr_try_t)-sizeof(void*))
 
-#define OPCODE_TRYEND ((knh_opcode_t)71)
+#define OPCODE_TRYEND ((knh_opcode_t)72)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1184,7 +1215,7 @@ typedef struct {
 
 #define OPSIZE_TRYEND  (sizeof(klr_tryend_t)-sizeof(void*))
 
-#define OPCODE_CATCH ((knh_opcode_t)72)
+#define OPCODE_CATCH ((knh_opcode_t)73)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1205,7 +1236,7 @@ typedef struct {
 
 #define OPSIZE_CATCH  (sizeof(klr_catch_t)-sizeof(void*))
 
-#define OPCODE_THROW ((knh_opcode_t)73)
+#define OPCODE_THROW ((knh_opcode_t)74)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1217,7 +1248,7 @@ typedef struct {
 
 #define OPSIZE_THROW  (sizeof(klr_throw_t)-sizeof(void*))
 
-#define OPCODE_THROWS ((knh_opcode_t)74)
+#define OPCODE_THROWS ((knh_opcode_t)75)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1229,7 +1260,7 @@ typedef struct {
 
 #define OPSIZE_THROWS  (sizeof(klr_throws_t)-sizeof(void*))
 
-#define OPCODE_THROW_AGAIN ((knh_opcode_t)75)
+#define OPCODE_THROW_AGAIN ((knh_opcode_t)76)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1241,7 +1272,7 @@ typedef struct {
 
 #define OPSIZE_THROW_AGAIN  (sizeof(klr_throw_again_t)-sizeof(void*))
 
-#define OPCODE_P ((knh_opcode_t)76)
+#define OPCODE_P ((knh_opcode_t)77)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1255,7 +1286,7 @@ typedef struct {
 
 #define OPSIZE_P  (sizeof(klr_p_t)-sizeof(void*))
 
-#define OPCODE_PMSG ((knh_opcode_t)77)
+#define OPCODE_PMSG ((knh_opcode_t)78)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1268,7 +1299,7 @@ typedef struct {
 
 #define OPSIZE_PMSG  (sizeof(klr_pmsg_t)-sizeof(void*))
 
-#define OPCODE_ICAST ((knh_opcode_t)78)
+#define OPCODE_ICAST ((knh_opcode_t)79)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1280,7 +1311,7 @@ typedef struct {
 
 #define OPSIZE_ICAST  (sizeof(klr_icast_t)-sizeof(void*))
 
-#define OPCODE_INCAST ((knh_opcode_t)79)
+#define OPCODE_INCAST ((knh_opcode_t)80)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1292,7 +1323,7 @@ typedef struct {
 
 #define OPSIZE_INCAST  (sizeof(klr_incast_t)-sizeof(void*))
 
-#define OPCODE_FCAST ((knh_opcode_t)80)
+#define OPCODE_FCAST ((knh_opcode_t)81)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1304,7 +1335,7 @@ typedef struct {
 
 #define OPSIZE_FCAST  (sizeof(klr_fcast_t)-sizeof(void*))
 
-#define OPCODE_FNCAST ((knh_opcode_t)81)
+#define OPCODE_FNCAST ((knh_opcode_t)82)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1316,7 +1347,7 @@ typedef struct {
 
 #define OPSIZE_FNCAST  (sizeof(klr_fncast_t)-sizeof(void*))
 
-#define OPCODE_BNOT ((knh_opcode_t)82)
+#define OPCODE_BNOT ((knh_opcode_t)83)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1329,7 +1360,7 @@ typedef struct {
 
 #define OPSIZE_BNOT  (sizeof(klr_bnot_t)-sizeof(void*))
 
-#define OPCODE_INEG ((knh_opcode_t)83)
+#define OPCODE_INEG ((knh_opcode_t)84)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1342,7 +1373,7 @@ typedef struct {
 
 #define OPSIZE_INEG  (sizeof(klr_ineg_t)-sizeof(void*))
 
-#define OPCODE_IADD ((knh_opcode_t)84)
+#define OPCODE_IADD ((knh_opcode_t)85)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1356,7 +1387,7 @@ typedef struct {
 
 #define OPSIZE_IADD  (sizeof(klr_iadd_t)-sizeof(void*))
 
-#define OPCODE_IADDN ((knh_opcode_t)85)
+#define OPCODE_IADDN ((knh_opcode_t)86)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1370,7 +1401,7 @@ typedef struct {
 
 #define OPSIZE_IADDN  (sizeof(klr_iaddn_t)-sizeof(void*))
 
-#define OPCODE_ISUB ((knh_opcode_t)86)
+#define OPCODE_ISUB ((knh_opcode_t)87)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1384,7 +1415,7 @@ typedef struct {
 
 #define OPSIZE_ISUB  (sizeof(klr_isub_t)-sizeof(void*))
 
-#define OPCODE_ISUBN ((knh_opcode_t)87)
+#define OPCODE_ISUBN ((knh_opcode_t)88)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1398,7 +1429,7 @@ typedef struct {
 
 #define OPSIZE_ISUBN  (sizeof(klr_isubn_t)-sizeof(void*))
 
-#define OPCODE_IMUL ((knh_opcode_t)88)
+#define OPCODE_IMUL ((knh_opcode_t)89)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1412,7 +1443,7 @@ typedef struct {
 
 #define OPSIZE_IMUL  (sizeof(klr_imul_t)-sizeof(void*))
 
-#define OPCODE_IMULN ((knh_opcode_t)89)
+#define OPCODE_IMULN ((knh_opcode_t)90)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1426,7 +1457,7 @@ typedef struct {
 
 #define OPSIZE_IMULN  (sizeof(klr_imuln_t)-sizeof(void*))
 
-#define OPCODE_IDIV ((knh_opcode_t)90)
+#define OPCODE_IDIV ((knh_opcode_t)91)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1440,7 +1471,7 @@ typedef struct {
 
 #define OPSIZE_IDIV  (sizeof(klr_idiv_t)-sizeof(void*))
 
-#define OPCODE_IDIVN ((knh_opcode_t)91)
+#define OPCODE_IDIVN ((knh_opcode_t)92)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1454,7 +1485,7 @@ typedef struct {
 
 #define OPSIZE_IDIVN  (sizeof(klr_idivn_t)-sizeof(void*))
 
-#define OPCODE_IMOD ((knh_opcode_t)92)
+#define OPCODE_IMOD ((knh_opcode_t)93)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1468,7 +1499,7 @@ typedef struct {
 
 #define OPSIZE_IMOD  (sizeof(klr_imod_t)-sizeof(void*))
 
-#define OPCODE_IMODN ((knh_opcode_t)93)
+#define OPCODE_IMODN ((knh_opcode_t)94)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1482,7 +1513,7 @@ typedef struct {
 
 #define OPSIZE_IMODN  (sizeof(klr_imodn_t)-sizeof(void*))
 
-#define OPCODE_IEQ ((knh_opcode_t)94)
+#define OPCODE_IEQ ((knh_opcode_t)95)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1496,7 +1527,7 @@ typedef struct {
 
 #define OPSIZE_IEQ  (sizeof(klr_ieq_t)-sizeof(void*))
 
-#define OPCODE_IEQN ((knh_opcode_t)95)
+#define OPCODE_IEQN ((knh_opcode_t)96)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1510,7 +1541,7 @@ typedef struct {
 
 #define OPSIZE_IEQN  (sizeof(klr_ieqn_t)-sizeof(void*))
 
-#define OPCODE_INEQ ((knh_opcode_t)96)
+#define OPCODE_INEQ ((knh_opcode_t)97)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1524,7 +1555,7 @@ typedef struct {
 
 #define OPSIZE_INEQ  (sizeof(klr_ineq_t)-sizeof(void*))
 
-#define OPCODE_INEQN ((knh_opcode_t)97)
+#define OPCODE_INEQN ((knh_opcode_t)98)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1538,7 +1569,7 @@ typedef struct {
 
 #define OPSIZE_INEQN  (sizeof(klr_ineqn_t)-sizeof(void*))
 
-#define OPCODE_ILT ((knh_opcode_t)98)
+#define OPCODE_ILT ((knh_opcode_t)99)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1552,7 +1583,7 @@ typedef struct {
 
 #define OPSIZE_ILT  (sizeof(klr_ilt_t)-sizeof(void*))
 
-#define OPCODE_ILTN ((knh_opcode_t)99)
+#define OPCODE_ILTN ((knh_opcode_t)100)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1566,7 +1597,7 @@ typedef struct {
 
 #define OPSIZE_ILTN  (sizeof(klr_iltn_t)-sizeof(void*))
 
-#define OPCODE_ILTE ((knh_opcode_t)100)
+#define OPCODE_ILTE ((knh_opcode_t)101)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1580,7 +1611,7 @@ typedef struct {
 
 #define OPSIZE_ILTE  (sizeof(klr_ilte_t)-sizeof(void*))
 
-#define OPCODE_ILTEN ((knh_opcode_t)101)
+#define OPCODE_ILTEN ((knh_opcode_t)102)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1594,7 +1625,7 @@ typedef struct {
 
 #define OPSIZE_ILTEN  (sizeof(klr_ilten_t)-sizeof(void*))
 
-#define OPCODE_IGT ((knh_opcode_t)102)
+#define OPCODE_IGT ((knh_opcode_t)103)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1608,7 +1639,7 @@ typedef struct {
 
 #define OPSIZE_IGT  (sizeof(klr_igt_t)-sizeof(void*))
 
-#define OPCODE_IGTN ((knh_opcode_t)103)
+#define OPCODE_IGTN ((knh_opcode_t)104)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1622,7 +1653,7 @@ typedef struct {
 
 #define OPSIZE_IGTN  (sizeof(klr_igtn_t)-sizeof(void*))
 
-#define OPCODE_IGTE ((knh_opcode_t)104)
+#define OPCODE_IGTE ((knh_opcode_t)105)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1636,7 +1667,7 @@ typedef struct {
 
 #define OPSIZE_IGTE  (sizeof(klr_igte_t)-sizeof(void*))
 
-#define OPCODE_IGTEN ((knh_opcode_t)105)
+#define OPCODE_IGTEN ((knh_opcode_t)106)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1650,7 +1681,7 @@ typedef struct {
 
 #define OPSIZE_IGTEN  (sizeof(klr_igten_t)-sizeof(void*))
 
-#define OPCODE_FNEG ((knh_opcode_t)106)
+#define OPCODE_FNEG ((knh_opcode_t)107)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1663,7 +1694,7 @@ typedef struct {
 
 #define OPSIZE_FNEG  (sizeof(klr_fneg_t)-sizeof(void*))
 
-#define OPCODE_FADD ((knh_opcode_t)107)
+#define OPCODE_FADD ((knh_opcode_t)108)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1677,7 +1708,7 @@ typedef struct {
 
 #define OPSIZE_FADD  (sizeof(klr_fadd_t)-sizeof(void*))
 
-#define OPCODE_FADDN ((knh_opcode_t)108)
+#define OPCODE_FADDN ((knh_opcode_t)109)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1691,7 +1722,7 @@ typedef struct {
 
 #define OPSIZE_FADDN  (sizeof(klr_faddn_t)-sizeof(void*))
 
-#define OPCODE_FSUB ((knh_opcode_t)109)
+#define OPCODE_FSUB ((knh_opcode_t)110)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1705,7 +1736,7 @@ typedef struct {
 
 #define OPSIZE_FSUB  (sizeof(klr_fsub_t)-sizeof(void*))
 
-#define OPCODE_FSUBN ((knh_opcode_t)110)
+#define OPCODE_FSUBN ((knh_opcode_t)111)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1719,7 +1750,7 @@ typedef struct {
 
 #define OPSIZE_FSUBN  (sizeof(klr_fsubn_t)-sizeof(void*))
 
-#define OPCODE_FMUL ((knh_opcode_t)111)
+#define OPCODE_FMUL ((knh_opcode_t)112)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1733,7 +1764,7 @@ typedef struct {
 
 #define OPSIZE_FMUL  (sizeof(klr_fmul_t)-sizeof(void*))
 
-#define OPCODE_FMULN ((knh_opcode_t)112)
+#define OPCODE_FMULN ((knh_opcode_t)113)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1747,7 +1778,7 @@ typedef struct {
 
 #define OPSIZE_FMULN  (sizeof(klr_fmuln_t)-sizeof(void*))
 
-#define OPCODE_FDIV ((knh_opcode_t)113)
+#define OPCODE_FDIV ((knh_opcode_t)114)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1761,7 +1792,7 @@ typedef struct {
 
 #define OPSIZE_FDIV  (sizeof(klr_fdiv_t)-sizeof(void*))
 
-#define OPCODE_FDIVN ((knh_opcode_t)114)
+#define OPCODE_FDIVN ((knh_opcode_t)115)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1775,7 +1806,7 @@ typedef struct {
 
 #define OPSIZE_FDIVN  (sizeof(klr_fdivn_t)-sizeof(void*))
 
-#define OPCODE_FEQ ((knh_opcode_t)115)
+#define OPCODE_FEQ ((knh_opcode_t)116)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1789,7 +1820,7 @@ typedef struct {
 
 #define OPSIZE_FEQ  (sizeof(klr_feq_t)-sizeof(void*))
 
-#define OPCODE_FEQN ((knh_opcode_t)116)
+#define OPCODE_FEQN ((knh_opcode_t)117)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1803,7 +1834,7 @@ typedef struct {
 
 #define OPSIZE_FEQN  (sizeof(klr_feqn_t)-sizeof(void*))
 
-#define OPCODE_FNEQ ((knh_opcode_t)117)
+#define OPCODE_FNEQ ((knh_opcode_t)118)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1817,7 +1848,7 @@ typedef struct {
 
 #define OPSIZE_FNEQ  (sizeof(klr_fneq_t)-sizeof(void*))
 
-#define OPCODE_FNEQN ((knh_opcode_t)118)
+#define OPCODE_FNEQN ((knh_opcode_t)119)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1831,7 +1862,7 @@ typedef struct {
 
 #define OPSIZE_FNEQN  (sizeof(klr_fneqn_t)-sizeof(void*))
 
-#define OPCODE_FLT ((knh_opcode_t)119)
+#define OPCODE_FLT ((knh_opcode_t)120)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1845,7 +1876,7 @@ typedef struct {
 
 #define OPSIZE_FLT  (sizeof(klr_flt_t)-sizeof(void*))
 
-#define OPCODE_FLTN ((knh_opcode_t)120)
+#define OPCODE_FLTN ((knh_opcode_t)121)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1859,7 +1890,7 @@ typedef struct {
 
 #define OPSIZE_FLTN  (sizeof(klr_fltn_t)-sizeof(void*))
 
-#define OPCODE_FLTE ((knh_opcode_t)121)
+#define OPCODE_FLTE ((knh_opcode_t)122)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1873,7 +1904,7 @@ typedef struct {
 
 #define OPSIZE_FLTE  (sizeof(klr_flte_t)-sizeof(void*))
 
-#define OPCODE_FLTEN ((knh_opcode_t)122)
+#define OPCODE_FLTEN ((knh_opcode_t)123)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1887,7 +1918,7 @@ typedef struct {
 
 #define OPSIZE_FLTEN  (sizeof(klr_flten_t)-sizeof(void*))
 
-#define OPCODE_FGT ((knh_opcode_t)123)
+#define OPCODE_FGT ((knh_opcode_t)124)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1901,7 +1932,7 @@ typedef struct {
 
 #define OPSIZE_FGT  (sizeof(klr_fgt_t)-sizeof(void*))
 
-#define OPCODE_FGTN ((knh_opcode_t)124)
+#define OPCODE_FGTN ((knh_opcode_t)125)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1915,7 +1946,7 @@ typedef struct {
 
 #define OPSIZE_FGTN  (sizeof(klr_fgtn_t)-sizeof(void*))
 
-#define OPCODE_FGTE ((knh_opcode_t)125)
+#define OPCODE_FGTE ((knh_opcode_t)126)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1929,7 +1960,7 @@ typedef struct {
 
 #define OPSIZE_FGTE  (sizeof(klr_fgte_t)-sizeof(void*))
 
-#define OPCODE_FGTEN ((knh_opcode_t)126)
+#define OPCODE_FGTEN ((knh_opcode_t)127)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1943,7 +1974,175 @@ typedef struct {
 
 #define OPSIZE_FGTEN  (sizeof(klr_fgten_t)-sizeof(void*))
 
-#define OPCODE_INITCODE ((knh_opcode_t)127)
+#define OPCODE_AGET ((knh_opcode_t)128)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_sfi_t a3;
+	void *nextaddr;
+} klr_aget_t;
+
+#define OPSIZE_AGET  (sizeof(klr_aget_t)-sizeof(void*))
+
+#define OPCODE_AGETN ((knh_opcode_t)129)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_intptr_t a3;
+	void *nextaddr;
+} klr_agetn_t;
+
+#define OPSIZE_AGETN  (sizeof(klr_agetn_t)-sizeof(void*))
+
+#define OPCODE_IAGET ((knh_opcode_t)130)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_sfi_t a3;
+	void *nextaddr;
+} klr_iaget_t;
+
+#define OPSIZE_IAGET  (sizeof(klr_iaget_t)-sizeof(void*))
+
+#define OPCODE_IAGETN ((knh_opcode_t)131)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_intptr_t a3;
+	void *nextaddr;
+} klr_iagetn_t;
+
+#define OPSIZE_IAGETN  (sizeof(klr_iagetn_t)-sizeof(void*))
+
+#define OPCODE_FAGET ((knh_opcode_t)132)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_sfi_t a3;
+	void *nextaddr;
+} klr_faget_t;
+
+#define OPSIZE_FAGET  (sizeof(klr_faget_t)-sizeof(void*))
+
+#define OPCODE_FAGETN ((knh_opcode_t)133)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_intptr_t a3;
+	void *nextaddr;
+} klr_fagetn_t;
+
+#define OPSIZE_FAGETN  (sizeof(klr_fagetn_t)-sizeof(void*))
+
+#define OPCODE_ASET ((knh_opcode_t)134)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_sfi_t a3;
+	void *nextaddr;
+} klr_aset_t;
+
+#define OPSIZE_ASET  (sizeof(klr_aset_t)-sizeof(void*))
+
+#define OPCODE_ASETN ((knh_opcode_t)135)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_intptr_t a3;
+	void *nextaddr;
+} klr_asetn_t;
+
+#define OPSIZE_ASETN  (sizeof(klr_asetn_t)-sizeof(void*))
+
+#define OPCODE_IASET ((knh_opcode_t)136)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_sfi_t a3;
+	void *nextaddr;
+} klr_iaset_t;
+
+#define OPSIZE_IASET  (sizeof(klr_iaset_t)-sizeof(void*))
+
+#define OPCODE_IASETN ((knh_opcode_t)137)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_intptr_t a3;
+	void *nextaddr;
+} klr_iasetn_t;
+
+#define OPSIZE_IASETN  (sizeof(klr_iasetn_t)-sizeof(void*))
+
+#define OPCODE_FASET ((knh_opcode_t)138)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_sfi_t a3;
+	void *nextaddr;
+} klr_faset_t;
+
+#define OPSIZE_FASET  (sizeof(klr_faset_t)-sizeof(void*))
+
+#define OPCODE_FASETN ((knh_opcode_t)139)
+typedef struct {
+#ifdef KNH_USING_THREADEDCODE
+	void *codeaddr;
+#endif/*KNH_USING_THREADEDCODE*/
+	knh_opcode_t opcode;
+	knh_sfi_t a1;
+	knh_sfi_t a2;
+	knh_intptr_t a3;
+	void *nextaddr;
+} klr_fasetn_t;
+
+#define OPSIZE_FASETN  (sizeof(klr_fasetn_t)-sizeof(void*))
+
+#define OPCODE_INITCODE ((knh_opcode_t)140)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -1955,7 +2154,7 @@ typedef struct {
 
 #define OPSIZE_INITCODE  (sizeof(klr_initcode_t)-sizeof(void*))
 
-#define OPCODE_NOP ((knh_opcode_t)128)
+#define OPCODE_NOP ((knh_opcode_t)141)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
