@@ -57,6 +57,7 @@ static knh_StringData_t StringData[] = {
 	{"/dev/stdout", TSN_DEVSTDOUT}, 
 	{"@label", TSN_ATlabel}, 
 	{"Security!!", TSN_SecurityException}, 
+	{"Assertion!!", TSN_AssertionException}, 
 	{"(", TSN_LP}, 
 	{")", TSN_RP}, 
 	{"{", TSN_LB}, 
@@ -429,7 +430,7 @@ static knh_StructData_t StructData[] = {
 		knh_Regex_getkey
 	},
 	{
-		"BytesConv", STRUCT_BytesConv, FLAG_BytesConv, sizeof(knh_BytesConv_struct),
+		"BytesConv", STRUCT_BytesConv, FLAG_BytesConv, 0,
 		(knh_fstruct_init)knh_BytesConv_init,
 		(knh_fstruct_copy)knh_BytesConv_copy, 
 		(knh_fstruct_traverse)knh_BytesConv_traverse,
@@ -602,7 +603,7 @@ static knh_StructData_t StructData[] = {
 };
 
 static knh_ClassData_t ClassData[] = {
-	{"konoha.Object", CLASS_Object, CFLAG_Object, STRUCT_Object, CLASS_Object, 0, 33, 0, knh_Object_fdefault},
+	{"konoha.Object", CLASS_Object, CFLAG_Object, STRUCT_Object, CLASS_Object, 0, 34, 0, knh_Object_fdefault},
 	{"konoha.Any", CLASS_Any, CFLAG_Any, STRUCT_Any, CLASS_Object, 0, 0, 0, knh_Any_fdefault},
 	{"konoha.This", CLASS_This, CFLAG_This, STRUCT_This, CLASS_Any, 0, 0, 0, knh_This_fdefault},
 	{"konoha.Any0", CLASS_Any0, CFLAG_Any0, STRUCT_Any0, CLASS_Any, 0, 0, 0, knh_Any0_fdefault},
@@ -612,13 +613,13 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.Nue", CLASS_Nue, CFLAG_Nue, STRUCT_Nue, CLASS_Object, 0, 1, 0, knh_Nue_fdefault},
 	{"konoha.Boolean", CLASS_Boolean, CFLAG_Boolean, STRUCT_Boolean, CLASS_Object, 0, 3, 0, knh_Boolean_fdefault},
 	{"konoha.Number", CLASS_Number, CFLAG_Number, STRUCT_Number, CLASS_Object, 0, 0, 0, knh_Number_fdefault},
-	{"konoha.Int", CLASS_Int, CFLAG_Int, STRUCT_Int, CLASS_Number, 0, 30, 0, knh_Int_fdefault},
+	{"konoha.Int", CLASS_Int, CFLAG_Int, STRUCT_Int, CLASS_Number, 0, 31, 0, knh_Int_fdefault},
 	{"konoha.IntX", CLASS_IntX, CFLAG_IntX, STRUCT_IntX, CLASS_Number, 0, 0, 0, knh_IntX_fdefault},
-	{"konoha.Float", CLASS_Float, CFLAG_Float, STRUCT_Float, CLASS_Number, 0, 20, 0, knh_Float_fdefault},
+	{"konoha.Float", CLASS_Float, CFLAG_Float, STRUCT_Float, CLASS_Number, 0, 22, 0, knh_Float_fdefault},
 	{"konoha.FloatX", CLASS_FloatX, CFLAG_FloatX, STRUCT_FloatX, CLASS_Number, 0, 0, 0, knh_FloatX_fdefault},
-	{"konoha.String", CLASS_String, CFLAG_String, STRUCT_String, CLASS_Object, 0, 24, 0, knh_String_fdefault},
+	{"konoha.String", CLASS_String, CFLAG_String, STRUCT_String, CLASS_Object, 0, 36, 0, knh_String_fdefault},
 	{"konoha.StringX", CLASS_StringX, CFLAG_StringX, STRUCT_StringX, CLASS_String, 0, 0, 0, knh_StringX_fdefault},
-	{"konoha.Bytes", CLASS_Bytes, CFLAG_Bytes, STRUCT_Bytes, CLASS_Object, 0, 9, 0, knh_Bytes_fdefault},
+	{"konoha.Bytes", CLASS_Bytes, CFLAG_Bytes, STRUCT_Bytes, CLASS_Object, 0, 10, 0, knh_Bytes_fdefault},
 	{"konoha.Tuple2", CLASS_Tuple2, CFLAG_Tuple2, STRUCT_Tuple2, CLASS_Object, 0, 0, 0, knh_Tuple2_fdefault},
 	{"konoha.Range", CLASS_Range, CFLAG_Range, STRUCT_Range, CLASS_Object, 0, 0, 0, knh_Range_fdefault},
 	{"konoha.Array", CLASS_Array, CFLAG_Array, STRUCT_Array, CLASS_Object, 0, 31, 0, knh_Array_fdefault},
@@ -640,11 +641,11 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.AffineConv", CLASS_AffineConv, CFLAG_AffineConv, STRUCT_AffineConv, CLASS_Object, 0, 0, 0, knh_AffineConv_fdefault},
 	{"konoha.IntUnit", CLASS_IntUnit, CFLAG_IntUnit, STRUCT_IntUnit, CLASS_Object, 0, 1, 0, knh_IntUnit_fdefault},
 	{"konoha.FloatUnit", CLASS_FloatUnit, CFLAG_FloatUnit, STRUCT_FloatUnit, CLASS_Object, 0, 0, 0, knh_FloatUnit_fdefault},
-	{"konoha.Regex", CLASS_Regex, CFLAG_Regex, STRUCT_Regex, CLASS_Object, 0, 2, 0, knh_Regex_fdefault},
-	{"konoha.BytesConv", CLASS_BytesConv, CFLAG_BytesConv, STRUCT_BytesConv, CLASS_Object, 0, 1, 0, knh_BytesConv_fdefault},
+	{"konoha.Regex", CLASS_Regex, CFLAG_Regex, STRUCT_Regex, CLASS_Object, 0, 3, 0, knh_Regex_fdefault},
+	{"konoha.BytesConv", CLASS_BytesConv, CFLAG_BytesConv, STRUCT_BytesConv, CLASS_Object, 0, 0, 0, knh_BytesConv_fdefault},
 	{"konoha.StringUnit", CLASS_StringUnit, CFLAG_StringUnit, STRUCT_StringUnit, CLASS_Object, 0, 0, 0, knh_StringUnit_fdefault},
-	{"konoha.InputStream", CLASS_InputStream, CFLAG_InputStream, STRUCT_InputStream, CLASS_Object, 0, 6, 0, knh_InputStream_fdefault},
-	{"konoha.OutputStream", CLASS_OutputStream, CFLAG_OutputStream, STRUCT_OutputStream, CLASS_Object, 0, 16, 0, knh_OutputStream_fdefault},
+	{"konoha.InputStream", CLASS_InputStream, CFLAG_InputStream, STRUCT_InputStream, CLASS_Object, 0, 9, 0, knh_InputStream_fdefault},
+	{"konoha.OutputStream", CLASS_OutputStream, CFLAG_OutputStream, STRUCT_OutputStream, CLASS_Object, 0, 17, 0, knh_OutputStream_fdefault},
 	{"konoha.Socket", CLASS_Socket, CFLAG_Socket, STRUCT_Socket, CLASS_Object, 0, 5, 0, knh_Socket_fdefault},
 	{"konoha.Connection", CLASS_Connection, CFLAG_Connection, STRUCT_Connection, CLASS_Object, 0, 4, 0, knh_Connection_fdefault},
 	{"konoha.ResultSet", CLASS_ResultSet, CFLAG_ResultSet, STRUCT_ResultSet, CLASS_Object, 0, 9, 0, knh_ResultSet_fdefault},
@@ -652,7 +653,7 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.ExceptionHandler", CLASS_ExceptionHandler, CFLAG_ExceptionHandler, STRUCT_ExceptionHandler, CLASS_Object, 0, 0, 0, knh_ExceptionHandler_fdefault},
 	{"konoha.Script", CLASS_Script, CFLAG_Script, STRUCT_Script, CLASS_Object, 0, 8, 0, knh_Script_fdefault},
 	{"konoha.NameSpace", CLASS_NameSpace, CFLAG_NameSpace, STRUCT_NameSpace, CLASS_Object, 0, 0, 0, knh_NameSpace_fdefault},
-	{"konoha.System", CLASS_System, CFLAG_System, STRUCT_System, CLASS_Object, 0, 15, 0, knh_System_fdefault},
+	{"konoha.System", CLASS_System, CFLAG_System, STRUCT_System, CLASS_Object, 0, 16, 0, knh_System_fdefault},
 	{"konoha.Context", CLASS_Context, CFLAG_Context, STRUCT_Context, CLASS_Object, 0, 16, 0, knh_Context_fdefault},
 	{"konoha.Token", CLASS_Token, CFLAG_Token, STRUCT_Token, CLASS_Object, 0, 3, 0, knh_Token_fdefault},
 	{"konoha.Stmt", CLASS_Stmt, CFLAG_Stmt, STRUCT_Stmt, CLASS_Object, 0, 2, 0, knh_Stmt_fdefault},
@@ -704,7 +705,6 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"add", FIELDN_add},
 	{"add:fast", FIELDN_add__fast},
 	{"addHistory", FIELDN_addHistory},
-	{"addr", FIELDN_addr},
 	{"all", FIELDN_all},
 	{"autoFlush", FIELDN_autoFlush},
 	{"b", FIELDN_b},
@@ -712,10 +712,13 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"base", FIELDN_base},
 	{"bits", FIELDN_bits},
 	{"buf", FIELDN_buf},
+	{"byte", FIELDN_byte},
+	{"bytes", FIELDN_bytes},
 	{"c", FIELDN_c},
 	{"callcc", FIELDN_callcc},
 	{"ch", FIELDN_ch},
 	{"changeChannel", FIELDN_changeChannel},
+	{"char", FIELDN_char},
 	{"class", FIELDN_class},
 	{"clear", FIELDN_clear},
 	{"close", FIELDN_close},
@@ -734,8 +737,9 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"enc", FIELDN_enc},
 	{"encoding", FIELDN_encoding},
 	{"endsWith", FIELDN_endsWith},
+	{"endsWith:IgnoreCase", FIELDN_endsWith__IgnoreCase},
 	{"equals", FIELDN_equals},
-	{"equalsIgnoreCase", FIELDN_equalsIgnoreCase},
+	{"equals:IgnoreCase", FIELDN_equals__IgnoreCase},
 	{"err", FIELDN_err},
 	{"eval", FIELDN_eval},
 	{"exec", FIELDN_exec},
@@ -743,6 +747,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"f", FIELDN_f},
 	{"file", FIELDN_file},
 	{"float", FIELDN_float},
+	{"floatToIntBits", FIELDN_floatToIntBits},
 	{"flush", FIELDN_flush},
 	{"format", FIELDN_format},
 	{"formatted", FIELDN_formatted},
@@ -760,10 +765,12 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"in", FIELDN_in},
 	{"index", FIELDN_index},
 	{"indexOf", FIELDN_indexOf},
+	{"indexOf:IgnoreCase", FIELDN_indexOf__IgnoreCase},
 	{"init", FIELDN_init},
 	{"inputStream", FIELDN_inputStream},
 	{"insert", FIELDN_insert},
 	{"int", FIELDN_int},
+	{"intToFloatBits", FIELDN_intToFloatBits},
 	{"invoke", FIELDN_invoke},
 	{"isAbstract", FIELDN_isAbstract},
 	{"isAspect", FIELDN_isAspect},
@@ -779,6 +786,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"isGenerated", FIELDN_isGenerated},
 	{"isIgnoreCase", FIELDN_isIgnoreCase},
 	{"isImmutable", FIELDN_isImmutable},
+	{"isInteractive", FIELDN_isInteractive},
 	{"isInterface", FIELDN_isInterface},
 	{"isLocal", FIELDN_isLocal},
 	{"isLogging", FIELDN_isLogging},
@@ -796,27 +804,32 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"isStatement", FIELDN_isStatement},
 	{"isStatic", FIELDN_isStatic},
 	{"isStoringBuffer", FIELDN_isStoringBuffer},
+	{"isStrict", FIELDN_isStrict},
 	{"isSynchronized", FIELDN_isSynchronized},
 	{"isSynonym", FIELDN_isSynonym},
 	{"isTemporal", FIELDN_isTemporal},
 	{"isTotal", FIELDN_isTotal},
+	{"isTrusted", FIELDN_isTrusted},
 	{"isUnsigned", FIELDN_isUnsigned},
 	{"isVarArgs", FIELDN_isVarArgs},
 	{"isVerbose", FIELDN_isVerbose},
-	{"isVerbose2", FIELDN_isVerbose2},
 	{"isVirtual", FIELDN_isVirtual},
+	{"istrim", FIELDN_istrim},
 	{"j", FIELDN_j},
 	{"k", FIELDN_k},
 	{"key", FIELDN_key},
 	{"l", FIELDN_l},
 	{"lambda", FIELDN_lambda},
 	{"lastIndexOf", FIELDN_lastIndexOf},
+	{"lastIndexOf:IgnoreCase", FIELDN_lastIndexOf__IgnoreCase},
 	{"len", FIELDN_len},
+	{"likely", FIELDN_likely},
 	{"linenum", FIELDN_linenum},
 	{"listProperties", FIELDN_listProperties},
 	{"m", FIELDN_m},
 	{"main", FIELDN_main},
 	{"man", FIELDN_man},
+	{"match", FIELDN_match},
 	{"mode", FIELDN_mode},
 	{"modified", FIELDN_modified},
 	{"msg", FIELDN_msg},
@@ -832,10 +845,10 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"o", FIELDN_o},
 	{"offset", FIELDN_offset},
 	{"opAdd", FIELDN_opAdd},
+	{"opAddr", FIELDN_opAddr},
 	{"opDiv", FIELDN_opDiv},
 	{"opEq", FIELDN_opEq},
 	{"opFill", FIELDN_opFill},
-	{"opFmt", FIELDN_opFmt},
 	{"opGt", FIELDN_opGt},
 	{"opGte", FIELDN_opGte},
 	{"opHas", FIELDN_opHas},
@@ -864,10 +877,12 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"opSubsete", FIELDN_opSubsete},
 	{"opTo", FIELDN_opTo},
 	{"opXor", FIELDN_opXor},
+	{"option", FIELDN_option},
 	{"out", FIELDN_out},
 	{"outputStream", FIELDN_outputStream},
 	{"p", FIELDN_p},
 	{"path", FIELDN_path},
+	{"pattern", FIELDN_pattern},
 	{"pop", FIELDN_pop},
 	{"port", FIELDN_port},
 	{"print", FIELDN_print},
@@ -875,30 +890,36 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"proceed", FIELDN_proceed},
 	{"prompt", FIELDN_prompt},
 	{"property", FIELDN_property},
+	{"putChar", FIELDN_putChar},
 	{"putc", FIELDN_putc},
 	{"q", FIELDN_q},
 	{"query", FIELDN_query},
 	{"r", FIELDN_r},
 	{"random", FIELDN_random},
+	{"randomSeed", FIELDN_randomSeed},
 	{"re", FIELDN_re},
+	{"read", FIELDN_read},
 	{"readLine", FIELDN_readLine},
 	{"register", FIELDN_register},
-	{"release", FIELDN_release},
 	{"remove", FIELDN_remove},
 	{"replace", FIELDN_replace},
 	{"reverse", FIELDN_reverse},
 	{"s", FIELDN_s},
 	{"script", FIELDN_script},
+	{"seed", FIELDN_seed},
 	{"shuffle", FIELDN_shuffle},
 	{"size", FIELDN_size},
 	{"sort", FIELDN_sort},
 	{"split", FIELDN_split},
 	{"stackdump", FIELDN_stackdump},
 	{"startsWith", FIELDN_startsWith},
+	{"startsWith:IgnoreCase", FIELDN_startsWith__IgnoreCase},
 	{"status", FIELDN_status},
 	{"stdErr", FIELDN_stdErr},
 	{"storingBuffer", FIELDN_storingBuffer},
+	{"strict", FIELDN_strict},
 	{"string", FIELDN_string},
+	{"substring", FIELDN_substring},
 	{"super", FIELDN_super},
 	{"swap", FIELDN_swap},
 	{"t", FIELDN_t},
@@ -909,12 +930,12 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"trim", FIELDN_trim},
 	{"typeof", FIELDN_typeof},
 	{"u", FIELDN_u},
+	{"unlikely", FIELDN_unlikely},
 	{"urn", FIELDN_urn},
 	{"v", FIELDN_v},
 	{"value", FIELDN_value},
 	{"vargs", FIELDN_vargs},
 	{"verbose", FIELDN_verbose},
-	{"verbose2", FIELDN_verbose2},
 	{"w", FIELDN_w},
 	{"write", FIELDN_write},
 	{"x", FIELDN_x},
@@ -998,26 +1019,33 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_nFloat, 0+ 0, NNTYPE_Float},
 	{MFN_nBoolean_Float, 1+ 0, NNTYPE_Boolean, TYPE_Float, FIELDN_v},
 	{MFN_nBoolean_nFloat, 1+ 0, NNTYPE_Boolean, NNTYPE_Float, FIELDN_v},
+	{MFN_nInt_nFloat, 1+ 0, NNTYPE_Int, NNTYPE_Float, FIELDN_n},
 	{MFN_nThis_Any_, 1+ 0, NNTYPE_This, TYPE_Any, FIELDN_v},
 	{MFN_nBoolean_Any, 1+ 0, NNTYPE_Boolean, TYPE_Any, FIELDN_v},
 	{MFN_nBoolean_nClass, 1+ 0, NNTYPE_Boolean, NNTYPE_Class, FIELDN_c},
 	{MFN_nClass, 0+ 0, NNTYPE_Class},
 	{MFN_iThis, 0+ 0, NNITYPE_This},
-	{MFN_InputStream_nString, 1+ 0, TYPE_InputStream, NNTYPE_String, FIELDN_u},
+	{MFN_InputStream_nString_String, 2+ 0, TYPE_InputStream, NNTYPE_String, FIELDN_u, TYPE_String, FIELDN_m},
+	{MFN_nInt_nBytes_Int_Int, 3+ 0, NNTYPE_Int, NNTYPE_Bytes, FIELDN_b, TYPE_Int, FIELDN_o, TYPE_Int, FIELDN_l},
+	{MFN_nString, 0+ 0, NNTYPE_String},
 	{MFN_OutputStream_nString_String, 2+ 0, TYPE_OutputStream, NNTYPE_String, FIELDN_u, TYPE_String, FIELDN_m},
 	{MFN_void_Any, 1+ 0, TYPE_void, TYPE_Any, FIELDN_v},
 	{MFN_nSocket_nString_nInt, 2+ 0, NNTYPE_Socket, NNTYPE_String, FIELDN_h, NNTYPE_Int, FIELDN_p},
 	{MFN_nInputStream, 0+ 0, NNTYPE_InputStream},
 	{MFN_nOutputStream, 0+ 0, NNTYPE_OutputStream},
 	{MFN_void_String, 1+ 0, TYPE_void, TYPE_String, FIELDN_e},
+	{MFN_nString_nBytes_String, 2+ 0, NNTYPE_String, NNTYPE_Bytes, FIELDN_b, TYPE_String, FIELDN_e},
+	{MFN_nBytes_String, 1+ 0, NNTYPE_Bytes, TYPE_String, FIELDN_e},
 	{MFN_nString_Any, 1+ 0, NNTYPE_String, TYPE_Any, FIELDN_v},
-	{MFN_nString_Any_, 1+ 0, NNTYPE_String, TYPE_Any, FIELDN_v},
 	{MFN_nString_nString, 1+ 0, NNTYPE_String, NNTYPE_String, FIELDN_t},
-	{MFN_nString_nString_nString, 2+ 0, NNTYPE_String, NNTYPE_String, FIELDN_o, NNTYPE_String, FIELDN_n},
+	{MFN_nString_Any_, 1+ 0, NNTYPE_String, TYPE_Any, FIELDN_v},
 	{MFN_nString_String, 1+ 0, NNTYPE_String, TYPE_String, FIELDN_v},
+	{MFN_nString_nString_nString, 2+ 0, NNTYPE_String, NNTYPE_String, FIELDN_o, NNTYPE_String, FIELDN_n},
 	{MFN_nString_Int_Int, 2+ 0, NNTYPE_String, TYPE_Int, FIELDN_o, TYPE_Int, FIELDN_l},
-	{MFN_nString, 0+ 0, NNTYPE_String},
-	{MFN_aString_Any, 1+ 0, ATYPE_String, TYPE_Any, FIELDN_d},
+	{MFN_aString_String_Int, 2+ 0, ATYPE_String, TYPE_String, FIELDN_d, TYPE_Int, FIELDN_i},
+	{MFN_Regex_nString_String, 2+ 0, TYPE_Regex, NNTYPE_String, FIELDN_p, TYPE_String, FIELDN_o},
+	{MFN_nBoolean_nRegex, 1+ 0, NNTYPE_Boolean, NNTYPE_Regex, FIELDN_r},
+	{MFN_aString_Regex, 1+ 0, ATYPE_String, TYPE_Regex, FIELDN_p},
 	{MFN_void_InputStream, 1+ 0, TYPE_void, TYPE_InputStream, FIELDN_i},
 	{MFN_void_OutputStream, 1+ 0, TYPE_void, TYPE_OutputStream, FIELDN_o},
 	{MFN_void_Int, 1+ 0, TYPE_void, TYPE_Int, FIELDN_s},
@@ -1030,7 +1058,6 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_nException_String_String_Object, 3+ 0, NNTYPE_Exception, TYPE_String, FIELDN_e, TYPE_String, FIELDN_m, TYPE_Object, FIELDN_b},
 	{MFN_String, 0+ 0, TYPE_String},
 	{MFN_nThis, 0+ 0, NNTYPE_This},
-	{MFN_nBoolean_nRegex, 1+ 0, NNTYPE_Boolean, NNTYPE_Regex, FIELDN_r},
 	{MFN_void_String_String_Int, 3+ 0, TYPE_void, TYPE_String, FIELDN_s, TYPE_String, FIELDN_f, TYPE_Int, FIELDN_l},
 	{MFN_String_String, 1+ 0, TYPE_String, TYPE_String, FIELDN_p},
 	{MFN_Boolean_nClass, 1+ 0, TYPE_Boolean, NNTYPE_Class, FIELDN_c},
@@ -1119,6 +1146,7 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Bytes_get, 0, CLASS_Bytes, METHODN_get, 0, MFN_nInt_nInt, (void*)NULL},
 	{knh__Bytes_set, 0, CLASS_Bytes, METHODN_set, 0, MFN_void_nInt_nInt, (void*)NULL},
 	{knh__Bytes_opFill, 0, CLASS_Bytes, METHODN_opFill, 0, MFN_void_nInt, (void*)NULL},
+	{knh_fmethod_movableText, 0, CLASS_Bytes, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Bytes__k},
 	{knh_fmethod_movableText, 0, CLASS_Bytes, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Bytes__dump},
 	{knh__Connection_new, 0, CLASS_Connection, METHODN_new, 0, MFN_nConnection_nString, (void*)NULL},
 	{knh__Connection_query, 0, CLASS_Connection, METHODN_query, 0, MFN_nResultSet_nString, (void*)NULL},
@@ -1198,6 +1226,8 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Float_opNext, KNH_FLAG_MF_CONST, CLASS_Float, METHODN_opNext, 0, MFN_nFloat, (void*)NULL},
 	{knh__Float_opPrev, KNH_FLAG_MF_CONST, CLASS_Float, METHODN_opPrev, 0, MFN_nFloat, (void*)NULL},
 	{knh__Float_random, KNH_FLAG_MF_STATIC, CLASS_Float, METHODN_random, 0, MFN_nFloat, (void*)NULL},
+	{knh__Float_floatToIntBits, KNH_FLAG_MF_STATIC, CLASS_Float, METHODN_floatToIntBits, 0, MFN_nInt_nFloat, (void*)NULL},
+	{knh__Float_intToFloatBits, KNH_FLAG_MF_STATIC, CLASS_Float, METHODN_intToFloatBits, 0, MFN_nFloat_nInt, (void*)NULL},
 	{knh__Float_getSize, KNH_FLAG_MF_NULLBASE|KNH_FLAG_MF_CONST, CLASS_Float, METHODN_getSize, 0, MFN_nInt, (void*)NULL},
 	{knh__Boolean__s, 0, CLASS_Boolean, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Boolean__k, 0, CLASS_Boolean, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)NULL},
@@ -1208,12 +1238,14 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Int__f, 0, CLASS_Int, METHODN__f, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Int__x, 0, CLASS_Int, METHODN__x, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Int__bits, 0, CLASS_Int, METHODN__bits, 0, MFN_void_OutputStream_Any, (void*)NULL},
+	{knh__Int__c, 0, CLASS_Int, METHODN__c, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Float__s, 0, CLASS_Float, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Float__d, 0, CLASS_Float, METHODN__d, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Float__f, 0, CLASS_Float, METHODN__f, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Float__bits, 0, CLASS_Float, METHODN__bits, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Float__k, 0, CLASS_Float, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh__Object_new__init, KNH_FLAG_MF_VARARGS, CLASS_Object, METHODN_new__init, 0, MFN_nThis_Any_, (void*)NULL},
+	{knh__Object_opAddr, KNH_FLAG_MF_NULLBASE, CLASS_Object, METHODN_opAddr, 0, MFN_nInt, (void*)NULL},
 	{knh__Object_opEq, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_Object, METHODN_opEq, 0, MFN_nBoolean_Any, (void*)NULL},
 	{knh__Object_opNeq, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_Object, METHODN_opNeq, 0, MFN_nBoolean_Any, (void*)NULL},
 	{knh__Object_opLt, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_Object, METHODN_opLt, 0, MFN_nBoolean_Any, (void*)NULL},
@@ -1228,10 +1260,10 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Object_isNotNull, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_Object, METHODN_isNotNull, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_opItr, 0, CLASS_Object, METHODN_opItr, 0, MFN_iThis, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_Object, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Object__s},
+	{knh__Object__p, 0, CLASS_Object, METHODN__p, 0, MFN_void_OutputStream_Any, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_Object, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Object__k},
 	{knh_fmethod_movableText, 0, CLASS_Object, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Object__dump},
 	{knh_fmethod_movableText, 0, CLASS_Object, METHODN__empty, 0, MFN_void_OutputStream_Any, (void*)knh_Object__empty},
-	{knh_fmethod_movableText, 0, CLASS_Object, METHODN__addr, 0, MFN_void_OutputStream_Any, (void*)knh_Object__addr},
 	{knh_fmethod_movableText, 0, CLASS_Nue, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Nue__k},
 	{knh_fmethod_movableText, 0, CLASS_Class, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Class__s},
 	{knh_fmethod_movableText, 0, CLASS_Class, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Class__k},
@@ -1240,17 +1272,21 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_Method, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Method__s},
 	{knh_fmethod_movableText, 0, CLASS_Method, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Method__k},
 	{knh_fmethod_movableText, 0, CLASS_Method, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Method__dump},
-	{knh__InputStream_new, 0, CLASS_InputStream, METHODN_new, 0, MFN_InputStream_nString, (void*)NULL},
+	{knh__InputStream_new, 0, CLASS_InputStream, METHODN_new, 0, MFN_InputStream_nString_String, (void*)NULL},
+	{knh__InputStream_getChar, 0, CLASS_InputStream, METHODN_getChar, 0, MFN_nInt, (void*)NULL},
+	{knh__InputStream_read, 0, CLASS_InputStream, METHODN_read, 0, MFN_nInt_nBytes_Int_Int, (void*)NULL},
+	{knh__InputStream_getEncoding, 0, CLASS_InputStream, METHODN_getEncoding, 0, MFN_nString, (void*)NULL},
 	{knh__InputStream_isClosed, 0, CLASS_InputStream, METHODN_isClosed, 0, MFN_nBoolean, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_InputStream, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_InputStream__k},
 	{knh__OutputStream_new, 0, CLASS_OutputStream, METHODN_new, 0, MFN_OutputStream_nString_String, (void*)NULL},
-	{knh__OutputStream_putc, 0, CLASS_OutputStream, METHODN_putc, 0, MFN_void_nInt, (void*)NULL},
+	{knh__OutputStream_putChar, 0, CLASS_OutputStream, METHODN_putChar, 0, MFN_void_nInt, (void*)NULL},
 	{knh__OutputStream_write, 0, CLASS_OutputStream, METHODN_write, 0, MFN_void_nBytes_Int_Int, (void*)NULL},
 	{knh__OutputStream_isClosed, 0, CLASS_OutputStream, METHODN_isClosed, 0, MFN_nBoolean, (void*)NULL},
 	{knh__OutputStream_print, 0, CLASS_OutputStream, METHODN_print, 0, MFN_void_Any, (void*)NULL},
 	{knh__OutputStream_println, 0, CLASS_OutputStream, METHODN_println, 0, MFN_void_Any, (void*)NULL},
 	{knh__OutputStream_opLshift, KNH_FLAG_MF_VARARGS, CLASS_OutputStream, METHODN_opLshift, 0, MFN_void_Any_, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_OutputStream, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_OutputStream__k},
+	{knh__OutputStream_getEncoding, 0, CLASS_OutputStream, METHODN_getEncoding, 0, MFN_nString, (void*)NULL},
 	{knh__Socket_new, 0, CLASS_Socket, METHODN_new, 0, MFN_nSocket_nString_nInt, (void*)NULL},
 	{knh__Socket_getInputStream, 0, CLASS_Socket, METHODN_getInputStream, 0, MFN_nInputStream, (void*)NULL},
 	{knh__Socket_getOutputStream, 0, CLASS_Socket, METHODN_getOutputStream, 0, MFN_nOutputStream, (void*)NULL},
@@ -1258,35 +1294,52 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Socket_isClosed, 0, CLASS_Socket, METHODN_isClosed, 0, MFN_nBoolean, (void*)NULL},
 	{knh__InputStream_setEncoding, 0, CLASS_InputStream, METHODN_setEncoding, 0, MFN_void_String, (void*)NULL},
 	{knh__OutputStream_setEncoding, 0, CLASS_OutputStream, METHODN_setEncoding, 0, MFN_void_String, (void*)NULL},
+	{knh__String_new, KNH_FLAG_MF_CONST, CLASS_String, METHODN_new, 0, MFN_nString_nBytes_String, (void*)NULL},
+	{knh__String_getBytes, KNH_FLAG_MF_CONST, CLASS_String, METHODN_getBytes, 0, MFN_nBytes_String, (void*)NULL},
 	{knh__String_equals, KNH_FLAG_MF_CONST, CLASS_String, METHODN_equals, 0, MFN_nBoolean_nString, (void*)NULL},
-	{knh__String_equalsIgnoreCase, KNH_FLAG_MF_CONST, CLASS_String, METHODN_equalsIgnoreCase, 0, MFN_nBoolean_nString, (void*)NULL},
+	{knh__String_equals__IgnoreCase, KNH_FLAG_MF_CONST, CLASS_String, METHODN_equals__IgnoreCase, 0, MFN_nBoolean_nString, (void*)NULL},
 	{knh__String_startsWith, KNH_FLAG_MF_CONST, CLASS_String, METHODN_startsWith, 0, MFN_nBoolean_nString, (void*)NULL},
+	{knh__String_startsWith__IgnoreCase, KNH_FLAG_MF_CONST, CLASS_String, METHODN_startsWith__IgnoreCase, 0, MFN_nBoolean_nString, (void*)NULL},
 	{knh__String_endsWith, KNH_FLAG_MF_CONST, CLASS_String, METHODN_endsWith, 0, MFN_nBoolean_nString, (void*)NULL},
+	{knh__String_endsWith__IgnoreCase, KNH_FLAG_MF_CONST, CLASS_String, METHODN_endsWith__IgnoreCase, 0, MFN_nBoolean_nString, (void*)NULL},
 	{knh__String_indexOf, KNH_FLAG_MF_CONST, CLASS_String, METHODN_indexOf, 0, MFN_nInt_nString, (void*)NULL},
+	{knh__String_indexOf__IgnoreCase, KNH_FLAG_MF_CONST, CLASS_String, METHODN_indexOf__IgnoreCase, 0, MFN_nInt_nString, (void*)NULL},
+	{knh__String_lastIndexOf, KNH_FLAG_MF_CONST, CLASS_String, METHODN_lastIndexOf, 0, MFN_nInt_nString, (void*)NULL},
+	{knh__String_lastIndexOf__IgnoreCase, KNH_FLAG_MF_CONST, CLASS_String, METHODN_lastIndexOf__IgnoreCase, 0, MFN_nInt_nString, (void*)NULL},
 	{knh__String_getSize, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_String, METHODN_getSize, 0, MFN_nInt, (void*)NULL},
 	{knh__String_opAdd, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_String, METHODN_opAdd, 0, MFN_nString_Any, (void*)NULL},
-	{knh__String_concat, KNH_FLAG_MF_VARARGS|KNH_FLAG_MF_STATIC|KNH_FLAG_MF_NULLBASE, CLASS_String, METHODN_concat, 0, MFN_nString_Any_, (void*)NULL},
 	{knh__String_opSub, KNH_FLAG_MF_CONST, CLASS_String, METHODN_opSub, 0, MFN_nString_nString, (void*)NULL},
-	{knh__String_replace, KNH_FLAG_MF_CONST, CLASS_String, METHODN_replace, 0, MFN_nString_nString_nString, (void*)NULL},
 	{knh__String_opMul, KNH_FLAG_MF_CONST, CLASS_String, METHODN_opMul, 0, MFN_nString_nInt, (void*)NULL},
-	{knh__String_opFmt, KNH_FLAG_MF_VARARGS, CLASS_String, METHODN_opFmt, 0, MFN_nString_Any_, (void*)NULL},
+	{knh__String_concat, KNH_FLAG_MF_VARARGS|KNH_FLAG_MF_STATIC|KNH_FLAG_MF_NULLBASE, CLASS_String, METHODN_concat, 0, MFN_nString_Any_, (void*)NULL},
 	{knh__String_opDiv, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_String, METHODN_opDiv, 0, MFN_nString_String, (void*)NULL},
 	{knh__String_opMod, KNH_FLAG_MF_CONST|KNH_FLAG_MF_NULLBASE, CLASS_String, METHODN_opMod, 0, MFN_nString_String, (void*)NULL},
+	{knh__String_format, KNH_FLAG_MF_VARARGS, CLASS_String, METHODN_format, 0, MFN_nString_Any_, (void*)NULL},
+	{knh__String_replace, KNH_FLAG_MF_CONST, CLASS_String, METHODN_replace, 0, MFN_nString_nString_nString, (void*)NULL},
 	{knh__String_opHas, KNH_FLAG_MF_CONST, CLASS_String, METHODN_opHas, 0, MFN_nBoolean_nString, (void*)NULL},
 	{knh__String_get, KNH_FLAG_MF_CONST, CLASS_String, METHODN_get, 0, MFN_nString_nInt, (void*)NULL},
+	{knh__String_getChar, KNH_FLAG_MF_CONST, CLASS_String, METHODN_getChar, 0, MFN_nInt_Int, (void*)NULL},
+	{knh__String_substring, KNH_FLAG_MF_CONST, CLASS_String, METHODN_substring, 0, MFN_nString_Int_Int, (void*)NULL},
 	{knh__String_opOffset, KNH_FLAG_MF_CONST, CLASS_String, METHODN_opOffset, 0, MFN_nString_Int_Int, (void*)NULL},
+	{knh__String_opSubset, KNH_FLAG_MF_CONST, CLASS_String, METHODN_opSubset, 0, MFN_nString_Int_Int, (void*)NULL},
+	{knh__String_opSubsete, KNH_FLAG_MF_CONST, CLASS_String, METHODN_opSubsete, 0, MFN_nString_Int_Int, (void*)NULL},
 	{knh__String_toLower, KNH_FLAG_MF_CONST, CLASS_String, METHODN_toLower, 0, MFN_nString, (void*)NULL},
 	{knh__String_toUpper, KNH_FLAG_MF_CONST, CLASS_String, METHODN_toUpper, 0, MFN_nString, (void*)NULL},
 	{knh__String_trim, KNH_FLAG_MF_CONST, CLASS_String, METHODN_trim, 0, MFN_nString, (void*)NULL},
-	{knh__String_split, KNH_FLAG_MF_CONST, CLASS_String, METHODN_split, 0, MFN_aString_Any, (void*)NULL},
+	{knh__String_split, KNH_FLAG_MF_CONST, CLASS_String, METHODN_split, 0, MFN_aString_String_Int, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_String, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_String__s},
 	{knh_fmethod_movableText, 0, CLASS_String, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_String__k},
+	{knh__Regex_new, KNH_FLAG_MF_CONST, CLASS_Regex, METHODN_new, 0, MFN_Regex_nString_String, (void*)NULL},
+	{knh__String_opMatch, 0, CLASS_String, METHODN_opMatch, 0, MFN_nBoolean_nRegex, (void*)NULL},
+	{knh__Regex_opMatch, 0, CLASS_Regex, METHODN_opMatch, 0, MFN_nBoolean_nString, (void*)NULL},
+	{knh__String_match, KNH_FLAG_MF_CONST, CLASS_String, METHODN_match, 0, MFN_aString_Regex, (void*)NULL},
+	{knh_fmethod_movableText, 0, CLASS_Regex, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Regex__s},
 	{knh__System_getIn, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_getIn, 0, MFN_nInputStream, (void*)NULL},
 	{knh__System_setIn, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_setIn, 0, MFN_void_InputStream, (void*)NULL},
 	{knh__System_getOut, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_getOut, 0, MFN_nOutputStream, (void*)NULL},
 	{knh__System_setOut, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_setOut, 0, MFN_void_OutputStream, (void*)NULL},
 	{knh__System_getErr, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_getErr, 0, MFN_nOutputStream, (void*)NULL},
 	{knh__System_setErr, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_setStdErr, 0, MFN_void_OutputStream, (void*)NULL},
+	{knh__System_setRandomSeed, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_setRandomSeed, 0, MFN_void_Int, (void*)NULL},
 	{knh__System_exit, KNH_FLAG_MF_STATIC, CLASS_System, METHODN_exit, 0, MFN_void_Int, (void*)NULL},
 	{knh__System_hasDir, 0, CLASS_System, METHODN_hasDir, 0, MFN_nBoolean_nString, (void*)NULL},
 	{knh__System_hasFile, 0, CLASS_System, METHODN_hasFile, 0, MFN_nBoolean_nString, (void*)NULL},
@@ -1305,7 +1358,6 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_System, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_System__k},
 	{knh_fmethod_movableText, 0, CLASS_System, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_System__dump},
 	{knh__Context_setEncoding, 0, CLASS_Context, METHODN_setEncoding, 0, MFN_void_String, (void*)NULL},
-	{knh_fmethod_movableText, 0, CLASS_BytesConv, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_BytesConv__k},
 	{knh_fmethod_movableText, 0, CLASS_ClassMap, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_ClassMap__dump},
 	{knh__Closure_new, 0, CLASS_Closure, METHODN_new, 0, MFN_nThis_nAny_nMethod, (void*)NULL},
 	{knh__Closure_invoke, 0, CLASS_Closure, METHODN_invoke, 0, MFN_Any0_Any1_Any2_Any3, (void*)NULL},
@@ -1328,9 +1380,6 @@ static knh_MethodData_t MethodData[] = {
 	{knh__OutputStream_flush, 0, CLASS_OutputStream, METHODN_flush, 0, MFN_void, (void*)NULL},
 	{knh__OutputStream_clear, 0, CLASS_OutputStream, METHODN_clear, 0, MFN_void, (void*)NULL},
 	{knh__OutputStream_close, 0, CLASS_OutputStream, METHODN_close, 0, MFN_void, (void*)NULL},
-	{knh__String_opMatch, 0, CLASS_String, METHODN_opMatch, 0, MFN_nBoolean_nRegex, (void*)NULL},
-	{knh__Regex_opMatch, 0, CLASS_Regex, METHODN_opMatch, 0, MFN_nBoolean_nString, (void*)NULL},
-	{knh_fmethod_movableText, 0, CLASS_Regex, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Regex__s},
 	{knh__ResultSet_next, 0, CLASS_ResultSet, METHODN_next, 0, MFN_nBoolean, (void*)NULL},
 	{knh__ResultSet_close, 0, CLASS_ResultSet, METHODN_close, 0, MFN_void, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_Script, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Script__k},
@@ -1403,14 +1452,14 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Exception_isFatal, 0, CLASS_Exception, METHODN_isFatal, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Exception_isLogging, 0, CLASS_Exception, METHODN_isLogging, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Exception_isGenerated, 0, CLASS_Exception, METHODN_isGenerated, 0, MFN_nBoolean, (void*)NULL},
-	{knh__Context_isRelease, 0, CLASS_Context, METHODN_isRelease, 0, MFN_nBoolean, (void*)NULL},
+	{knh__Context_isStrict, 0, CLASS_Context, METHODN_isStrict, 0, MFN_nBoolean, (void*)NULL},
+	{knh__Context_setStrict, 0, CLASS_Context, METHODN_setStrict, 0, MFN_void_Boolean, (void*)NULL},
 	{knh__Context_isDebug, 0, CLASS_Context, METHODN_isDebug, 0, MFN_nBoolean, (void*)NULL},
-	{knh__Context_setRelease, 0, CLASS_Context, METHODN_setRelease, 0, MFN_void_Boolean, (void*)NULL},
 	{knh__Context_setDebug, 0, CLASS_Context, METHODN_setDebug, 0, MFN_void_Boolean, (void*)NULL},
 	{knh__Context_isVerbose, 0, CLASS_Context, METHODN_isVerbose, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Context_setVerbose, 0, CLASS_Context, METHODN_setVerbose, 0, MFN_void_Boolean, (void*)NULL},
-	{knh__Context_isVerbose2, 0, CLASS_Context, METHODN_isVerbose2, 0, MFN_nBoolean, (void*)NULL},
-	{knh__Context_setVerbose2, 0, CLASS_Context, METHODN_setVerbose2, 0, MFN_void_Boolean, (void*)NULL},
+	{knh__Context_isTrusted, 0, CLASS_Context, METHODN_isTrusted, 0, MFN_nBoolean, (void*)NULL},
+	{knh__Context_isInteractive, 0, CLASS_Context, METHODN_isInteractive, 0, MFN_nBoolean, (void*)NULL},
 	{NULL, 0}
 };
 
