@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -28,7 +28,17 @@
 #ifndef KONOHA_CONFIG_H_
 #define KONOHA_CONFIG_H_
 
-#include"license.h"
+/* ======================================================================== */
+/* You are aggreed to use konoha under the following License */
+
+#define KONOHA_LICENSE         "LGPL3.0"
+#define KONOHA_UNDER_LGPL3      3
+
+/* ======================================================================== */
+/* configure */
+
+//#define KNH_USING_RCGC       1
+#define KNH_USING_UNBOXFIELD 1
 
 /* ======================================================================== */
 /* Manifesto */
@@ -38,15 +48,15 @@
 #define KONOHA_VERSION   PACKAGE_VERSION
 #else
 #define KONOHA_NAME     "konoha"
-#define KONOHA_MAJOR_VERSION "0.3"
-#define KONOHA_VERSION  "0.3.10"
+#define KONOHA_MAJOR_VERSION "0.5"
+#define KONOHA_VERSION  "0.5.0"
 #endif
 
-#define KONOHA_XCODE    "(Rufy)"
+#define KONOHA_XCODE    "Toguro"
 #define KONOHA_URL		"http://konoha.sourceforge.jp/"
 #define KONOHA_URLBASE  "kttp://konoha.sourceforge.jp/wiki/"
 
-#define KONOHA_BUILDID_TOOOLD  360
+#define KONOHA_BUILDID_TOOOLD  545
 #define KNH_CHECK_VERSION(id)  ((id) <= KONOHA_BUILDID_TOOOLD)
 
 /* ======================================================================== */
@@ -69,18 +79,13 @@
 #define KONOHA_STRING_BUFSIZ               128
 #define CLASSNAME_BUFSIZ                   128
 #define FILENAME_BUFSIZ                    512
+#define ERRMSG_BUFSIZ                      512
 
 /* object specification */
 
 #define KNH_USING_I18N                     1
 #define KONOHA_ENCODING                    "UTF-8"
 
-/* ======================================================================== */
-/* [OPTIONS] */
-
-//#define KNH_USING_RCGC    1                /* Reference Counting */
-//#define KNH_USING_DRCGC   1
-
 /* ======================================================================== */
 
 #ifdef KONOHA_OS__LINUX
@@ -111,7 +116,8 @@
 	#define KONOHA_CC_VERSION "GCC " __VERSION__
 	#ifdef __i386__
 		#define KNH_CC_FASTCALL   __attribute__((fastcall))
-		#define KNH_USING_UNBOXFIELD 1
+		#define likely(x)       __builtin_expect(!!(x), 1)
+		#define unlikely(x)     __builtin_expect(!!(x), 0)
 	#else
 		#define KNH_CC_FASTCALL   /* __attribute__((fastcall)) */
 	#endif
@@ -144,7 +150,6 @@
 #ifdef _MSC_VER
 //#define KNH_DBGMODE2
 #define KNH_CC_FASTCALL  /* __fastcall */
-#define KNH_USING_UNBOXFIELD 1
 #if _MSC_VER == 1500
 	#define KONOHA_CC_VERSION "Microsoft Visual C++ 9.0 (1500)"
 #else
