@@ -20,8 +20,6 @@
 /* ======================================================================== */
 /* [DBGMODE2] */
 
-#define KNH_USING_FASTMALLOC 1
-
 #ifdef KNH_DBGMODE2
 #define KNH_DBGMODE     1
 
@@ -29,7 +27,7 @@
 
 #define DBG2_P(fmt, ...) \
 	fflush(stdout); \
-	fprintf(stderr, "DBG2[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+	fprintf(stderr, "DBG2[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 	fprintf(stderr, fmt, ## __VA_ARGS__); \
 	fprintf(stderr, "\n"); \
 
@@ -40,24 +38,19 @@
 	fprintf(stdout, "\n"); \
 
 #define TODO2(msg) \
-	fprintf(stdout, "TODO2[%s:%d/%s]: %s\n", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__, msg); \
+	fprintf(stdout, "TODO2[%s:%d/%s]: %s\n", knh_safefile(__FILE__), __LINE__, __FUNCTION__, msg); \
 
 #define DBG2_RESIZE(o, p, os, ns) \
 	DBG2_P("RESIZE %s(%p) %d => %d\n\tOLD_BLOCK(%p-%p)", STRUCTN((o->h.bcid)), o, (int)os, (int)ns, p, (p + os)); \
 
 #define DBG2_ASSERT(c) KNH_ASSERT(c);
 
-#ifdef KNH_USING_FASTMALLOC2
-#define KNH_MALLOC(ctx, size)    knh_fastmalloc(ctx, size)
-#define KNH_FREE(ctx, p, size)   knh_fastfree(ctx, p, size)
-#else
 #define KNH_MALLOC(ctx, size)    DBG2_malloc(ctx, size, (char*)__FUNCTION__)
 #define KNH_FREE(ctx, p, size)   DBG2_free(ctx, p, size, (char*)__FUNCTION__)
-#endif
 
 #define DBG2_ABORT()    abort()
 
-#else/*KNH_DBGMODE2*/
+#elif /*KNH_DBGMODE2*/ !defined(KONOHA_ON_LKM)
 
 #define DBG2_(stmt)
 #define DBG2_P(fmt, ...)
@@ -71,7 +64,7 @@
 #define 	KNH_MALLOC(ctx, size)    knh_fastmalloc(ctx, size)
 #define 	KNH_FREE(ctx, p, size)   knh_fastfree(ctx, p, size)
 #else/*KNH_USING_FASTMALLOC*/
-#define 	KNH_MALLOC(ctx, size) knh_malloc(ctx, size)
+#define 	KNH_MALLOC(ctx, size)    knh_malloc(ctx, size)
 #define 	KNH_FREE(ctx, p, size)   knh_free(ctx, p, size)
 #endif/*KNH_USING_FASTMALLOC*/
 
@@ -86,22 +79,23 @@
 
 #define DBG_P(fmt, ...) \
 	fflush(stdout); \
-	fprintf(stderr, "DBG[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+	fprintf(stderr, "DBG[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 	fprintf(stderr, fmt, ## __VA_ARGS__); \
 	fprintf(stderr, "\n"); \
 
 #define DBG_DUMP(ctx, o, opt, msg) \
 	fprintf(stdout, "DBG[%s]: %s\n", __FUNCTION__, msg); \
-	knh_format(ctx, KNH_STDOUT, METHODN__dump, o, opt); \
+	knh_format(ctx, KNH_STDOUT, METHODN__dump, UP(o), UP(opt)); \
+	knh_flush(ctx, KNH_STDOUT);\
 	fprintf(stdout, "\n"); \
 
 #define TODO() \
-	fprintf(stderr, "TODO[%s:%d/%s]:\n", KNH_SAFEFILE(__FILE__) , __LINE__, __FUNCTION__); \
+	fprintf(stderr, "TODO[%s:%d/%s]:\n", knh_safefile(__FILE__) , __LINE__, __FUNCTION__); \
 
 #define DBG_ASSERT(c, fmt, ...) \
 	if(!(c)) { \
 		fflush(stdout); \
-		fprintf(stderr, "DBG!![%s:%d/%s]:\n\t", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+		fprintf(stderr, "DBG!![%s:%d/%s]:\n\t", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 		fprintf(stderr, fmt, ## __VA_ARGS__); \
 		fprintf(stderr, "\n"); \
 		KNH_ASSERT(c); \
@@ -109,7 +103,7 @@
 
 #define KNH_CTXASSERT(ctx, c, fmt, ...) \
 	if(!(c)) { \
-		fprintf(stderr, "DEBUG[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+		fprintf(stderr, "DEBUG[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 		fprintf(stderr, fmt, ## __VA_ARGS__); \
 		fprintf(stderr, "\n"); \
 		KNH_ASSERT(c); \
@@ -119,14 +113,21 @@
 
 #define DBG_(stmt)
 #define DBG_P(fmt, ...) \
-	if(konoha_isDebugMode()) {\
+	if(konoha_isSystemDump2()) {\
 		fflush(stdout); \
-		fprintf(stderr, "DBG[%s:%d/%s]: ", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
+		fprintf(stderr, "DBG[%s:%d/%s]: ", knh_safefile(__FILE__), __LINE__, __FUNCTION__); \
 		fprintf(stderr, fmt, ## __VA_ARGS__); \
 		fprintf(stderr, "\n"); \
 	}\
 
-#define DBG_DUMP(ctx, o, opt, msg)
+#define DBG_DUMP(ctx, o, opt, msg) \
+	if(konoha_isSystemDump2()) {\
+		fprintf(stdout, "DBG[%s]: %s\n", __FUNCTION__, msg); \
+		knh_format(ctx, KNH_STDOUT, METHODN__dump, UP(o), UP(opt)); \
+		knh_flush(ctx, KNH_STDOUT);\
+		fprintf(stdout, "\n"); \
+	}\
+
 #define TODO()
 #define DBG_ASSERT(c, fmt, ...)   KNH_ASSERT(c)
 
@@ -144,7 +145,7 @@
 /* [STUPID] */
 
 #define KNH_FUTURE(msg) {\
-		fprintf(stderr, "FUTURE[%s:%d] %s will be supported in the near future.\n", KNH_SAFEFILE(__FILE__), __LINE__, msg); \
+		fprintf(stderr, "FUTURE[%s:%d] %s will be supported in the near future.\n", knh_safefile(__FILE__), __LINE__, msg); \
 	}\
 
 /* ======================================================================== */
@@ -161,10 +162,10 @@
 #ifdef KNH_USING_THREAD
 
 #define TODO_LOCK() \
-	fprintf(stderr, "TODO[%s:%d/%s]: LOCK\n", KNH_SAFEFILE(__FILE__) , __LINE__, __FUNCTION__); \
+	fprintf(stderr, "TODO[%s:%d/%s]: LOCK\n", knh_safefile(__FILE__) , __LINE__, __FUNCTION__); \
 
 #define TODO_UNLOCK() \
-	fprintf(stderr, "TODO[%s:%d/%s]: UNLOCK\n", KNH_SAFEFILE(__FILE__) , __LINE__, __FUNCTION__); \
+	fprintf(stderr, "TODO[%s:%d/%s]: UNLOCK\n", knh_safefile(__FILE__) , __LINE__, __FUNCTION__); \
 
 #else
 
