@@ -1,87 +1,116 @@
 #ifndef KONOHA_DEPS_H_
 #define KONOHA_DEPS_H_
-#include"konoha_t.h"
+#include<konoha/konoha_t.h>
+
+#ifdef KNH_USING_STDC
 #include<stdlib.h>
 #include<string.h>
+#include<setjmp.h>
+#endif
+
 
 /* ======================================================================== */
 /* [UNIX] */
 
-#ifdef KONOHA_OS__UNKONWN
-	#define KNH_USING_POSIX 1
-	#define KNH_USING_UNIX  1
+#ifdef KONOHA_ON_UNKNOWN
+	#define KNH_USING_STDC  1
 	#define KONOHA_OS_ENCODING "UTF-8"
 	#define KONOHA_OS_LINEFEED "\n"
+	#define KONOHA_OS_DLLEXT   ".so"
 #endif
 
-#ifdef KONOHA_OS__LINUX
+#ifdef KONOHA_ON_LINUX
 #define		KNH_USING_POSIX 1
 #define		KNH_USING_UNIX  1
 #define     KNH_USING_SOCKET 1
+#define 	KNH_USING_THREAD   1
+#define     KNH_USING_PTHREAD  1
 #define		KONOHA_OS_ENCODING "UTF-8"
 #define 	KNH_USING_KONOHAGET 1
-#ifdef HAVE_ICONV_H
-#define		KNH_USING_ICONV 1
-#endif
+	#ifdef HAVE_ICONV_H
+	#define		KNH_USING_ICONV 1
+	#endif
 //#ifdef HAVE_REGEX_H
 #define		KNH_USING_REGEX 1
 //#endif
 #define		KONOHA_OS_LINEFEED "\n"
 #define		KONOHA_OS_DLLEXT   ".so"
-#ifdef KONOHA_CC_WITH_ECLIPSE
-#define 	KNH_USING_READLINE 1
-#define		KONOHA_MONOLITHIC  1
-#endif
 #endif
 
-#ifdef KONOHA_OS__MACOSX
+#ifdef KONOHA_ON_MACOSX
 #define 	KNH_USING_POSIX    1
 #define 	KNH_USING_UNIX     1
 #define 	KNH_USING_SOCKET   1
 #define 	KNH_USING_ICONV    1
 #define 	KNH_USING_REGEX    1
 #define 	KNH_USING_SQLITE3  1
-#define 	KNH_USING_KONOHAGET 1
+#define 	KNH_USING_THREAD   1
+#define     KNH_USING_PTHREAD  1
 #define 	KONOHA_OS_ENCODING "UTF-8"
 #define 	KONOHA_OS_LINEFEED "\n"
 #define 	KONOHA_OS_DLLEXT ".dylib"
-#ifdef KONOHA_CC_WITH_ECLIPSE
-#define 	KNH_USING_READLINE 1
-#define		KONOHA_MONOLITHIC  1
 #endif
+
+/* ------------------------------------------------------------------------ */
+
+#ifdef KONOHA_ON_LKM
+#define KNH_USING_INT32      1
+#define KNH_USING_NOFLOAT    1
+#define KNH_USING_NOFILE     1
+
+#define KNH_EXT_QSORT  1
+#define KNH_EXT_SETJMP 1
+
+#define FILEPATH_BUFSIZ     80  /* TO SAVE STACKSIZE */
+#define KONOHA_HOMEPATH     "/dev/konoha"   /* meaningless */
+
+#undef KNH_USING_SOCKET
+#undef KNH_USING_ICONV
+#undef KNH_USING_REGEX
+#undef KNH_USING_SQLITE3
+#undef KNH_USING_KONOHAGET
+#undef KNH_USING_PTHREAD
+#undef KNH_USING_THREAD
+#undef KNH_USING_READLINE
+
+#define KONOHA_OS_ENCODING "UTF-8"
+#define KONOHA_OS_LINEFEED "\n"
+#define KONOHA_OS_DLLEXT   ""
+
 #endif
 
 /* ======================================================================== */
 /* [WINDOWS] */
 
-#ifdef KONOHA_OS__WINDOWS
+#ifdef KONOHA_ON_WINDOWS
 #include<windows.h>
 #define 	KNH_USING_WIN32    1
 #define 	KNH_USING_WINDOWS  1
 #define     KNH_USING_MATH     1
 #define 	KONOHA_OS_LINEFEED "\r\n"
 #ifdef KNHAPI_IMPORTS
-#define 	KNHAPI(T)     __declspec(dllimport) T __cdecl
+#define 	KNHAPI(T__)     __declspec(dllimport) T__ __cdecl
 #else
-#define 	KNHAPI(T)     __declspec(dllexport) T __cdecl
+#define 	KNHAPI(T__)     __declspec(dllexport) T__ __cdecl
 #endif
 
-#define 	KNH_EXPORTS(T)     __declspec(dllexport) T __cdecl
+#define 	KNH_EXPORTS(T__)     __declspec(dllexport) T__ __cdecl
 
 #define 	KONOHA_OS_DLLEXT ".dll"
 #define		KONOHA_OS_FILE_SEPARATOR '\\'
+#define     KONOHA_FOLDER "Konoha"
 #endif
 
-#ifdef KONOHA_OS__MINGW
+#ifdef KONOHA_ON_MINGW
 #define 	KNH_USING_WIN32    1
 #define 	KNH_USING_WINDOWS  1
 #define 	KONOHA_OS_LINEFEED "\r\n"
 #ifdef KNHAPI_IMPORTS
-#define 	KNHAPI(T) __declspec(dllimport) T __cdecl
+#define 	KNHAPI(T__) __declspec(dllimport) T__ __cdecl
 #else
-#define 	KNHAPI(T) __declspec(dllexport) T __cdecl
+#define 	KNHAPI(T__) __declspec(dllexport) T__ __cdecl
 #endif
-#define 	KNH_EXPORTS(T)  __declspec(dllexport) T __cdecl
+#define 	KNH_EXPORTS(T__)  __declspec(dllexport) T__ __cdecl
 #define 	KONOHA_OS_DLLEXT ".dll"
 #define		KONOHA_OS_FILE_SEPARATOR '\\'
 #ifndef HAVE_CONFIG_H
@@ -90,7 +119,7 @@
 #endif
 #endif
 
-#ifdef KONOHA_OS__CYGWIN
+#ifdef KONOHA_ON_CYGWIN
 #define 	KNH_USING_POSIX    1
 #define 	KNH_USING_UNIX     1
 #define 	KNH_USING_REGEX    1
@@ -99,10 +128,11 @@
 #endif
 
 /* ======================================================================== */
-/* [WINDOWS] */
+/* [TEABOARD] */
 
-#ifdef KONOHA_OS__TB
+#ifdef KONOHA_ON_TB
 #define 	KNH_USING_BTRON    1
+#define 	KNH_USING_TKAPI    1
 #define 	KONOHA_OS_ENCODING KONOHA_ENCODING
 #define 	KONOHA_OS_LINEFEED "\n"
 #define 	KONOHA_OS_DLLEXT ".so"
@@ -115,21 +145,29 @@
 
 #ifndef KNHAPI_
 #ifdef KNHAPI
-#define KNHAPI_(T)       KNHAPI(T)
+#define KNHAPI_(T__)       KNHAPI(T__)
 #else
 #define KNHAPI_(T)       T KNH_CC_FASTCALL
 #endif
 #endif
 
+#ifndef KONOHA_FOLDER
+#define KONOHA_FOLDER ".konoha"
+#endif
+
 #ifndef KNHAPI
+#ifdef  KONOHA_ON_WINDOWS
+#define KNHAPI(T__)         T__ __declspec(dllexport)
+#else
 #define KNHAPI(T)         T
+#endif
 #define KNH_EXPORTS(T)    T
 #endif
 
 #ifdef KONOHA_MONOLITHIC
 #undef KNHAPI
-#define KNHAPI(T)        T
-#define KNH_EXPORTS(T)   T
+#define KNHAPI(T__)        T__
+#define KNH_EXPORTS(T__)   T__
 #endif
 
 #ifndef KONOHA_OS_FILE_SEPARATOR
@@ -198,11 +236,8 @@
 
 /* ======================================================================== */
 
-#ifdef KNH_USING_POSIX
+#if defined(KNH_USING_POSIX) && !defined(KONOHA_ON_LKM)
 #include<dlfcn.h>
-#define KNH_RTLD_LAZY               RTLD_LAZY
-#else
-#define KNH_RTLD_LAZY               0
 #endif
 
 /* ------------------------------------------------------------------------ */
@@ -213,6 +248,11 @@
 	#define knh_getenv(n)           getenv(n)
 #endif
 
+
+#ifndef FILEPATH_BUFSIZ
+#define FILEPATH_BUFSIZ  256
+#endif
+
 /* ======================================================================== */
 
 #endif /*KONOHA_DEPS_H_*/
