@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -385,6 +385,11 @@ typedef struct {
 	knh_sline_t  line;
 } knh_FileLine_t ;
 
+typedef struct {
+	const char *file;
+	int   line;
+} knh_fline_t;
+
 #define knh_Object_data(o)       (((Int*)(o))->n.data)
 
 /* ------------------------------------------------------------------------ */
@@ -798,13 +803,14 @@ typedef struct {
 /* KNH_DRVAPI_TYPE__BYTECONV */
 
 struct knh_BytesConv_t;
-typedef size_t (*knh_fbcnv)(Ctx *ctx, struct knh_BytesConv_t *o, knh_bytes_t t, struct knh_Bytes_t *ba);
+typedef size_t (*knh_fbyteconv)(Ctx *ctx, struct knh_BytesConv_t *o, knh_bytes_t t, struct knh_Bytes_t *ba);
+typedef void   (*knh_fbyteconvfree)(Ctx *ctx, struct knh_BytesConv_t *);
 
 typedef struct {
 	int  type;
 	char *name;
-	knh_fbcnv  fbconv;
-	knh_fbcnv  fbconv_inverse;
+	knh_fbyteconv  fbconv;
+	knh_fbyteconv  fbconv_inverse;
 } knh_bconv_driapi_t;
 
 /* ------------------------------------------------------------------------ */
@@ -813,6 +819,7 @@ typedef struct {
 typedef knh_intptr_t knh_io_t;
 
 typedef knh_io_t (*f_io_open)(Ctx *ctx, knh_bytes_t urn, char *mode);
+typedef void (*f_io_init)(Ctx *ctx, Object *stream, char *mode);
 typedef knh_intptr_t (*f_io_read)(Ctx *ctx, knh_io_t fd, char *buf, size_t bufsiz);
 typedef knh_intptr_t (*f_io_write)(Ctx *ctx, knh_io_t fd, char *buf, size_t bufsiz);
 typedef void   (*f_io_close)(Ctx *ctx, knh_io_t fd);
@@ -822,6 +829,7 @@ typedef struct {
 	char *name;
 	size_t bufsiz;  /* knh_io_t == FILE* if bufsiz == 0 */
 	f_io_open    fopen;
+	f_io_init    finit;
 	f_io_read    fread;
 	f_io_write   fwrite;
 	f_io_close   fclose;
@@ -855,7 +863,7 @@ typedef struct {
 } knh_regmatch_t;
 
 typedef knh_regex_t* (*knh_fregmalloc)(Ctx *);
-typedef int (*knh_fregcomp)(Ctx *, knh_regex_t *, char *pattern, int flags);
+typedef int (*knh_fregcomp)(Ctx *, knh_regex_t *, char *pattern, char *option);
 typedef int (*knh_fregexec)(Ctx *, knh_regex_t *, char *str, size_t nmatch, knh_regmatch_t p[], int flags);
 typedef void (*knh_fregfree)(Ctx *, knh_regex_t *);
 
