@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -46,6 +46,18 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 
 /* ------------------------------------------------------------------------ */
 
+#define KNH_THROW_iZERODIV(n)  \
+	if(unlikely(n == 0)) { \
+		KNH_THROWs(ctx, "Arithmetic!!: Zero Divided"); \
+	}\
+
+#define KNH_THROW_fZERODIV(n)  \
+	if(unlikely(n == 0.0)) { \
+		KNH_THROWs(ctx, "Arithmetic!!: Zero Divided"); \
+	}\
+
+/* ------------------------------------------------------------------------ */
+
 #define KLR_MOV__wogc(ctx, v1, v2) {\
 		Object *v2_ = (Object*)v2;\
 		knh_Object_RCinc(v2_); \
@@ -193,6 +205,82 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 
 /* ------------------------------------------------------------------------ */
 
+#define KLR_ARRAY_INDEX(ctx, n, size)   (size_t)n
+
+#define KLR_AGET(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, sfp[n].ivalue, ((Array*)sfp[a].o)->size);\
+		if(idxn_ >= ((Array*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		KLR_MOV(ctx, sfp[b].o, KNH_FIELDn(sfp[a].o, idxn_));\
+	}\
+
+#define KLR_AGETn(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, n, ((Array*)sfp[a].o)->size);\
+		if(idxn_ >= ((Array*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		KLR_MOV(ctx, sfp[b].o, KNH_FIELDn(sfp[n].o, idxn_));\
+	}\
+
+#define KLR_IAGET(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, sfp[n].ivalue, ((IArray*)sfp[a].o)->size);\
+		if(idxn_ >= ((IArray*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		sfp[b].ivalue = ((IArray*)sfp[a].o)->ilist[idxn_];\
+	}\
+
+#define KLR_IAGETn(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, n, ((IArray*)sfp[a].o)->size);\
+		if(idxn_ >= ((IArray*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		sfp[b].ivalue = ((IArray*)sfp[a].o)->ilist[idxn_];\
+	}\
+
+#define KLR_FAGET(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, sfp[n].ivalue, ((FArray*)sfp[a].o)->size);\
+		if(idxn_ >= ((FArray*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		sfp[b].fvalue = ((FArray*)sfp[a].o)->flist[idxn_];\
+	}\
+
+#define KLR_FAGETn(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, n, ((FArray*)sfp[a].o)->size);\
+		if(idxn_ >= ((FArray*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		sfp[b].fvalue = ((FArray*)sfp[a].o)->flist[idxn_];\
+	}\
+
+#define KLR_ASET(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, sfp[n].ivalue, ((Array*)sfp[a].o)->size);\
+		if(idxn_ >= ((Array*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		KLR_MOV(ctx, KNH_FIELDn(sfp[a].o, idxn_), sfp[b].o);\
+	}\
+
+#define KLR_ASETn(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, n, ((Array*)sfp[a].o)->size);\
+		if(idxn_ >= ((Array*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		KLR_MOV(ctx, KNH_FIELDn(sfp[n].o, idxn_), sfp[b].o);\
+	}\
+
+#define KLR_IASET(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, sfp[n].ivalue, ((IArray*)sfp[a].o)->size);\
+		if(idxn_ >= ((IArray*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		((IArray*)sfp[a].o)->ilist[idxn_] = sfp[b].ivalue;\
+	}\
+
+#define KLR_IASETn(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, n, ((IArray*)sfp[a].o)->size);\
+		if(idxn_ >= ((IArray*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		((IArray*)sfp[a].o)->ilist[idxn_] = sfp[b].ivalue;\
+	}\
+
+#define KLR_FASET(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, sfp[n].ivalue, ((FArray*)sfp[a].o)->size);\
+		if(idxn_ >= ((FArray*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		((FArray*)sfp[a].o)->flist[idxn_] = sfp[b].fvalue;\
+	}\
+
+#define KLR_FASETn(ctx, b, a, n)  {\
+		size_t idxn_ = KLR_ARRAY_INDEX(ctx, n, ((FArray*)sfp[a].o)->size);\
+		if(idxn_ >= ((FArray*)sfp[a].o)->size) knh_throw_OutOfIndex(ctx, idxn_, ((Array*)sfp[a].o)->size, _HERE_); \
+		((FArray*)sfp[a].o)->flist[idxn_] = sfp[b].fvalue;\
+	}\
+
+/* ------------------------------------------------------------------------ */
+
 #define KLR_INITCODE(ctx, n) { \
 		knh_code_thread(ctx, pc, OPJUMP); \
 		((knh_kode_t*)pc)->opcode = OPCODE_SETESP;\
@@ -497,13 +585,13 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 	} \
 
 #define KLR_MAP(ctx, n, tcid)  { \
-		KLR_MOV(ctx, sfp[n+1].o, knh_tMapper_find(ctx, knh_Object_cid(sfp[n].o), tcid)); \
+		KLR_MOV(ctx, sfp[n+1].o, konoha_findMapper(ctx, knh_Object_cid(sfp[n].o), tcid)); \
 		(sfp[n+1].mpr)->fmap_1(ctx, sfp + n); \
 	} \
 
 #define KLR_MAPnc(ctx, n, tcid)  { \
 		if(likely(IS_NOTNULL(sfp[n].o))) {\
-			KLR_MOV(ctx, sfp[n+1].o, knh_tMapper_find(ctx, knh_Object_cid(sfp[n].o), tcid)); \
+			KLR_MOV(ctx, sfp[n+1].o, konoha_findMapper(ctx, knh_Object_cid(sfp[n].o), tcid)); \
 			(sfp[n+1].mpr)->fmap_1(ctx, sfp + n); \
 		}\
 	}\
@@ -511,11 +599,18 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 #define KLR_AMAP(ctx, n, tcid)  { \
 		knh_class_t scid = knh_Object_cid(sfp[n].o);\
 		if(scid != ((knh_class_t)tcid) && !knh_class_instanceof(ctx, scid, tcid) && scid != CLASS_Nue) { \
-			KLR_MOV(ctx, sfp[n+1].o, knh_tMapper_find(ctx, scid, tcid)); \
+			KLR_MOV(ctx, sfp[n+1].o, konoha_findMapper(ctx, scid, tcid)); \
 			(sfp[n+1].mpr)->fmap_1(ctx, sfp + n); \
 		} \
 	} \
 
+#define KLR_DCAST(ctx, n, tcid)  { \
+		knh_class_t scid = knh_Object_cid(sfp[n].o);\
+		if(scid != ((knh_class_t)tcid) && !knh_class_instanceof(ctx, scid, tcid) && scid != CLASS_Nue) { \
+			KNH_THROWs(ctx, "ClassCast!!"); \
+		} \
+	} \
+
 #define KLR_NNMAP(ctx, a, tcid)  { \
 		if(unlikely(IS_NULL(sfp[a].o))) {\
 			Int *v_ = (Int*)KNH_DEF(ctx, tcid);\
@@ -539,7 +634,7 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 	}\
 
 #define KLR_SKIP(ctx, PC, JUMP) \
-	if(!IS_DEBUG(ctx, sfp[0].o)) { \
+	if(!knh_Context_isDebug(ctx)) { \
 		KLR_JMP(ctx, PC, JUMP); \
 	} \
 
@@ -613,7 +708,7 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 			} \
 			ncid_ = knh_Object_cid(sfp[sc].o);\
 			if(ncid_ == reqc || knh_class_instanceof(ctx, reqc, ncid_)) break;\
-			KLR_MOV(ctx, sfp[sc+1].o, knh_tMapper_find(ctx, ncid_, reqc));\
+			KLR_MOV(ctx, sfp[sc+1].o, konoha_findMapper(ctx, ncid_, reqc));\
 			(sfp[sc+1].mpr)->fmap_1(ctx, sfp + sc); \
 		}while(IS_NOTNULL(sfp[sc].o));\
 		KLR_MOV(ctx, sfp[na].o, sfp[sc].o); sfp[na].data = sfp[sc].data;\
@@ -664,7 +759,7 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 			} \
 			ncid_ = knh_Object_cid(sfp[sc].o);\
 			if(ncid_ == ((knh_class_t)reqc) || knh_class_instanceof(ctx, reqc, ncid_)) break;\
-			KLR_MOV(ctx, sfp[sc+1].o, knh_tMapper_find(ctx, ncid_, reqc));\
+			KLR_MOV(ctx, sfp[sc+1].o, konoha_findMapper(ctx, ncid_, reqc));\
 			(sfp[sc+1].mpr)->fmap_1(ctx, sfp + sc); \
 		}while(IS_NOTNULL(sfp[sc].o));\
 		KLR_MOV(ctx, sfp[na].o, sfp[sc].o); sfp[na].data = sfp[sc].data;\
@@ -883,9 +978,17 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 #define KLR_iSUBn(ctx, c, a, n) SFi(c) = (SFi(a) - n)
 #define KLR_iMUL(ctx, c, a, b)  SFi(c) = (SFi(a) * SFi(b))
 #define KLR_iMULn(ctx, c, a, n) SFi(c) = (SFi(a) * n)
-#define KLR_iDIV(ctx, c, a, b)  SFi(c) = (SFi(a) / SFi(b))
+#define KLR_iDIV(ctx, c, a, b)  { \
+		KNH_THROW_iZERODIV(SFi(b)); \
+		SFi(c) = (SFi(a) / SFi(b)); \
+	} \
+
 #define KLR_iDIVn(ctx, c, a, n)  SFi(c) = (SFi(a) / n)
-#define KLR_iMOD(ctx, c, a, b)  SFi(c) = (SFi(a) % SFi(b))
+#define KLR_iMOD(ctx, c, a, b)  { \
+		KNH_THROW_iZERODIV(SFi(b)); \
+		SFi(c) = (SFi(a) % SFi(b)); \
+	} \
+
 #define KLR_iMODn(ctx, c, a, n)  SFi(c) = (SFi(a) % n)
 
 #define KLR_iEQ(ctx, c, a, b)  SFi(c) = (SFi(a) == SFi(b))
@@ -911,13 +1014,17 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 #define KLR_fSUBn(ctx, c, a, n) SFf(c) = (SFf(a) - n)
 #define KLR_fMUL(ctx, c, a, b)  SFf(c) = (SFf(a) * SFf(b))
 #define KLR_fMULn(ctx, c, a, n) SFf(c) = (SFf(a) * n)
-#define KLR_fDIV(ctx, c, a, b)  SFf(c) = (SFf(a) / SFf(b))
+#define KLR_fDIV(ctx, c, a, b)  { \
+		KNH_THROW_fZERODIV(SFf(b)); \
+		SFf(c) = (SFf(a) / SFf(b)); \
+	} \
+
 #define KLR_fDIVn(ctx, c, a, n)  SFf(c) = (SFf(a) / n)
 
-#define KLR_fEQ(ctx, c, a, b)  SFf(c) = (SFf(a) == SFf(b))
-#define KLR_fEQn(ctx, c, a, n)  SFf(c) = (SFf(a) == n)
-#define KLR_fNEQ(ctx, c, a, b)  SFf(c) = (SFf(a) != SFf(b))
-#define KLR_fNEQn(ctx, c, a, n)  SFf(c) = (SFf(a) != n)
+#define KLR_fEQ(ctx, c, a, b)  SFb(c) = (SFf(a) == SFf(b))
+#define KLR_fEQn(ctx, c, a, n)  SFb(c) = (SFf(a) == n)
+#define KLR_fNEQ(ctx, c, a, b)  SFb(c) = (SFf(a) != SFf(b))
+#define KLR_fNEQn(ctx, c, a, n)  SFb(c) = (SFf(a) != n)
 #define KLR_fLT(ctx, c, a, b)  SFb(c) = (SFf(a) < SFf(b))
 #define KLR_fLTn(ctx, c, a, n)  SFb(c) = (SFf(a) < n)
 #define KLR_fLTE(ctx, c, a, b)  SFb(c) = (SFf(a) <= SFf(b))
@@ -938,11 +1045,20 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 #define JIT_iSUBn(ctx, c, a, n) SFi(c) = (SFi(a) - n)
 #define JIT_iMUL(ctx, c, a, b)  SFi(c) = (SFi(a) * SFi(b))
 #define JIT_iMULn(ctx, c, a, n) SFi(c) = (SFi(a) * n)
-#define JIT_iDIV(ctx, c, a, b)  SFi(c) = (SFi(a) / SFi(b))
+#define JIT_iDIV(ctx, c, a, b)  { \
+		KNH_THROW_iZERODIV(SFi(b)); \
+		SFi(c) = (SFi(a) / SFi(b)); \
+	} \
+
 #define JIT_iDIVn(ctx, c, a, n)  SFi(c) = (SFi(a) / n)
-#define JIT_iMOD(ctx, c, a, b)  SFi(c) = (SFi(a) % SFi(b))
+#define JIT_iMOD(ctx, c, a, b) { \
+		KNH_THROW_iZERO(SFi(b)); \
+		SFi(c) = (SFi(a) % SFi(b)); \
+	} \
+
 #define JIT_iMODn(ctx, c, a, n)  SFi(c) = (SFi(a) % n)
 
+
 #define JIT_iEQ(ctx, c, a, b)  SFi(c) = (SFi(a) == SFi(b))
 #define JIT_iEQn(ctx, c, a, n)  SFi(c) = (SFi(a) == n)
 #define JIT_iNEQ(ctx, c, a, b)  SFi(c) = (SFi(a) != SFi(b))
@@ -964,13 +1080,17 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 #define JIT_fSUBn(ctx, c, a, n) SFf(c) = (SFf(a) - n)
 #define JIT_fMUL(ctx, c, a, b)  SFf(c) = (SFf(a) * SFf(b))
 #define JIT_fMULn(ctx, c, a, n) SFf(c) = (SFf(a) * n)
-#define JIT_fDIV(ctx, c, a, b)  SFf(c) = (SFf(a) / SFf(b))
+#define JIT_fDIV(ctx, c, a, b)  { \
+		KNH_THROW_fZERODIV(SFf(b)); \
+		SFf(c) = (SFf(a) / SFf(b)); \
+	} \
+
 #define JIT_fDIVn(ctx, c, a, n)  SFf(c) = (SFf(a) / n)
 
-#define JIT_fEQ(ctx, c, a, b)  SFf(c) = (SFf(a) == SFf(b))
-#define JIT_fEQn(ctx, c, a, n)  SFf(c) = (SFf(a) == n)
-#define JIT_fNEQ(ctx, c, a, b)  SFf(c) = (SFf(a) != SFf(b))
-#define JIT_fNEQn(ctx, c, a, n)  SFf(c) = (SFf(a) != n)
+#define JIT_fEQ(ctx, c, a, b)  SFb(c) = (SFf(a) == SFf(b))
+#define JIT_fEQn(ctx, c, a, n)  SFb(c) = (SFf(a) == n)
+#define JIT_fNEQ(ctx, c, a, b)  SFb(c) = (SFf(a) != SFf(b))
+#define JIT_fNEQn(ctx, c, a, n)  SFb(c) = (SFf(a) != n)
 #define JIT_fLT(ctx, c, a, b)  SFb(c) = (SFf(a) < SFf(b))
 #define JIT_fLTn(ctx, c, a, n)  SFb(c) = (SFf(a) < n)
 #define JIT_fLTE(ctx, c, a, b)  SFb(c) = (SFf(a) <= SFf(b))
@@ -1014,7 +1134,7 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 
 
 #define KNH_SECURE(ctx) \
-	if(((ctx)->flag & KNH_FLAG_CTXF_ADMIN) != KNH_FLAG_CTXF_ADMIN) { \
+	if(((ctx)->flag & KNH_FLAG_CTXF_TRUSTED) != KNH_FLAG_CTXF_TRUSTED) { \
 		knh_throwException(ctx, new_Exception(ctx, TS_SecurityException), KNH_SAFEFILE(__FILE__), __LINE__); \
 	} \
 
