@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -598,7 +598,8 @@ static METHOD knh__Array_lastIndexOf(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Array *o = (Array*)sfp[0].o;
 	knh_sfp_boxing(ctx, sfp + 1);
-	size_t i;
+	/* size_t i; */
+	int i;
 	for(i = o->size - 1; i >= 0; i--) {
 		if(knh_Object_compareTo(ctx, o->list[i], sfp[1].o) == 0) {
 			KNH_RETURN_Int(ctx, sfp, i);
@@ -648,7 +649,7 @@ static METHOD knh__Array_reverse(Ctx *ctx, knh_sfp_t *sfp)
 	Array *o = (Array*)sfp[0].o;
 	size_t i;
 	for(i = 0; i < o->size / 2; i++) {
-		size_t n = o->size - i;
+		size_t n = o->size - i - 1;
 		Object *temp = o->list[i];
 		o->list[i] = o->list[n];
 		o->list[n] = temp;
@@ -725,7 +726,7 @@ static METHOD knh__IArray_reverse(Ctx *ctx, knh_sfp_t *sfp)
 	IArray *o = (IArray*)sfp[0].o;
 	size_t i;
 	for(i = 0; i < o->size / 2; i++) {
-		size_t n = o->size - i;
+		size_t n = o->size - i - 1;
 		knh_int_t temp = o->ilist[i];
 		o->ilist[i] = o->ilist[n];
 		o->ilist[n] = temp;
@@ -803,7 +804,7 @@ static METHOD knh__FArray_reverse(Ctx *ctx, knh_sfp_t *sfp)
 	FArray *o = (FArray*)sfp[0].o;
 	size_t i;
 	for(i = 0; i < o->size / 2; i++) {
-		size_t n = o->size - i;
+		size_t n = o->size - i - 1;
 		knh_float_t temp = o->flist[i];
 		o->flist[i] = o->flist[n];
 		o->flist[n] = temp;
