@@ -1,14 +1,14 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -47,7 +47,7 @@ static METHOD knh__Bytes_new(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Bytes *o = (Bytes*)sfp[0].o;
 	size_t init = IS_NULL(sfp[1].o) ? 0 : knh_bytes_newsize(p_size(sfp[1]));
-	KNH_ASSERT(o->buf == NULL);
+	KNH_ASSERT(o->capacity == 0);
 	if(init > 0) {
 	  o->buf = (knh_uchar_t*)KNH_MALLOC(ctx, init);
 		o->capacity = init;
@@ -99,7 +99,7 @@ static METHOD knh__Bytes_opLshift(Ctx *ctx, knh_sfp_t *sfp)
 	Bytes *o = (Bytes*)sfp[0].o;
 	if(!knh_Object_isImmutable(o)) {
 		knh_sfp_t *v = sfp + 1;
-		int i, ac = knh_sfp_argc(ctx, v);
+		int i, ac = knh_stack_argc(ctx, v);
 		for(i = 0; i < ac; i++) {
 			knh_Bytes_putc(ctx, o, p_int(v[i]));
 		}
@@ -154,9 +154,42 @@ static METHOD knh__Bytes_opFill(Ctx *ctx, knh_sfp_t *sfp)
 	KNH_RETURN(ctx, sfp, o);
 }
 
+/* ------------------------------------------------------------------------ */
+/* @method void Bytes.memcpy(Int off, Bytes! s, Int off, Int len)  */
+
+static METHOD knh__Bytes_memcpy(Ctx *ctx, knh_sfp_t *sfp)
+{
+	Bytes *d = (Bytes*)sfp[0].o;
+	if(!knh_Object_isImmutable(d)) {
+		size_t doff = IS_NULL(sfp[1].o) ? 0 : p_int(sfp[1]);
+		Bytes *s = (Bytes*)sfp[2].o;
+		size_t soff = IS_NULL(sfp[3].o) ? 0 : p_int(sfp[3]);
+		size_t slen = IS_NULL(sfp[4].o) ? 0 : ((s)->size - soff);
+		if(doff + slen < (d)->size) {
+			knh_memcpy(d->buf + doff, s->buf + soff, slen);
+		}
+		else {
+			KNH_THROW_OUTOFINDEX(ctx, doff + slen, (d)->size);
+		}
+	}
+	KNH_RETURN_void(ctx, sfp);
+}
+
+
+
 /* ======================================================================== */
 /* [movabletext] */
 
+/* ------------------------------------------------------------------------ */
+/* @method void Bytes.%k(OutputStream w, String m) */
+
+static
+void knh_Bytes__k(Ctx *ctx, Bytes *o, OutputStream *w, String *m)
+{
+	knh_printf(ctx, w, "byte[%d]", (o)->size);
+}
+
+/* ------------------------------------------------------------------------ */
 /* @method void Bytes.%dump(OutputStream w, String m) */
 
 static
@@ -177,9 +210,7 @@ void knh_Bytes__dump(Ctx *ctx, Bytes *o, OutputStream *w, String *m)
 				knh_write(ctx, w, STEXT("   "));
 			}
 		}
-
 		knh_write(ctx, w, STEXT("    "));
-
 		for(i = 0; i < 16; i++) {
 			n = j * 16 + i;
 			if(n < o->size && isprint(o->buf[n])) {
