@@ -1,14 +1,14 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -239,17 +239,17 @@ static void knh_ResultSet__dump(Ctx *ctx, ResultSet *o, OutputStream *w, String
 			knh_write_delim(ctx,w);
 		}
 		knh_write(ctx, w, knh_String_tobytes(DP(o)->column[n].name));
-		knh_printf(ctx, w, "(%d): ", (int)n);
+		knh_printf(ctx, w, "(%d): ", n);
 		char *p = knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start;
 		switch(DP(o)->column[n].ctype) {
 			case knh_ResultSet_CTYPE__null :
 				knh_write(ctx, w, STEXT("null"));
 				break;
 			case knh_ResultSet_CTYPE__integer :
-				knh_write_integerfmt(ctx, w, KNH_INT_FMT, (knh_int_t)(*((knh_int_t*)p)));
+				knh_write_ifmt(ctx, w, KNH_INT_FMT, (*((knh_int_t*)p)));
 				break;
 			case knh_ResultSet_CTYPE__float :
-				knh_write__f(ctx, w, (knh_float_t)(*((knh_float_t*)p)));
+				knh_write_ffmt(ctx, w, KNH_FLOAT_FMT, (*((knh_float_t*)p)));
 				break;
 			case knh_ResultSet_CTYPE__text :
 				knh_write(ctx, w, B2(p, DP(o)->column[n].len));
