@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -83,6 +83,7 @@ static METHOD knh__Int_opMul(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD knh__Int_opDiv(Ctx *ctx, knh_sfp_t *sfp)
 {
+	KNH_THROW_iZERODIV(p_integer(sfp[1]));
 	KNH_RETURN_Int(ctx, sfp, p_integer(sfp[0]) / p_integer(sfp[1]));
 }
 
@@ -91,6 +92,7 @@ static METHOD knh__Int_opDiv(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD knh__Int_opMod(Ctx *ctx, knh_sfp_t *sfp)
 {
+	KNH_THROW_iZERODIV(p_integer(sfp[1]));
 	KNH_RETURN_Int(ctx, sfp, p_integer(sfp[0]) % p_integer(sfp[1]));
 }
 
@@ -147,12 +149,11 @@ static METHOD knh__Int_opGte(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD knh__Int_opLand(Ctx *ctx, knh_sfp_t *sfp)
 {
-	int ac = knh_sfp_argc(ctx, sfp + 1);
-	knh_sfp_t *v = sfp + 1;
+	int ac = knh_sfp_argc(ctx, sfp);
 	knh_int_t n = p_integer(sfp[0]);
 	size_t i;
-	for(i = 0; i < ac; i++) {
-		n = n & p_integer(v[i]);
+	for(i = 1; i < ac; i++) {
+		n = n & p_integer(sfp[i]);
 	}
 	KNH_RETURN_Int(ctx, sfp, n);
 }
@@ -162,12 +163,11 @@ static METHOD knh__Int_opLand(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD knh__Int_opLor(Ctx *ctx, knh_sfp_t *sfp)
 {
-	int ac = knh_sfp_argc(ctx, sfp + 1);
-	knh_sfp_t *v = sfp + 1;
+	int ac = knh_sfp_argc(ctx, sfp);
 	knh_int_t n = p_integer(sfp[0]);
 	size_t i;
-	for(i = 0; i < ac; i++) {
-		n = n | p_integer(v[i]);
+	for(i = 1; i < ac; i++) {
+		n = n | p_integer(sfp[i]);
 	}
 	KNH_RETURN_Int(ctx, sfp, n);
 }
@@ -291,6 +291,7 @@ static METHOD knh__Float_opMul(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD knh__Float_opDiv(Ctx *ctx, knh_sfp_t *sfp)
 {
+	KNH_THROW_fZERODIV(p_float(sfp[1]));
 	KNH_RETURN_Float(ctx, sfp, p_float(sfp[0]) / p_float(sfp[1]));
 }
 
@@ -366,6 +367,22 @@ static METHOD knh__Float_random(Ctx *ctx, knh_sfp_t *sfp)
 	KNH_RETURN_Float(ctx, sfp, knh_float_rand());
 }
 
+/* ------------------------------------------------------------------------ */
+/* @method[STATIC] Int! Float.floatToIntBits(Float! n) */
+
+static METHOD knh__Float_floatToIntBits(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN_Int(ctx, sfp, sfp[1].ivalue);
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method[STATIC] Float! Float.intToFloatBits(Int! n) */
+
+static METHOD knh__Float_intToFloatBits(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN_Float(ctx, sfp, sfp[1].ivalue);
+}
+
 /* ------------------------------------------------------------------------ */
 /* @method[NULLBASE|CONST] Int! Float.getSize() */
 
@@ -503,6 +520,16 @@ static METHOD knh__Int__bits(Ctx *ctx, knh_sfp_t *sfp)
 	knh_write_bits(ctx, sfp[1].w, sfp[0].ivalue, sizeof(knh_int_t) * 8);
 }
 
+/* ------------------------------------------------------------------------ */
+/* @method void Int.%c(OutputStream w, String m) */
+
+static METHOD knh__Int__c(Ctx *ctx, knh_sfp_t *sfp)
+{
+	char buf[16];
+	knh_format_utf8(buf, sizeof(buf), sfp[0].ivalue);
+	knh_write(ctx, sfp[1].w, B(buf));
+}
+
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 /* @method void Float.%s(OutputStream w, String m) */
@@ -598,7 +625,7 @@ static MAPPER knh_String_Boolean(Ctx *ctx, knh_sfp_t *sfp)
 static MAPPER knh_String_Int(Ctx *ctx, knh_sfp_t *sfp)
 {
 	knh_int_t v = 0;
-	if(knh_bytes_parseinteger(knh_String_tobytes(sfp[0].s), &v)) {
+	if(knh_bytes_parseint(knh_String_tobytes(sfp[0].s), &v)) {
 		KNH_MAPPED_Int(ctx, sfp, v);
 	}
 	KNH_MAPPED(ctx, sfp, KNH_NULL);
