@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -66,6 +66,26 @@ static METHOD knh__Object_new__init(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ======================================================================== */
+
+/* ------------------------------------------------------------------------ */
+/* @method[NULLBASE] Int! Object.opAddr() */
+
+static METHOD knh__Object_opAddr(Ctx *ctx, knh_sfp_t *sfp)
+{
+	void *p = NULL;
+	if(IS_bInt(sfp[0].o) || IS_bFloat(sfp[0].o)) {
+		p = (void*)(&sfp[0] + sizeof(void*));
+	}
+	else if(IS_Boolean(sfp[0].o)) {
+		p = (sfp[0].bvalue) ? (void*)(KNH_TRUE) : (void*)(KNH_FALSE);
+	}
+	else if(IS_NOTNULL(sfp[0].o)) {
+		p = (void*)sfp[0].o;
+	}
+	KNH_RETURN_Int(ctx, sfp, (knh_intptr_t)p);
+}
+
+/* ------------------------------------------------------------------------ */
 /* @method[CONST|NULLBASE] Boolean! Object.opEq(Any value) */
 
 static METHOD knh__Object_opEq(Ctx *ctx, knh_sfp_t *sfp)
@@ -226,6 +246,24 @@ void knh_Object__s(Ctx *ctx, Object *b, OutputStream *w, String *m)
 	}
 }
 
+/* ------------------------------------------------------------------------ */
+/* @method void Object.%p(OutputStream w, String m) */
+
+static METHOD knh__Object__p(Ctx *ctx, knh_sfp_t *sfp)
+{
+	void *p = NULL;
+	if(IS_bInt(sfp[0].o) || IS_bFloat(sfp[0].o)) {
+		p = (void*)(&sfp[0] + sizeof(void*));
+	}
+	else if(IS_Boolean(sfp[0].o)) {
+		p = (sfp[0].bvalue) ? (void*)(KNH_TRUE) : (void*)(KNH_FALSE);
+	}
+	else if(IS_NOTNULL(sfp[0].o)) {
+		p = (void*)sfp[0].o;
+	}
+	knh_write__p(ctx, sfp[1].w, p);
+}
+
 /* ------------------------------------------------------------------------ */
 /* @method void Object.%k(OutputStream w, String m) */
 
@@ -317,15 +355,6 @@ void knh_Object__empty(Ctx *ctx, Object *b, OutputStream *w, String *m)
 {
 }
 
-/* ------------------------------------------------------------------------ */
-/* @method void Object.%addr(OutputStream w, String m) */
-
-static
-void knh_Object__addr(Ctx *ctx, Object *b, OutputStream *w, String *m)
-{
-	knh_write__p(ctx, w, (void*)b);
-}
-
 /* ------------------------------------------------------------------------ */
 /* @map Object String! */
 
@@ -455,7 +484,7 @@ char *knh_methodop_tochar(knh_methodn_t mn)
 static
 void knh_Class_NAME__man(Ctx *ctx, knh_class_t cid, OutputStream *w)
 {
-	knh_write__s(ctx, w, knh_message_text(KMSG_CLASS));
+	knh_write__s(ctx, w, _("Class"));
 	knh_write_EOL(ctx, w);
 
 	knh_write_TAB(ctx, w);
@@ -565,7 +594,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 			knh_DictMap_removeAt(ctx, dm, i);
 			if(DP(mtd)->cid == CLASS_Object && cid != CLASS_Object) continue;
 			if(hasCaption == 0) {
-				knh_write__s(ctx, w, knh_message_text(KMSG_OPERATOR));
+				knh_write__s(ctx, w, _("Operator"));
 				knh_write_EOL(ctx, w);
 				hasCaption = 1;
 			}
@@ -597,7 +626,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 			if(DP(mtd)->cid == CLASS_Object && cid != CLASS_Object) continue;
 //			if(DP(mtd)->cid != ctx->share->ClassTable[cid].bcid) continue;
 			if(hasCaption == 0) {
-				knh_write__s(ctx, w, knh_message_text(KMSG_METHOD));
+				knh_write__s(ctx, w, _("Method"));
 				knh_write_EOL(ctx, w);
 				hasCaption = 1;
 			}
@@ -616,7 +645,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 			if(!METHODN_IS_MOVTEXT(DP(mtd)->mn)) continue;
 			if(DP(mtd)->cid == CLASS_Object && cid != CLASS_Object) continue;
 			if(hasCaption == 0) {
-				knh_write__s(ctx, w, knh_message_text(KMSG_FORMATTER));
+				knh_write__s(ctx, w, _("Formatter"));
 				knh_write_EOL(ctx, w);
 				knh_write_TAB(ctx, w);
 				hasCaption = 1;
