@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -40,12 +40,49 @@ extern "C" {
 #ifdef KNH_CC_METHODAPI
 
 /* ------------------------------------------------------------------------ */
-/* @method InputStream InputStream.new(String! urn) */
+/* @method InputStream InputStream.new(String! urn, String mode) */
 
 static METHOD knh__InputStream_new(Ctx *ctx, knh_sfp_t *sfp)
 {
 	InputStream *o = (InputStream*)sfp[0].o;
-	KNH_RETURN(ctx, sfp, knh_InputStream_open(ctx, o, sfp[1].s));
+	KNH_RETURN(ctx, sfp, knh_InputStream_open(ctx, o, sfp[1].s, sfp[2].s));
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method Int! InputStream.getChar() */
+
+static METHOD knh__InputStream_getChar(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN_Int(ctx, sfp, knh_InputStream_getc(ctx, (InputStream*)sfp[0].o));
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method Int! InputStream.read(Bytes! buf, Int offset, Int len) */
+
+static METHOD knh__InputStream_read(Ctx *ctx, knh_sfp_t *sfp)
+{
+	Bytes *ba = (Bytes*)sfp[1].o;
+	knh_bytes_t buf = knh_Bytes_tobytes(ba);
+	size_t offset = 0;
+	if(IS_Int(sfp[2].o)) {
+		offset = (size_t)sfp[2].ivalue;
+		KNH_THROW_OUTOFINDEX(ctx, offset, buf.len);
+		buf = knh_bytes_last(buf, offset);
+	}
+	if(IS_Int(sfp[3].o)) {
+		size_t len = (size_t)sfp[3].ivalue;
+		knh_Bytes_ensureSize(ctx, ba, offset + len);
+		buf.len = len;
+	}
+	KNH_RETURN_Int(ctx, sfp, knh_InputStream_read(ctx, (InputStream*)sfp[0].o, (char*)buf.buf, buf.len));
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method String! InputStream.getEncoding() */
+
+static METHOD knh__InputStream_getEncoding(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN(ctx, sfp, DP((InputStream*)sfp[0].o)->enc);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -56,7 +93,6 @@ static METHOD knh__InputStream_isClosed(Ctx *ctx, knh_sfp_t *sfp)
 	KNH_RETURN_Boolean(ctx, sfp, knh_InputStream_isClosed((InputStream*)sfp[0].o));
 }
 
-
 /* ======================================================================== */
 /* [iterators] */
 
@@ -112,9 +148,9 @@ static METHOD knh__OutputStream_new(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void OutputStream.putc(Int! ch) */
+/* @method void OutputStream.putChar(Int! ch) */
 
-static METHOD knh__OutputStream_putc(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD knh__OutputStream_putChar(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Bytes *ba = DP(sfp[0].w)->ba;
 	KNH_ASSERT(IS_Bytes(ba));
@@ -187,7 +223,7 @@ static METHOD knh__OutputStream_println(Ctx *ctx, knh_sfp_t *sfp)
 static METHOD knh__OutputStream_opLshift(Ctx *ctx, knh_sfp_t *sfp)
 {
 	OutputStream *out = (OutputStream*)sfp[0].o;
-	knh_sfp_t *v = sfp + 1;
+	knh_sfp_t *v = sfp + 1, *esp1 = ctx->esp + 1;
 	int ac = knh_sfp_argc(ctx, v);
 	size_t i;
 	for(i = 0; i < ac; i++) {
@@ -206,7 +242,9 @@ static METHOD knh__OutputStream_opLshift(Ctx *ctx, knh_sfp_t *sfp)
 			}
 		}
 		else {
-			KNH_SHIFTESP(ctx, sfp + i); // Trickey
+			KNH_SETv(ctx, esp1[0].o, v[i].o);
+			esp1[0].data = v[i].data;
+			KNH_ASSERT(ctx->esp + 1 == esp1);
 			knh_esp1_format(ctx, METHODN__s, out, KNH_NULL);
 		}
 	}
@@ -226,6 +264,14 @@ void knh_OutputStream__k(Ctx *ctx, OutputStream *o, OutputStream *w, String *m)
 	knh_putc(ctx, w, '\'');
 }
 
+/* ------------------------------------------------------------------------ */
+/* @method String! OutputStream.getEncoding() */
+
+static METHOD knh__OutputStream_getEncoding(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN(ctx, sfp, DP((OutputStream*)sfp[0].o)->enc);
+}
+
 /* ======================================================================== */
 /* [Socket] */
 
