@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -53,7 +53,7 @@ static METHOD knh__System_getIn(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD knh__System_setIn(Ctx *ctx, knh_sfp_t *sfp)
 {
-	if(knh_Context_isAdmin(ctx)) {
+	if(knh_Context_isTrusted(ctx)) {
 		if(IS_NULL(sfp[1].o)) {
 			KNH_SETv(ctx, DP(ctx->sys)->in, konoha_getClassDefaultValue(ctx, CLASS_InputStream));
 		}
@@ -77,7 +77,7 @@ static METHOD knh__System_getOut(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD knh__System_setOut(Ctx *ctx, knh_sfp_t *sfp)
 {
-	if(knh_Context_isAdmin(ctx)) {
+	if(knh_Context_isTrusted(ctx)) {
 		if(IS_NULL(sfp[1].o)) {
 			KNH_SETv(ctx, DP(ctx->sys)->out, konoha_getClassDefaultValue(ctx, CLASS_OutputStream));
 		}
@@ -101,7 +101,7 @@ static METHOD knh__System_getErr(Ctx *ctx, knh_sfp_t *sfp)
 
 static METHOD knh__System_setErr(Ctx *ctx, knh_sfp_t *sfp)
 {
-	if(knh_Context_isAdmin(ctx)) {
+	if(knh_Context_isTrusted(ctx)) {
 		if(IS_NULL(sfp[1].o)) {
 			KNH_SETv(ctx, DP(ctx->sys)->err, konoha_getClassDefaultValue(ctx, CLASS_OutputStream));
 		}
@@ -112,6 +112,16 @@ static METHOD knh__System_setErr(Ctx *ctx, knh_sfp_t *sfp)
 	KNH_RETURN_void(ctx, sfp);
 }
 
+/* ------------------------------------------------------------------------ */
+/* @method[STATIC] void System.setRandomSeed(Int seed) */
+
+METHOD knh__System_setRandomSeed(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_uint_t seed = IS_NULL(sfp[1].o) ? 0 : p_uinteger(sfp[1]);
+	knh_srand(seed);
+	KNH_RETURN_void(ctx, sfp);
+}
+
 /* ------------------------------------------------------------------------ */
 /* @method[STATIC] void System.exit(Int status) */
 
@@ -347,11 +357,8 @@ void knh_Exception__dump(Ctx *ctx, Exception *o, OutputStream *w, String *m)
 
 void knh_System__k(Ctx *ctx, System *o, OutputStream *w, String *m)
 {
-#ifdef KONOHA_UNDER_GPL2
-	knh_printf(ctx, w, "Konoha %s %s GPL2 (#%d, %s, %s)", KONOHA_VERSION, KONOHA_XCODE, KONOHA_BUILDID, __DATE__, __TIME__);
-#else
-	knh_printf(ctx, w, "Konoha %s %s (#%d, %s, %s)" TERM_INIT, KONOHA_VERSION, KONOHA_XCODE, KONOHA_BUILDID, __DATE__, __TIME__);
-#endif
+	knh_printf(ctx, w, "Konoha %s(%s) %s (#%d, %s, %s)",
+		KONOHA_VERSION, KONOHA_XCODE, KONOHA_LICENSE, KONOHA_BUILDID, __DATE__, __TIME__);
 }
 
 
@@ -362,8 +369,7 @@ void knh_System__dump(Ctx *ctx, System *o, OutputStream *w, String *m)
 {
 	knh_System__k(ctx, o, w, m);
 	knh_write_EOL(ctx, w);
-	knh_printf(ctx, w, "[%s] on %s (%d, %s, %s)\n", KONOHA_CC_VERSION, KONOHA_PLATFORM,
-			sizeof(void*) * 8, konoha_encoding(), knh_message_text(KMSG_LANG));
+	knh_printf(ctx, w, "[%s] on %s (%d, %s)\n", KONOHA_CC_VERSION, KONOHA_PLATFORM, sizeof(void*) * 8, konoha_encoding());
 	knh_write__s(ctx, w, "Options:");
 
 #ifdef KNH_FASTMODE
