@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -135,6 +135,16 @@ KNHAPI(void) knh_Bytes_clear(Bytes *o, size_t pos)
 
 /* ------------------------------------------------------------------------ */
 
+KNHAPI(void) knh_Bytes_ensureSize(Ctx *ctx, Bytes *o, size_t len)
+{
+	if(o->capacity < len) {
+		knh_Bytes_expands(ctx, o, len);
+	}
+	o->size = len;
+}
+
+/* ------------------------------------------------------------------------ */
+
 KNHAPI(void) knh_Bytes_putc(Ctx *ctx, Bytes *o, int ch)
 {
 	if(o->size == o->capacity) {
