@@ -1,14 +1,14 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -29,7 +29,7 @@
 
 #include"commons.h"
 
-#ifdef KONOHA_OS__WINDOWS
+#ifdef KONOHA_ON_WINDOWS
 #pragma comment(lib, "iconv.lib")
 #endif
 
@@ -39,53 +39,75 @@
 extern "C" {
 #endif
 
-/* ======================================================================== */
-/* [macros] */
-
-static
-size_t f_bconv__iconv(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba);
-
 /* ======================================================================== */
 /* [fbconv] */
 
 size_t knh_BytesConv_conv(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba)
 {
-	return DP(o)->fbconv(ctx, o, t, ba);
+	return o->fbconv(ctx, o, t, ba);
 }
 
 
+///* ------------------------------------------------------------------------ */
+//
+//BytesConv* new_BytesConv(Ctx *ctx, knh_fbyteconv fbconv)
+//{
+//	knh_BytesConv_t *o = (BytesConv*)new_Object_bcid(ctx, CLASS_BytesConv, 0);
+//	DP(o)->fbconv = (fbconv == NULL) ? knh_fbyteconv_nop : fbconv;
+//	return o;
+//}
+
 /* ------------------------------------------------------------------------ */
 
-BytesConv* new_BytesConv(Ctx *ctx, String *name, knh_fbcnv fbconv)
+static
+size_t knh_fbyteconv_noconv(Ctx *ctx, BytesConv *bc, knh_bytes_t t, knh_Bytes_t *ba)
 {
-	knh_BytesConv_t *o = (BytesConv*)new_Object_bcid(ctx, CLASS_BytesConv, 0);
-	KNH_SETv(ctx, DP(o)->name, name);
-	DP(o)->fbconv = (fbconv == NULL) ? f_bconv__NOP : fbconv;
-	return o;
+	knh_Bytes_write(ctx, ba, t);
+	return t.len;
 }
 
 /* ------------------------------------------------------------------------ */
 /* [iconv] */
 
+iconv_t knh_iconv_open(Ctx *ctx, char *to, char *from)
+{
+#if defined(KNH_USING_ICONV)
+	return iconv_open(to, from);
+#else
+	return ((iconv_t)-1);
+#endif
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_iconv_close(Ctx *ctx, iconv_t cd)
+{
+#if defined(KNH_USING_ICONV)
+	iconv_close(cd);
+#endif
+}
+
+/* ------------------------------------------------------------------------ */
+
 static
-size_t f_bconv__iconv(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba)
+size_t knh_fbyteconv_iconv(Ctx *ctx, BytesConv *bc, knh_bytes_t t, knh_Bytes_t *ba)
 {
 #if defined(KNH_USING_ICONV)
 	char buffer[4096], *ibuf = (char*)t.buf;
 	size_t ilen = t.len, rsize = 0, ilen_prev = ilen;
-	iconv_t cd = DP(o)->iconv_d;
+	iconv_t cd = bc->iconv_d;
 	KNH_ASSERT(cd != (iconv_t)-1);
 
 	while(ilen > 0) {
 		char *obuf = buffer;
 		size_t olen = sizeof(buffer);
 //		DBG2_P("S in=%p,%d out=%p,%d", ibuf, ilen, obuf, olen);
-#if defined(KONOHA_OS__MACOSX)
+#if defined(KONOHA_ON_MACOSX)
 		size_t rc = iconv(cd, &ibuf, &ilen, &obuf, &olen);
 #else
 		size_t rc = iconv(cd, &ibuf, &ilen, &obuf, &olen);
 #endif
-		//		DBG2_P("E in=%p,%d out=%p,%d", ibuf, ilen, obuf, olen);
+//		DBG2_P("E in=%p,%d out=%p,%d", ibuf, ilen, obuf, olen);
 		olen = sizeof(buffer) - olen;
 //		DBG2_P("E2 in=%p,%d out=%p,%d", ibuf, ilen, obuf, olen);
 		if(rc == (size_t)-1) {
@@ -101,54 +123,54 @@ size_t f_bconv__iconv(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba)
 	}
 	return rsize;
 #else
-	return 0;
+	knh_Bytes_write(ctx, ba, t);
+	return t.len;
 #endif
 }
 
 /* ------------------------------------------------------------------------ */
 
+static
+void knh_fbyteconvfree_iconv(Ctx *ctx, BytesConv *bc)
+{
+	if(bc->iconv_d != (iconv_t)-1) {
+		knh_iconv_close(ctx, bc->iconv_d);
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
 BytesConv* new_BytesConv__iconv(Ctx *ctx, char *from, char *to)
 {
-#ifdef KNH_USING_ICONV
-	iconv_t cd = iconv_open(to, from);
+	iconv_t cd = knh_iconv_open(ctx, to, from);
 	if(cd == (iconv_t)-1) {
-		return (BytesConv*)KNH_NULL;
+		KNH_WARNING(ctx, _("unsupported iconv(from=%s, to=%s)"), from, to);
+	}
+
+	BytesConv* o = (BytesConv*)new_Object_bcid(ctx, CLASS_BytesConv, 0);
+	if(cd != (iconv_t)-1) {
+		o->iconv_d = cd;
+		o->fbconv = knh_fbyteconv_iconv;
+		o->fbconvfree = knh_fbyteconvfree_iconv;
 	}
 	else {
-		BytesConv* o = (BytesConv*)new_Object_bcid(ctx, CLASS_BytesConv, 0);
-		char buf[FILENAME_BUFSIZ];
-		knh_snprintf(buf, sizeof(buf), "iconv:%s=>%s", from, to);
-		KNH_SETv(ctx, DP(o)->name, new_String(ctx, B(buf), NULL));
-		DP(o)->iconv_d = cd;
-		DP(o)->fbconv = f_bconv__iconv;
-		return o;
+		o->fbconv = knh_fbyteconv_noconv;
 	}
-#else
-	DBG2_P("uninstalled iconv!!");
-	return (BytesConv*)KNH_NULL;
-#endif
+	return o;
 }
 
 /* ------------------------------------------------------------------------ */
 
 BytesConv* new_BytesConv__in(Ctx *ctx, char *from)
 {
-#ifdef KNH_USING_ICONV
 	return new_BytesConv__iconv(ctx, from, KONOHA_ENCODING);
-#else
-	return (BytesConv*)KNH_NULL;
-#endif
 }
 
 /* ------------------------------------------------------------------------ */
 
 BytesConv* new_BytesConv__out(Ctx *ctx, char *to)
 {
-#ifdef KNH_USING_ICONV
 	return new_BytesConv__iconv(ctx, KONOHA_ENCODING, to);
-#else
-	return (BytesConv*)KNH_NULL;
-#endif
 }
 
 
@@ -169,8 +191,6 @@ static void knh_Context_closeBConvBuf(Ctx* ctx, Bytes *ba)
 	knh_Bytes_clear(ba, 0);
 }
 
-/* ------------------------------------------------------------------------ */
-
 /* ======================================================================== */
 /* [String] */
 
@@ -180,7 +200,7 @@ String *new_String__bconv(Ctx *ctx, knh_bytes_t t, BytesConv *bc)
 	if(t.len == 0) return TS_EMPTY;
 	Bytes *ba = knh_Context_openBConvBuf(ctx);
 	String *s;
-	DP(bc)->fbconv(ctx, bc, t, ba);
+	bc->fbconv(ctx, bc, t, ba);
 	s = new_String(ctx, knh_Bytes_tobytes(ba), NULL);
 	knh_Context_closeBConvBuf(ctx, ba);
 	return s;
@@ -188,28 +208,28 @@ String *new_String__bconv(Ctx *ctx, knh_bytes_t t, BytesConv *bc)
 
 /* ------------------------------------------------------------------------ */
 
-String *new_String__cwbconv(Ctx *ctx, knh_cwb_t cwb, BytesConv *bc)
+String *new_String__cwbconv(Ctx *ctx, knh_cwb_t *cwb, BytesConv *bc)
 {
 	if(knh_cwb_size(cwb) == 0) {
 		return TS_EMPTY;
 	}
 	else if(IS_NULL(bc)) {
 		String *s = new_String(ctx, knh_cwb_tobytes(cwb), NULL);
-		knh_cwb_clear(cwb);
+		knh_cwb_close(cwb);
 		return s;
 	}else {
 		Bytes *ba = knh_Context_openBConvBuf(ctx);
-		DP(bc)->fbconv(ctx, bc, knh_cwb_tobytes(cwb), ba);
+		bc->fbconv(ctx, bc, knh_cwb_tobytes(cwb), ba);
 		String *s = new_String(ctx, knh_Bytes_tobytes(ba), NULL);
 		knh_Context_closeBConvBuf(ctx, ba);
-		knh_cwb_clear(cwb);
+		knh_cwb_close(cwb);
 		return s;
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(String*) new_String__fbcnv(Ctx *ctx, String *s, knh_fbcnv fbcnv, BytesConv *bc)
+KNHAPI(String*) new_String__fbcnv(Ctx *ctx, String *s, knh_fbyteconv fbcnv, BytesConv *bc)
 {
 	knh_bytes_t base = knh_String_tobytes(s);
 	Bytes *ba = knh_Context_openBConvBuf(ctx);
@@ -232,16 +252,6 @@ void knh_OutputStream_write__bconv(Ctx *ctx, OutputStream *w, knh_bytes_t t)
 	knh_Context_closeBConvBuf(ctx, ba);
 }
 
-/* ======================================================================== */
-/* [movabletext] */
-
-/* @method void BytesConv.%k(OutputStream w, String m) */
-
-void knh_BytesConv__k(Ctx *ctx, BytesConv *o, OutputStream *w, String *m)
-{
-	knh_write(ctx, w, knh_String_tobytes(DP(o)->name));
-}
-
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
