@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -35,28 +35,6 @@
 extern "C" {
 #endif
 
-/* ======================================================================== */
-/* [macros] */
-
-#define _CLASS_newid          ((knh_class_t)-1)
-#define _CLASS_unknown        ((knh_class_t)-2)
-
-#define _knh_Class_cid(c)     (knh_class_t)(c)->cid
-#define _KNH_FLAG_CF2OF(f)        (f)
-#define _knh_Class_isGenerics(cid)    (ctx->share->ClassTable[cid].p1 != CLASS_Nue)
-
-/* ------------------------------------------------------------------------ */
-
-#define _KNH_FLAG_SF_FIELD          KNH_FLAG_T1
-
-#define _STRUCT_ISFIELD(sid)        ((sid & KNH_FLAG_SF_FIELD) == KNH_FLAG_SF_FIELD)
-#define _BSIZE_TOSID(bsize)         (((knh_struct_t)bsize)|KNH_FLAG_SF_FIELD)
-#define _STRUCT_FIELD(bsize)        (((knh_struct_t)bsize)|KNH_FLAG_SF_FIELD)
-#define _STRUCT_FIELDSIZE(sid)      (sid & (~KNH_FLAG_SF_FIELD))
-#define _STRUCT_UNMASK(sid)         (sid & (~KNH_FLAG_SF_FIELD))
-
-#define _KNH_ASSERT_sid(sid)   KNH_ASSERT(((knh_struct_t)sid) < ctx->share->StructTableSize)
-
 /* ======================================================================== */
 /* [ClassTable] */
 
@@ -94,8 +72,6 @@ knh_class_t knh_ClassTable_newId(Ctx *ctx)
 	return CLASS_unknown;
 }
 
-#define _KNH_ASSERT_cid(cid)    KNH_ASSERT(cid < KNH_TCLASS_SIZE)
-
 /* ------------------------------------------------------------------------ */
 
 static knh_Class_t *new_Class(Ctx *ctx, knh_class_t cid)
@@ -111,7 +87,7 @@ static knh_Class_t *new_Class(Ctx *ctx, knh_class_t cid)
 void konoha_setClassName(Ctx *ctx, knh_class_t cid, String *lname)
 {
 	KNH_ASSERT_cid(cid);
-	//KNH_NOTICE(ctx, "added new class: %s", knh_String_tochar(lname));
+	//KNH_NOTICE(ctx, _("added new class: %s"), knh_String_tochar(lname));
 	KNH_ASSERT(ctx->share->ClassTable[cid].class_cid == NULL);
 	KNH_INITv(ctx->share->ClassTable[cid].class_cid, new_Class(ctx, cid));
 	KNH_INITv(ctx->share->ClassTable[cid].lname, lname);
@@ -120,7 +96,7 @@ void konoha_setClassName(Ctx *ctx, knh_class_t cid, String *lname)
 		knh_index_t idx = knh_bytes_index(n, '{');
 		if(idx != -1) {
 			KNH_INITv(ctx->share->ClassTable[cid].sname, lname);
-			return;
+			goto L_ADDDICT;
 		}
 		if(knh_bytes_endsWith(n, STEXT(".."))) {
 			n.len -= 2;
@@ -137,6 +113,7 @@ void konoha_setClassName(Ctx *ctx, knh_class_t cid, String *lname)
 			KNH_INITv(ctx->share->ClassTable[cid].sname, new_String(ctx, knh_bytes_last(n, idx + 1), lname));
 		}
 	}
+	L_ADDDICT:
 	//DBG2_P("NEW CLASS: %s, %s", knh_String_tochar(lname), knh_String_tochar(ctx->share->ClassTable[cid].sname));
 	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
 	knh_DictSet_append(ctx, DP(ctx->sys)->ClassNameDictSet, lname, cid+1);
@@ -149,9 +126,6 @@ void konoha_setClassName(Ctx *ctx, knh_class_t cid, String *lname)
 /* ------------------------------------------------------------------------ */
 /* [name] */
 
-#define _CLASSN(cid)   knh_ClassTable_CLASSN(ctx, cid)
-#define _CLASSNo(o)    knh_ClassTable_CLASSN(ctx, knh_Object_cid(o))
-
 char *knh_ClassTable_CLASSN(Ctx *ctx, knh_class_t cid)
 {
 	if(cid == CLASS_unknown) {
@@ -290,6 +264,25 @@ konoha_addGenericsClass(Ctx *ctx, knh_class_t cid, String *name, knh_class_t bci
 	return cid;
 }
 
+/* ======================================================================== */
+
+void KNH_ACLASS(Ctx *ctx, knh_class_t cid, knh_class_t p1)
+{
+	char buf[CLASSNAME_BUFSIZ];
+	knh_snprintf(buf, sizeof(buf), "%s[]", CLASSN(p1));
+	konoha_addGenericsClass(ctx, cid, new_String(ctx, B(buf), NULL), CLASS_Array, p1, CLASS_Nue);
+}
+
+/* ------------------------------------------------------------------------ */
+
+void KNH_ICLASS(Ctx *ctx, knh_class_t cid, knh_class_t p1)
+{
+	char buf[CLASSNAME_BUFSIZ];
+	knh_snprintf(buf, sizeof(buf), "%s..", CLASSN(p1));
+	konoha_addGenericsClass(ctx, cid, new_String(ctx, B(buf), NULL), CLASS_Iterator, p1, CLASS_Nue);
+}
+
+
 /* ======================================================================== */
 /* [ClassStruct] */
 
@@ -309,6 +302,7 @@ knh_index_t knh_Class_indexOfField(Ctx *ctx, knh_class_t cid, knh_fieldn_t fn)
 {
 	L_TAIL:;
 	KNH_ASSERT_cid(cid);
+	DBG2_P("******* cid=%s, offset=%d, supcid=%s", CLASSN(cid), ctx->share->ClassTable[cid].offset, CLASSN(ctx->share->ClassTable[cid].supcid));
 	{
 		knh_index_t idx = -1;
 		ClassStruct *cs = ctx->share->ClassTable[cid].cstruct;
@@ -360,7 +354,7 @@ knh_index_t knh_Class_queryField(Ctx *ctx, knh_class_t cid, knh_fieldn_t fnq)
 knh_cfield_t *knh_Class_fieldAt(Ctx *ctx, knh_class_t cid, size_t n)
 {
 	KNH_ASSERT_cid(cid);
-	KNH_ASSERT(0 <= n && n < ctx->share->ClassTable[cid].size);
+	KNH_ASSERT(/*0 <= n &&*/ n < ctx->share->ClassTable[cid].size);
 	L_TAIL:;
 	{
 		ClassStruct *cs = ctx->share->ClassTable[cid].cstruct;;
@@ -392,7 +386,7 @@ void knh_cfield_dump(Ctx *ctx, knh_cfield_t *f, size_t offset, size_t fsize, Out
 			continue;
 		}
 		if(f[idx].fn == FIELDN_register || f[idx].type == CLASS_unknown) continue;
-		knh_printf(ctx, w, "[%d] %F %T %N = %O\n", (offset+idx), f[idx].flag, f[idx].type, f[idx].fn, f[idx].value);
+		knh_printf(ctx, w, "[%d] %T %N = %O\n", (offset+idx), f[idx].type, f[idx].fn, f[idx].value);
 	}
 }
 
@@ -502,7 +496,8 @@ void knh_ClassMap__man(Ctx *ctx, ClassMap *o, OutputStream *w, knh_class_t cid)
 	knh_ClassMap_sort(ctx, o);
 	for(i = 0; i < DP(o)->size; i++) {
 		if(hasCaption == 0) {
-			knh_printf(ctx, w, "%s\n", knh_message_text(KMSG_MAPPING));
+			knh_write__s(ctx, w, _("Mapper"));
+			knh_write_EOL(ctx, w);
 			hasCaption = 1;
 		}
 		if(from == 0) {
@@ -533,6 +528,7 @@ void knh_ClassMap__man(Ctx *ctx, ClassMap *o, OutputStream *w, knh_class_t cid)
 	}
 }
 
+/* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
 }
