@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -45,6 +45,7 @@ KNHAPI(InputStream*) new_InputStream__io(Ctx *ctx, String *urn, knh_io_t fd, knh
 	DP(o)->fd = fd;
 	if(fd != -1) {
 		DP(o)->driver = drv;
+		DP(o)->bufsiz = drv->bufsiz;
 		if(DP(o)->bufsiz > 0) {
 			KNH_SETv(ctx, DP(o)->ba, new_Bytes(ctx, DP(o)->bufsiz));
 			DP(o)->buf = (char*)(DP(o)->ba)->buf;
@@ -75,7 +76,7 @@ KNHAPI(InputStream*) new_InputStream__FILE(Ctx *ctx, String *urn, FILE *fp, knh_
 /* ======================================================================== */
 /* [methods] */
 
-Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn)
+Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn, String *m)
 {
 	knh_bytes_t fname = knh_String_tobytes(urn);
 	knh_index_t loc = knh_bytes_index(fname, ':');
@@ -86,8 +87,11 @@ Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn)
 		DP(o)->driver = konoha_getIODriver(ctx, knh_bytes_first(fname, loc));
 	}
 	KNH_SETv(ctx, DP(o)->urn, new_String(ctx, fname, NULL));
-	DP(o)->fd = DP(o)->driver->fopen(ctx, fname, "r");
+	char *mode = "r";
+	if(IS_NOTNULL(m)) mode = knh_String_tochar(m);
+	DP(o)->fd = DP(o)->driver->fopen(ctx, fname, mode);
 	if(DP(o)->fd != -1) {
+		KNH_SETv(ctx, DP(o)->urn, urn);
 		DP(o)->bufsiz = DP(o)->driver->bufsiz;
 		if(DP(o)->bufsiz > 0) {
 			KNH_SETv(ctx, DP(o)->ba, new_Bytes(ctx, DP(o)->bufsiz));
@@ -98,6 +102,7 @@ Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn)
 		}
 		DP(o)->bufpos = 0;
 		DP(o)->bufend = 0;  /* empty */
+		DP(o)->driver->finit(ctx, (Object*)o, mode);
 	}
 	else {
 		DP(o)->driver = konoha_getDefaultIODriver();
