@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -35,7 +35,6 @@
 extern "C" {
 #endif
 
-
 /* ======================================================================== */
 /* [macros] */
 
@@ -44,7 +43,7 @@ extern "C" {
 /* ======================================================================== */
 /* [constructors] */
 
-Int* new_Int(Ctx *ctx, knh_int_t value)
+KNHAPI(Int*) new_Int(Ctx *ctx, knh_int_t value)
 {
 	knh_Int_t *b = (knh_Int_t*)new_hObject(ctx, FLAG_Int, CLASS_Int, CLASS_Int);
 	b->n.ivalue = value;
@@ -53,7 +52,7 @@ Int* new_Int(Ctx *ctx, knh_int_t value)
 
 /* ------------------------------------------------------------------------ */
 
-Int* new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_int_t value)
+KNHAPI(Int*) new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_int_t value)
 {
 	knh_Int_t *b = (knh_Int_t*)new_hObject(ctx, FLAG_Int, CLASS_Int, cid);
 	b->n.ivalue = value;
@@ -62,15 +61,11 @@ Int* new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_int_t value)
 
 /* ------------------------------------------------------------------------ */
 
-Int *new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value)
+KNHAPI(Int*) new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value)
 {
 	IntUnit *u = (IntUnit*)ctx->share->ClassTable[cid].cspec;
 	KNH_ASSERT(IS_IntUnit(u));
 
-//	if(knh_IntUnit_isBuffered(u, value)) {
-//		return DP(u)->bfarray[value - DP(u)->bfoffset];
-//	}
-
 	if(DP(u)->fchk(u, value)) {
 		Int *n = (knh_Int_t*)new_hObject(ctx, FLAG_Int, CLASS_Int, cid);
 		n->n.ivalue = value;
@@ -89,50 +84,6 @@ Int *new_IntX(Ctx *ctx, knh_class_t cid, knh_int_t value)
 	}
 }
 
-
-///* ------------------------------------------------------------------------ */
-//
-//char *knh_format_Int(char *buf, size_t bufsiz, Int *n)
-//{
-//	IntUnit *u = (IntUnit*)ctx->share->ClassTable[n->h.cid].cspec;
-//	KNH_ASSERT(IS_IntUnit(u));
-//	DP(u)->ffmt(u, buf, bufsiz, n->value);
-//	return buf;
-//}
-
-///* ======================================================================== */
-///* [utils] */
-//
-//
-//knh_int_t knh_int_max(knh_int_t a, knh_int_t b)
-//{
-//	return (a > b) ? a : b;
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//
-//knh_int_t knh_int_min(knh_int_t a, knh_int_t b)
-//{
-//	return (a < b) ? a : b;
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//
-//knh_uint_t knh_uint_max(knh_uint_t a, knh_uint_t b)
-//{
-//	return (a > b) ? a : b;
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//
-//knh_uint_t knh_uint_min(knh_uint_t a, knh_uint_t b)
-//{
-//	return (a < b) ? a : b;
-//}
-
 /* ======================================================================== */
 
 knh_int_t knh_IntNULL_toint(Int *v)
