@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -133,14 +133,14 @@ knh_mparam_t knh_MethodField_param(MethodField *o, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-knh_type_t knh_Method_rtype(Ctx *ctx, knh_class_t cid, Method *mtd)
+knh_type_t knh_Method_rtype(Ctx *ctx, Method *mtd, knh_class_t cid)
 {
 	return knh_pmztype_totype(ctx, DP(mtd)->mf->rtype, cid);
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_type_t knh_Method_ptype(Ctx *ctx, knh_class_t cid, Method *o, size_t n)
+knh_type_t knh_Method_ptype(Ctx *ctx, Method *o, knh_class_t cid, size_t n)
 {
 	return knh_pmztype_totype(ctx, knh_MethodField_pztype(DP(o)->mf, n), cid);
 }
@@ -758,7 +758,7 @@ Method *konoha_lookupMethod(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
 
 Method *konoha_lookupFormatter(Ctx *ctx, knh_class_t cid, knh_methodn_t mn)
 {
-	KNH_ASSERT(METHODN_IS_MOVTEXT(mn));
+	DBG2_ASSERT(METHODN_IS_MOVTEXT(mn));
 	knh_hcode_t h = ((((knh_hcode_t)cid) << (sizeof(knh_class_t) * 8)) + mn) % ctx->cachesize;
 	Method *mtd = ctx->fmtCache[h];
 	if(mtd != NULL) {
