@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -92,6 +92,15 @@ void knh_System_setProperty(Ctx *ctx, System *o, String *key, Any *value)
 
 /* ------------------------------------------------------------------------ */
 
+KNHAPI(void) konoha_setSystemPropertyText(Ctx *ctx, char *key, char *value)
+{
+	String *k = new_String__T(ctx, key);
+	String *v = new_String__T(ctx, value);
+	knh_DictMap_set(ctx, DP(ctx->sys)->props, k, UP(v));
+}
+
+/* ------------------------------------------------------------------------ */
+
 knh_bool_t konoha_initcheck(void)
 {
 #ifdef KONOHA_SAFEMODE
