@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -229,7 +229,7 @@ void knh_Asm_finish(Ctx *ctx, Asm *abr)
 
 	if(knh_Asm_isCancelled(abr)) {
 		if(DP(mtd)->mn != METHODN_lambda) {
-			knh_Asm_perrorMTD(ctx, abr, KMSG_TOABSTRACT, DP(mtd)->cid, DP(mtd)->mn);
+			knh_Asm_perror(ctx, abr, KERR_DWARN, "defined %C.%M as an abstract method", DP(mtd)->cid, DP(mtd)->mn);
 		}
 		knh_Method_toAbstract(ctx, mtd);
 	}
@@ -268,7 +268,7 @@ static knh_rtti_t RuntimeNullCheck [] = {
 static
 void knh_rtti_nullChecked(Asm *abr, size_t n)
 {
-	if(0 <= n && n < (sizeof(knh_rtti_t) * 8)) {
+	if(/*0 <= n &&*/ n < (sizeof(knh_rtti_t) * 8)) {
 		DP(abr)->nnrtti = DP(abr)->nnrtti | RuntimeNullCheck[n];
 	}
 }
@@ -278,7 +278,7 @@ void knh_rtti_nullChecked(Asm *abr, size_t n)
 static
 void knh_rtti_nullable(Asm *abr, size_t n)
 {
-	if(0 <= n && n < (sizeof(knh_rtti_t) * 8)) {
+	if(/*0 <= n &&*/ n < (sizeof(knh_rtti_t) * 8)) {
 		DP(abr)->nnrtti = DP(abr)->nnrtti & (~RuntimeNullCheck[n]);
 	}
 }
@@ -376,18 +376,7 @@ void KNH_ASM_SETLINE(Ctx *ctx, Asm *abr, int line)
 
 #define sfi_(n)    ((knh_sfi_t)(n))
 
-#define IS_BOOL(t)  (t == NNTYPE_Boolean)
-
-#define IS_INT(t)  (t == NNTYPE_Int || t == TYPE_Int)
-#define IS_NNINT(t)  (t == NNTYPE_Int)
-#define IS_NAINT(t)  (t == TYPE_Int)
-
-#define IS_FLOAT(t)  (t == NNTYPE_Float || t == TYPE_Float)
-#define IS_NNFLOAT(t)  (t == NNTYPE_Float)
-#define IS_NAFLOAT(t)  (t == TYPE_Float)
-
 #define IS_VAL(t) (IS_BOOL(t)||IS_INT(t)||IS_FLOAT(t))
-#define IS_NNVAL(t) (IS_BOOL(t)||IS_NNINT(t)||IS_NNFLOAT(t))
 #define IS_ANY(t) (CLASS_type(t) == CLASS_Any || CLASS_type(t) == CLASS_Object)
 
 /* ------------------------------------------------------------------------ */
@@ -450,21 +439,21 @@ static
 void KNH_ASM_SMOVx(Ctx *ctx, Asm *abr, knh_type_t atype, int a, knh_type_t btype, knh_sfx_t bx)
 {
 #ifdef KNH_USING_UNBOXFIELD
-	if(btype == NNTYPE_Int) {
+	if(IS_ubxint(btype)) {
 		KNH_ASM_MOVxi_(ctx, abr, sfi_(a), bx);
 		if(!IS_NNTYPE(atype)) {
 			KNH_ASM_NNBOX_(ctx, abr, sfi_(a), CLASS_type(btype));
 		}
 		return;
 	}
-	if(btype == NNTYPE_Float) {
+	if(IS_ubxfloat(btype)) {
 		KNH_ASM_MOVxf_(ctx, abr, sfi_(a), bx);
 		if(!IS_NNTYPE(atype)) {
 			KNH_ASM_NNBOX_(ctx, abr, sfi_(a), CLASS_type(btype));
 		}
 		return;
 	}
-	if(btype == NNTYPE_Boolean) {
+	if(IS_ubxboolean(btype)) {
 		KNH_ASM_MOVxb_(ctx, abr, sfi_(a), bx);
 		if(!IS_NNTYPE(atype)) {
 			KNH_ASM_NNBOX_(ctx, abr, sfi_(a), CLASS_type(btype));
@@ -504,13 +493,13 @@ void KNH_ASM_SMOV(Ctx *ctx, Asm *abr, knh_type_t atype, int a, Token *tkb)
 				knh_rtti_nullChecked(abr, b);
 				btype = NNTYPE_cid(btype);
 			}
-			if(IS_BOOL(btype) || IS_NNINT(btype) || IS_NNFLOAT(btype)) {
+			if(IS_ubxtype(btype)) {
 				KNH_ASM_MOVn_(ctx, abr, sfi_(a), sfi_(b));
 				if(IS_ANY(atype)) {
 					KNH_ASM_NNBOX(ctx, abr, btype, a);
 				}
 			}
-			else if(IS_NAINT(btype) || IS_NAFLOAT(btype)) {
+			else if(IS_bxint(btype) || IS_bxfloat(btype)) {
 				KNH_ASM_MOVa_(ctx, abr, sfi_(a), sfi_(b));
 				if(IS_ANY(atype)) {
 					KNH_ASM_NNBOX(ctx, abr, btype, a);
@@ -585,8 +574,8 @@ void KNH_ASM_XMOVx(Ctx *ctx, Asm *abr, knh_type_t atype, knh_sfx_t ax, knh_type_
 	if(IS_NNTYPE(atype) && !IS_NNTYPE(btype)) {
 		KNH_ASM_ISNULLx_(ctx, abr, bx);
 	}
-	if(atype == NNTYPE_Int) {
-		if(btype == NNTYPE_Int) {
+	if(IS_ubxint(atype)) {
+		if(IS_ubxint(btype)) {
 			KNH_ASM_XMOVxi_(ctx, abr, ax, bx);
 		}
 		else {
@@ -594,8 +583,8 @@ void KNH_ASM_XMOVx(Ctx *ctx, Asm *abr, knh_type_t atype, knh_sfx_t ax, knh_type_
 		}
 		return;
 	}
-	if(atype == NNTYPE_Float) {
-		if(btype == NNTYPE_Float) {
+	if(IS_ubxfloat(atype)) {
+		if(IS_ubxfloat(btype)) {
 			KNH_ASM_XMOVxf_(ctx, abr, ax, bx);
 		}
 		else {
@@ -603,15 +592,15 @@ void KNH_ASM_XMOVx(Ctx *ctx, Asm *abr, knh_type_t atype, knh_sfx_t ax, knh_type_
 		}
 		return;
 	}
-	if(atype == NNTYPE_Boolean && btype == NNTYPE_Boolean) {
+	if(IS_ubxboolean(atype) && IS_ubxboolean(btype)) {
 		KNH_ASM_XMOVxb_(ctx, abr, ax, bx);
 		return;
 	}
-	if(atype == TYPE_Int && btype == NNTYPE_Int) {
+	if(IS_bxint(atype) && IS_ubxint(btype)) {
 		KNH_ASM_XMOVxBXi_(ctx, abr, ax, bx, CLASS_type(btype));
 		return;
 	}
-	if(atype == TYPE_Int && btype == NNTYPE_Int) {
+	if(IS_bxfloat(atype) && IS_ubxfloat(btype)) {
 		KNH_ASM_XMOVxBXf_(ctx, abr, ax, bx, CLASS_type(btype));
 		return;
 	}
@@ -631,15 +620,15 @@ void KNH_ASM_XMOV(Ctx *ctx, Asm *abr, knh_type_t atype, int a, size_t an, Token
 		case TT_CONST: {
 			Object *v = DP(tkb)->data;
 #ifdef KNH_USING_UNBOXFIELD
-			if(atype == NNTYPE_Int) {
+			if(IS_ubxint(atype)) {
 				KNH_ASM_XMOVoi_(ctx, abr, ax, v);
 				break;
 			}
-			if(atype == NNTYPE_Float) {
+			if(IS_ubxfloat(atype)) {
 				KNH_ASM_XMOVof_(ctx, abr, ax, v);
 				break;
 			}
-			if(atype == NNTYPE_Boolean) {
+			if(IS_ubxboolean(atype)) {
 				KNH_ASM_XMOVob_(ctx, abr, ax, v);
 				break;
 			}
@@ -655,15 +644,15 @@ void KNH_ASM_XMOV(Ctx *ctx, Asm *abr, knh_type_t atype, int a, size_t an, Token
 				knh_rtti_nullChecked(abr, b);
 			}
 #ifdef KNH_USING_UNBOXFIELD
-			if(atype == NNTYPE_Int) {
+			if(IS_ubxint(atype)) {
 				KNH_ASM_XMOVsi_(ctx, abr, ax, sfi_(b));
 				break;
 			}
-			if(atype == NNTYPE_Float) {
+			if(IS_ubxfloat(atype)) {
 				KNH_ASM_XMOVsf_(ctx, abr, ax, sfi_(b));
 				break;
 			}
-			if(atype == NNTYPE_Boolean) {
+			if(IS_ubxboolean(atype)) {
 				KNH_ASM_XMOVsb_(ctx, abr, ax, sfi_(b));
 				break;
 			}
@@ -698,15 +687,15 @@ void KNH_ASM_XMOV(Ctx *ctx, Asm *abr, knh_type_t atype, int a, size_t an, Token
 		}
 		case TT_DEFVAL: {
 #ifdef KNH_USING_UNBOXFIELD
-			if(atype == NNTYPE_Int) {
-				KNH_ASM_XMOVoi_(ctx, abr, ax, UP(KNH_INT0));
+			if(IS_ubxint(atype)) {
+				KNH_ASM_XMOVoi_(ctx, abr, ax, KNH_DEF(ctx, CLASS_type(atype)));
 				break;
 			}
-			if(atype == NNTYPE_Float) {
-				KNH_ASM_XMOVof_(ctx, abr, ax, UP(KNH_FLOAT0));
+			if(IS_ubxfloat(atype)) {
+				KNH_ASM_XMOVof_(ctx, abr, ax, KNH_DEF(ctx, CLASS_type(atype)));
 				break;
 			}
-			if(atype == NNTYPE_Boolean) {
+			if(IS_ubxfloat(atype)) {
 				KNH_ASM_XMOVob_(ctx, abr, ax, KNH_FALSE);
 				break;
 			}
@@ -821,7 +810,7 @@ void KNH_ASM_CALL(Ctx *ctx, Asm *abr, knh_type_t reqt, int sfpidx, Token *tkb, s
 	Method *mtd = DP(tkb)->mtd;
 	knh_type_t rtype = CLASS_Any;
 	if(IS_Method(mtd)) {
-		if(knh_Method_isFinal(mtd) || knh_Method_isConstructor(ctx, mtd)) {
+		if(knh_Method_isFinal(mtd) || knh_Method_isConstructor(ctx, mtd) || knh_Token_isSUPER(tkb)) {
 			KNH_ASM_SCALL_(ctx, abr, sfpidx, args + 2, UP(mtd));
 		}
 		else {
@@ -867,12 +856,15 @@ void KNH_ASM_NEW(Ctx *ctx, Asm *abr, knh_type_t reqt, int sfpidx, Token *tkb,
 /* ------------------------------------------------------------------------ */
 
 static
-void KNH_ASM_MAP(Ctx *ctx, Asm *abr, knh_type_t reqt, int sfpidx, Token *tkb, knh_type_t srct)
+void KNH_ASM_MAP(Ctx *ctx, Asm *abr, knh_type_t reqt, int sfpidx, Token *tkb, knh_type_t srct, int isNonNullCast)
 {
 	KNH_ASSERT(IS_Token(tkb));
 	KNH_ASSERT(sfpidx >= DP(abr)->stack);
 	Mapper *mpr = (Mapper*)DP(tkb)->data;
+
 	if(IS_Mapper(mpr)) {
+//		DBG2_P("reqt=%s%s tcid=%s srct=%s%s isNonNullCast=%d",
+//				TYPEQN(reqt), CLASSN(DP(mpr)->tcid), TYPEQN(srct), isNonNullCast);
 		if(knh_Mapper_isFinal(mpr)) {
 			if(IS_NNTYPE(srct)) {
 				if(DP(mpr)->scid == CLASS_Int && DP(mpr)->tcid == CLASS_Float) {
@@ -901,7 +893,9 @@ void KNH_ASM_MAP(Ctx *ctx, Asm *abr, knh_type_t reqt, int sfpidx, Token *tkb, kn
 		else {
 			if(IS_NNTYPE(srct)) {
 				KNH_ASM_MAP_(ctx, abr, sfpidx, DP(mpr)->tcid);
-				if(!knh_Mapper_isTotal(mpr)) srct = CLASS_type(srct);
+				if(!knh_Mapper_isTotal(mpr)) {
+					srct = CLASS_type(srct);
+				}
 			}
 			else {
 				KNH_ASM_MAPnc_(ctx, abr, sfpidx, DP(mpr)->tcid);
@@ -910,11 +904,11 @@ void KNH_ASM_MAP(Ctx *ctx, Asm *abr, knh_type_t reqt, int sfpidx, Token *tkb, kn
 	}
 	else {
 		KNH_ASSERT(SP(tkb)->tt == TT_MPR);
+		//DBG2_P("reqt=%s%s mprcid=%s srct=%s%s isNonNullCast=%d", TYPEQN(reqt), DP(tkb)->cid, TYPEQN(srct), isNonNullCast);
 		KNH_ASM_AMAP_(ctx, abr, sfpidx, DP(tkb)->cid);
 		srct = CLASS_type(srct);
 	}
-
-	if(!IS_NNTYPE(srct) && knh_Token_isNotNullType(tkb)) {
+	if(!IS_NNTYPE(srct) && isNonNullCast) {
 		KNH_ASM_NNMAP_(ctx, abr, sfpidx, DP(tkb)->cid);
 	}
 	else if(IS_NNTYPE(reqt)) {
@@ -922,7 +916,6 @@ void KNH_ASM_MAP(Ctx *ctx, Asm *abr, knh_type_t reqt, int sfpidx, Token *tkb, kn
 	}
 }
 
-
 /* ======================================================================== */
 /* [CONTROL] */
 
@@ -1070,11 +1063,11 @@ void TERMs_ASM_JIFF(Ctx *ctx, Stmt *stmt, size_t n, Asm *abr, knh_labelid_t labe
 	else {
 		if(TERMs_isCALLISNUL(stmt, n)) {
 			// o == null -> false  o == null;
-			TERMs_ASM_JIFNUL(ctx, DP(stmt)->stmts[n], 1, abr, label);
+			TERMs_ASM_JIFNN(ctx, DP(stmt)->stmts[n], 1, abr, label);
 			return;
 		}
 		else if(TERMs_isCALLISNN(stmt, n)) {
-			TERMs_ASM_JIFNN(ctx, DP(stmt)->stmts[n], 1, abr, label);
+			TERMs_ASM_JIFNUL(ctx, DP(stmt)->stmts[n], 1, abr, label);
 			return;
 		}
 		TERMs_asm(ctx, stmt, n, abr, NNTYPE_Boolean, DP(abr)->stack);
@@ -1089,16 +1082,16 @@ void TERMs_ASM_JIFF_LOOP(Ctx *ctx, Stmt *stmt, size_t n, Asm *abr, knh_labelid_t
 {
 	Token *tk = DP(stmt)->tokens[n];
 	if(IS_Token(tk) && SP(tk)->tt == TT_STACK /*TT_LOCAL ??*/) {
-		KNH_ASM_bJIFF_(ctx, abr, label, sfi_(DP(tk)->index));
+		KNH_ASM_bJIFF_LOOP_(ctx, abr, label, sfi_(DP(tk)->index));
 	}
 	else {
 		if(TERMs_isCALLISNUL(stmt, n)) {
 			// o == null -> false  o == null;
-			TERMs_ASM_JIFNUL(ctx, DP(stmt)->stmts[n], 1, abr, label);
+			TERMs_ASM_JIFNN(ctx, DP(stmt)->stmts[n], 1, abr, label);
 			return;
 		}
 		else if(TERMs_isCALLISNN(stmt, n)) {
-			TERMs_ASM_JIFNN(ctx, DP(stmt)->stmts[n], 1, abr, label);
+			TERMs_ASM_JIFNUL(ctx, DP(stmt)->stmts[n], 1, abr, label);
 			return;
 		}
 		TERMs_asm(ctx, stmt, n, abr, NNTYPE_Boolean, DP(abr)->stack);
@@ -1227,7 +1220,7 @@ int knh_StmtOP_checkConst(Ctx *ctx, Stmt *stmt, knh_methodn_t *mn, int swap)
 /* ------------------------------------------------------------------------ */
 
 static
-knh_int_t TERMs_integer(Stmt *stmt, size_t n)
+knh_int_t TERMs_int(Stmt *stmt, size_t n)
 {
 	Token *tk = DP(stmt)->tokens[n];
 	KNH_ASSERT(SP(tk)->tt == TT_CONST);
@@ -1292,7 +1285,7 @@ int knh_StmtOP_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx)
 		if(IS_OPSIM(mn)) {
 			if(knh_StmtOP_checkConst(ctx, stmt, &mn, /*swap*/ 1)) {
 				int a = TERMs_putSTACK(ctx, stmt, 1, abr, NNTYPE_Int, local + 1);
-				knh_int_t b = TERMs_integer(stmt, 2);
+				knh_int_t b = TERMs_int(stmt, 2);
 				switch(mn) {
 				case METHODN_opAdd:
 					KNH_ASM_iADDn_(ctx, abr, sfi_(sfpidx), sfi_(a), b);
@@ -1356,15 +1349,24 @@ int knh_StmtOP_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx)
 		else if(IS_OPASIM(mn)) {
 			if(knh_StmtOP_checkConst(ctx, stmt, &mn, /*swap*/ 0)) {
 				int a = TERMs_putSTACK(ctx, stmt, 1, abr, NNTYPE_Int, local + 1);
-				knh_int_t b = TERMs_integer(stmt, 2);
+				knh_int_t b = TERMs_int(stmt, 2);
 				switch(mn) {
 				case METHODN_opSub:
 					KNH_ASM_iSUBn_(ctx, abr, sfi_(sfpidx), sfi_(a), b);
 					return 1;
 				case METHODN_opDiv:
+					if(b == 0) {
+						b = 1;
+						knh_Asm_perror(ctx, abr, KERR_ERRATA, _("divided by zero: /0 ==> /1"));
+					}
 					KNH_ASM_iDIVn_(ctx, abr, sfi_(sfpidx), sfi_(a), b);
 					return 1;
+
 				case METHODN_opMod:
+					if(b == 0) {
+						b = 1;
+						knh_Asm_perror(ctx, abr, KERR_ERRATA, _("divided by zero: %0 ==> %1"));
+					}
 					KNH_ASM_iMODn_(ctx, abr, sfi_(sfpidx), sfi_(a), b);
 					return 1;
 				}
@@ -1461,6 +1463,10 @@ int knh_StmtOP_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx)
 					KNH_ASM_fSUBn_(ctx, abr, sfi_(sfpidx), sfi_(a), b);
 					return 1;
 				case METHODN_opDiv:
+					if(b == 0.0) {
+						b = 1.0;
+						knh_Asm_perror(ctx, abr, KERR_ERRATA, _("divided by zero: 0.0 ==> 1.0"));
+					}
 					KNH_ASM_fDIVn_(ctx, abr, sfi_(sfpidx), sfi_(a), b);
 					return 1;
 				}
@@ -1480,7 +1486,6 @@ int knh_StmtOP_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpidx)
 			return 0;
 		}
 	} /* CLASS_Float */
-
 	return 0;
 }
 
@@ -1511,6 +1516,68 @@ void knh_StmtCALL_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpid
 		KNH_ASM_AINVOKE_(ctx, abr, sfi_(local), (knh_ushort_t)DP(stmt)->size, UP(mtd));
 		return;
 	}
+	/* INSTRUCTION */
+	if(cid == CLASS_Array || cid == CLASS_IArray || cid == CLASS_FArray) {
+		if(DP(mtd)->mn == METHODN_get) {
+			int a = TERMs_putSTACK(ctx, stmt, 1, abr, NNTYPE_Array, local + 1);
+			if(TERMs_isCONST(stmt, 2)) {
+				knh_intptr_t n = (knh_intptr_t)TERMs_int(stmt, 2);
+				if(cid == CLASS_Array) {
+					KNH_ASM_AGETn_(ctx, abr, sfi_(sfpidx), sfi_(a), n);
+				}
+				else if(cid == CLASS_IArray) {
+					KNH_ASM_IAGETn_(ctx, abr, sfi_(sfpidx), sfi_(a), n);
+				}
+				else {
+					KNH_ASM_FAGETn_(ctx, abr, sfi_(sfpidx), sfi_(a), n);
+				}
+			}
+			else {
+				int an = TERMs_putSTACK(ctx, stmt, 2, abr, NNTYPE_Int, local + 2);
+				if(cid == CLASS_Array) {
+					KNH_ASM_AGET_(ctx, abr, sfi_(sfpidx), sfi_(a), sfi_(an));
+				}
+				else if(cid == CLASS_IArray) {
+					KNH_ASM_IAGET_(ctx, abr, sfi_(sfpidx), sfi_(a), sfi_(an));
+				}
+				else {
+					KNH_ASM_FAGET_(ctx, abr, sfi_(sfpidx), sfi_(a), sfi_(an));
+				}
+			}
+			return;
+		}
+		if(DP(mtd)->mn == METHODN_set) {
+			int a = TERMs_putSTACK(ctx, stmt, 1, abr, NNTYPE_Array, local + 1);
+			knh_type_t ptype = knh_Method_ptype(ctx, mtd, cid, 1);
+			int v = TERMs_putSTACK(ctx, stmt, 3, abr, ptype, local + 3);
+			if(TERMs_isCONST(stmt, 2)) {
+				knh_intptr_t n = (knh_intptr_t)TERMs_int(stmt, 2);
+				if(cid == CLASS_Array) {
+					KNH_ASM_ASETn_(ctx, abr, sfi_(v), sfi_(a), n);
+				}
+				else if(cid == CLASS_IArray) {
+					KNH_ASM_IASETn_(ctx, abr, sfi_(v), sfi_(a), n);
+				}
+				else {
+					KNH_ASM_FASETn_(ctx, abr, sfi_(v), sfi_(a), n);
+				}
+			}
+			else {
+				int an = TERMs_putSTACK(ctx, stmt, 2, abr, NNTYPE_Int, local + 2);
+				if(cid == CLASS_Array) {
+					KNH_ASM_ASET_(ctx, abr, sfi_(v), sfi_(a), sfi_(an));
+				}
+				else if(cid == CLASS_IArray) {
+					KNH_ASM_IASET_(ctx, abr, sfi_(v), sfi_(a), sfi_(an));
+				}
+				else {
+					KNH_ASM_FASET_(ctx, abr, sfi_(v), sfi_(a), sfi_(an));
+				}
+			}
+			return;
+		}
+	}/* INSTRUCTION*/
+
 	/* PEEPHOLE */
 	if(IS_Method(mtd) && knh_Method_isFinal(mtd) && TERMs_isSTACK(stmt, 1)) {
 		int a = DP(DP(stmt)->tokens[1])->index;
@@ -1527,7 +1594,6 @@ void knh_StmtCALL_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpid
 		KNH_ASM_FCALL_(ctx, abr, sfi_(local), (knh_ushort_t)DP(stmt)->size, sfi_(a), UP(mtd));
 		goto L_RTYPE;
 	}/*PEEPHOLE*/
-
 	for(i = 1; i < DP(stmt)->size; i++) {
 		knh_type_t reqt2 = knh_Method_reqtTERMs(mtd, cid, stmt, i);
 		TERMs_asm(ctx, stmt, i, abr, reqt2, local + i);
@@ -1536,7 +1602,7 @@ void knh_StmtCALL_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sfpid
 
 	L_RTYPE:;
 	knh_type_t rtype = DP(stmt)->type;
-	if(IS_NNVAL(rtype)) {
+	if(IS_ubxtype(rtype)) {
 		nMOVL(ctx, abr, local, sfpidx);
 	}
 	else {
@@ -1578,12 +1644,12 @@ void knh_StmtMAPCAST_asm(Ctx *ctx, Stmt *stmt, Asm *abr, knh_type_t reqt, int sf
 	int local = ASML(abr, sfpidx);
 	knh_type_t srct = TERMs_gettype(stmt, 1);
 	TERMs_asm(ctx, stmt, 1, abr, TYPE_Any, local);
-	if(knh_Stmt_isNNCAST(stmt)) {
-		KNH_ASM_NNMAP_(ctx, abr, local, DP(DP(stmt)->tokens[0])->cid);
-	}
-	else {
-		KNH_ASM_MAP(ctx, abr, reqt, local, DP(stmt)->tokens[0], srct);
-	}
+//	if(knh_Stmt_isNNCAST(stmt)) {
+//		KNH_ASM_NNMAP_(ctx, abr, local, DP(DP(stmt)->tokens[0])->cid);
+//	}
+//	else {
+		KNH_ASM_MAP(ctx, abr, reqt, local, DP(stmt)->tokens[0], srct, knh_Stmt_isNNCAST(stmt));
+//	}
 	MOVL(ctx, abr, local, sfpidx);
 }
 
@@ -1881,14 +1947,13 @@ void knh_StmtIF_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 /* ------------------------------------------------------------------------ */
 
 static
-Token *knh_StmtMETA_getLabel(Ctx *ctx, Stmt *o)
+Token *knh_StmtMETA_getLabelNULL(Ctx *ctx, Stmt *o)
 {
 	if(IS_DictMap(DP(o)->metaDictMap)) {
 		Token *tk = (Token*)knh_DictMap_get(ctx, DP(o)->metaDictMap, TS_ATlabel);
-		KNH_ASSERT(IS_Token(tk));
-		return tk;
+		if(IS_Token(tk)) return tk;
 	}
-	return (Token*)KNH_NULL;
+	return NULL;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1896,23 +1961,22 @@ Token *knh_StmtMETA_getLabel(Ctx *ctx, Stmt *o)
 static
 knh_labelid_t knh_Asm_pushLabelStack(Ctx *ctx, Asm *abr, Stmt *stmt)
 {
-	Token *ltk = knh_StmtMETA_getLabel(ctx, stmt);
-	if(IS_NULL(ltk)) {
+	Token *tkL = knh_StmtMETA_getLabelNULL(ctx, stmt);
+	if(tkL == NULL) {
 		char lbn[8];
 		knh_snprintf(lbn, sizeof(lbn), "%d", DP(abr)->llstep);
-		ltk = new_TokenCONST(ctx, FL(stmt), UP(new_String(ctx, B(lbn), NULL)));
-		SP(ltk)->tt = TT_LABEL;
+		tkL = new_TokenCONST(ctx, FL(stmt), UP(new_String(ctx, B(lbn), NULL)));
+		SP(tkL)->tt = TT_LABEL;
 	}
-	else {
-		DEBUG3("label=%s", sToken(ltk));
+	DP(tkL)->index = (DP(abr)->llstep)++;
+	knh_Array_add(ctx, DP(abr)->lstacks, UP(tkL));
+	{
+		knh_labelid_t labelid = knh_Asm_newLabelId(ctx, abr, tkL);
+		knh_labelid_t labelid2 = knh_Asm_newLabelId(ctx, abr, NULL);
+		DBG2_P("label %s (begin, end) = %d, %d", sToken(tkL), labelid, labelid2);
+		KNH_ASSERT(labelid + 1 == labelid2);
+		return labelid;
 	}
-	DP(ltk)->index = (DP(abr)->llstep)++;
-	knh_Array_add(ctx, DP(abr)->lstacks, UP(ltk));
-	knh_labelid_t labelid = knh_Asm_newLabelId(ctx, abr, ltk);
-	knh_labelid_t labelid2 = knh_Asm_newLabelId(ctx, abr, NULL);
-	DBG2_P("label L1, L2 = %d, %d", labelid, labelid2);
-	KNH_ASSERT(labelid + 1 == labelid2);
-	return labelid;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1922,31 +1986,35 @@ knh_labelid_t knh_Asm_stackLabelId(Ctx *ctx, Asm *abr, Stmt *stmt)
 {
 	size_t s = knh_Array_size(DP(abr)->lstacks);
 	if(s == 0) {
-		if(SP(stmt)->stt == STT_CONTINUE)
-			knh_Asm_perror(ctx, abr, KMSG_EABORT, "continue");
-		else
-			knh_Asm_perror(ctx, abr, KMSG_EABORT, "break");
+		knh_Asm_perror(ctx, abr, KERR_ERROR, _("don't use '%s' HERE"), knh_stmt_tochar(SP(stmt)->stt));
+		knh_Stmt_done(ctx, stmt);
 		return -1;
 	}
-	Token *tk = NULL;
-	if(DP(stmt)->size == 1) {
-		tk = DP(stmt)->tokens[0];
-	}
-	if(tk != NULL) {
-		int i;
-		for(i = s - 1; i >= 0; i--) {
-			Token *tk2 = (Token*)knh_Array_n(DP(abr)->lstacks, i);
-			if(knh_String_equals(DP(tk)->text, knh_String_tobytes(DP(tk2)->text))) {
-				tk = tk2;
+	else {
+		Token *tkL = NULL;
+		if(DP(stmt)->size == 1) {
+			tkL = DP(stmt)->tokens[0];
+			if(SP(tkL)->tt == TT_ASIS) tkL = NULL;
+		}
+		if(tkL != NULL) {
+			int i;
+			for(i = s - 1; i >= 0; i--) {
+				Token *tk2 = (Token*)knh_Array_n(DP(abr)->lstacks, i);
+				//DBG2_P("i=%d, %s %s", i, sToken(tkL), sToken(tk2));
+				if(knh_String_equals(DP(tkL)->text, knh_String_tobytes(DP(tk2)->text))) {
+					tkL = tk2;
+					break;
+				}
+			}
+			if(tkL == NULL) {
+				knh_Asm_perror(ctx, abr, KERR_EWARN, _("unknown label: %s"), sToken(tkL));
 			}
 		}
-		knh_Asm_perror(ctx, abr, KMSG_ULABEL, sToken(tk));
-		tk = NULL;
+		if(tkL == NULL) {
+			tkL = (Token*)knh_Array_n(DP(abr)->lstacks, s-1);
+		}
+		return knh_Asm_findLabelId(ctx, abr, tkL);
 	}
-	if(tk == NULL) {
-		tk = (Token*)knh_Array_n(DP(abr)->lstacks, s-1);
-	}
-	return knh_Asm_findLabelId(ctx, abr, tk);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1966,6 +2034,9 @@ void knh_StmtCONTINUE_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 	if(labelid != -1) {
 		KNH_ASM_JMP(ctx, abr, labelid);
 	}
+	else if(SP(stmt)->stt == STT_CONTINUE) {
+		KNH_ASM_PANIC(ctx, abr, "unknown continue label");
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1975,7 +2046,10 @@ void knh_StmtBREAK_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 {
 	knh_labelid_t labelid = knh_Asm_stackLabelId(ctx, abr, stmt);
 	if(labelid != -1) {
-		KNH_ASM_JMP(ctx, abr, labelid);
+		KNH_ASM_JMP(ctx, abr, labelid + 1);
+	}
+	else if(SP(stmt)->stt == STT_BREAK) {
+		KNH_ASM_PANIC(ctx, abr, "unknown break label");
 	}
 }
 
@@ -2096,7 +2170,7 @@ void knh_Asm_setFinallyStmt(Ctx *ctx, Asm *abr, Stmt *stmt)
 {
 	if(IS_NOTNULL(stmt)) {
 		if(IS_NOTNULL(DP(abr)->finallyStmt)) {
-			knh_Asm_perror(ctx, abr, KMSG_ETRY, NULL);
+			knh_Asm_perror(ctx, abr, KERR_ERROR, _("cannot use nested try statements"));
 			return;
 		}
 		KNH_SETv(ctx, DP(abr)->finallyStmt, stmt);
@@ -2208,6 +2282,14 @@ void knh_StmtERR_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 /* ======================================================================== */
 /* [PRINT] */
 
+static int knh_Asm_isDEBUG(Ctx *ctx, Asm *abr)
+{
+	if(knh_Method_isRelease(DP(abr)->mtd)) return 0;
+	return knh_Context_isDebug(ctx);
+}
+
+/* ------------------------------------------------------------------------ */
+
 static
 knh_methodn_t knh_Stmt_getMT(Ctx *ctx, Stmt *stmt, size_t n)
 {
@@ -2223,56 +2305,73 @@ knh_methodn_t knh_Stmt_getMT(Ctx *ctx, Stmt *stmt, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-static
-String *knh_Term_getNameNULL(Term *tm)
-{
-	Token *tk = (Token*)tm;
-	if(!IS_Token(tk)) {
-		return NULL;
-	}
-	else if(SP(tk)->tt == TT_NAME) {
-		DBG2_ASSERT(IS_String(DP(tk)->data));
-		return (String*)DP(tk)->data;
-	}
-	return NULL;
-}
-
-/* ------------------------------------------------------------------------ */
-
 void knh_StmtPRINT_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
 {
+	if(!knh_Asm_isDEBUG(ctx, abr) || DP(stmt)->size == 0) {
+		return ;
+	}
 	knh_flag_t flag = knh_StmtPRINT_flag(ctx, stmt);
-	if(DP(stmt)->size == 0) {
-		Object *o = (Object*)TS_EMPTY;
-		KNH_ASM_PMSG_(ctx, abr, flag | KNH_FLAG_PF_EOL, o);
+	knh_labelid_t lbskip = knh_Asm_newLabelId(ctx, abr, NULL);
+	KNH_ASM_SKIP_(ctx, abr, lbskip);
+	if(konoha_debugLevel() > 1) {
+		char buf[128];
+		knh_snprintf(buf, sizeof(buf), "[%s:%d]", FILEIDN(DP(abr)->fileid), DP(abr)->line);
+		KNH_ASM_PMSG_(ctx, abr, flag | KNH_FLAG_PF_BOL, UP(new_String(ctx, B(buf), NULL)));
 	}
-	else {
-		int i;
+	else if(flag != 0 ) {
+		KNH_ASM_PMSG_(ctx, abr, flag | KNH_FLAG_PF_BOL, UP(TS_EMPTY));
+	}
+	int i;
+	for(i = 0; i < DP(stmt)->size; i++) {
+		L_REDO:;
 		knh_flag_t mask = 0;
-		for(i = 0; i < DP(stmt)->size; i++) {
-			if(i == 0) {
-				mask |= KNH_FLAG_PF_BOL;
-			}
-			if(i == DP(stmt)->size - 1) {
-				mask |= KNH_FLAG_PF_EOL;
-			}
-			String *name = knh_Term_getNameNULL(DP(stmt)->terms[i]);
-			if(name != NULL) {
-				mask |= KNH_FLAG_PF_NAME;
-				KNH_ASM_PMSG_(ctx, abr, flag, UP(name));
-			}
-			knh_methodn_t mn = knh_Stmt_getMT(ctx, stmt, i);
-			TERMs_asm(ctx, stmt, i, abr, TYPE_Any, DP(abr)->stack);
-			KNH_ASM_P_(ctx, abr, flag | mask, mn, DP(abr)->stack);
-			mask = 0;
+		if(i == DP(stmt)->size - 1) {
+			mask |= KNH_FLAG_PF_EOL;
 		}
+		/* name= */ {
+			Token *tkn = DP(stmt)->tokens[i];
+			if(IS_Token(tkn) && knh_Token_isPNAME(tkn)) {
+				KNH_ASM_PMSG_(ctx, abr, flag | mask | KNH_FLAG_PF_NAME, DP(tkn)->data);
+				i++;
+				KNH_ASSERT(i < DP(stmt)->size);
+				goto L_REDO;
+			}
+		}
+		/* "literal"*/ {
+			Token *tkn = DP(stmt)->tokens[i];
+			if(IS_Token(tkn) && knh_Token_isCONST(tkn) && IS_String(DP(tkn)->data)) {
+				KNH_ASM_PMSG_(ctx, abr, flag | mask, DP(tkn)->data);
+				continue;
+			}
+		}
+		TERMs_asm(ctx, stmt, i, abr, TYPE_Any, DP(abr)->stack);
+		knh_methodn_t mn = knh_Stmt_getMT(ctx, stmt, i);
+		KNH_ASM_P_(ctx, abr, flag | mask, mn, DP(abr)->stack);
 	}
+	KNH_ASM_LLABEL(ctx, abr, lbskip);
 }
 
-
 /* ------------------------------------------------------------------------ */
 
-/* ------------------------------------------------------------------------ */
+void knh_StmtASSERT_asm(Ctx *ctx, Stmt *stmt, Asm *abr)
+{
+	if(!knh_StmtMETA_is(ctx, stmt, STEXT("@Release"))) {
+		if(!knh_Asm_isDEBUG(ctx, abr)) {
+			return ;
+		}
+	}
+	knh_labelid_t lbskip = knh_Asm_newLabelId(ctx, abr, NULL);
+	if(!knh_StmtMETA_is(ctx, stmt, STEXT("@Release"))) {
+		KNH_ASM_SKIP_(ctx, abr, lbskip);
+	}
+
+	/* if */
+	TERMs_ASM_JIFT(ctx, stmt, 0, abr, lbskip);
+	/*then*/
+	TERMs_asmBLOCK(ctx, stmt, 1, abr);
+	KNH_ASM_THROWs_(ctx, abr, UP(TS_AssertionException));
+	KNH_ASM_LLABEL(ctx, abr, lbskip);
+}
 
 /* ------------------------------------------------------------------------ */
 
@@ -2285,7 +2384,14 @@ void knh_Stmt_asmBLOCK(Ctx *ctx, Stmt *stmt, Asm *abr, int isIteration)
 		KNH_ASM_SETLINE(ctx, abr, SP(cur)->line);
 		switch(SP(cur)->stt) {
 		case STT_BLOCK:
-			knh_Stmt_asmBLOCK(ctx, DP(stmt)->stmts[0], abr, 1); break;
+			if(!knh_Stmt_isDEBUG(stmt)) {
+				knh_Stmt_asmBLOCK(ctx, DP(stmt)->stmts[0], abr, 1); break;
+			}
+			else {
+				if(knh_Asm_isDEBUG(ctx, abr)) {
+					knh_Stmt_asmBLOCK(ctx, DP(stmt)->stmts[0], abr, 1); break;
+				}
+			}
 		case STT_LET:
 			knh_StmtLET_asm(ctx, cur, abr, TYPE_void, DP(abr)->stack); break;
 		case STT_REGISTER:
@@ -2322,6 +2428,8 @@ void knh_Stmt_asmBLOCK(Ctx *ctx, Stmt *stmt, Asm *abr, int isIteration)
 			knh_StmtRETURN_asm(ctx, cur, abr); break;
 		case STT_PRINT:
 			knh_StmtPRINT_asm(ctx, cur, abr); break;
+		case STT_ASSERT:
+			knh_StmtASSERT_asm(ctx, cur, abr); break;
 		case STT_ERR:
 			knh_StmtERR_asm(ctx, cur, abr); break;
 
