@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -62,6 +62,10 @@ knh_flag_t knh_StmtCLASS_flag(Ctx *ctx, Stmt *o)
 		if(IS_NOTNULL(v)) {
 			flag |= KNH_FLAG_CF_SINGLETON;
 		}
+		v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Release"));
+		if(IS_NOTNULL(v)) {
+			flag |= KNH_FLAG_CF_RELEASE;
+		}
 	}
 	return flag;
 }
@@ -72,7 +76,7 @@ knh_flag_t knh_StmtCLASS_flag(Ctx *ctx, Stmt *o)
 static
 Mapper* new_Mapper__interface(Ctx *ctx, knh_class_t scid, knh_class_t tcid)
 {
-	return new_Mapper(ctx, KNH_FLAG_MMF_INTERFACE, scid, tcid, knh_Mapper_fInterface, KNH_NULL);
+	return new_Mapper(ctx, KNH_FLAG_MMF_INTERFACE, scid, tcid, knh_Mapper_fasis, KNH_NULL);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -151,24 +155,20 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 		KNH_ASSERT(ctx->share->ClassTable[cid].class_cid == NULL);
 	}
 	else {
-		knh_Asm_perror(ctx, abr, KMSG_EOVERRIDE, CLASSN(cid));
+		knh_Asm_perror(ctx, abr, KERR_ERROR, _("cannot redefine %C"), cid);
 		knh_Stmt_done(ctx, stmt);
 		return 0;
 	}
 
 	knh_class_t supcid = CLASS_Object;
 	if(SP(StmtCLASS_superclass(stmt))->tt != TT_ASIS) {
-		supcid = CLASS_Object;
-	}
-	else {
 		supcid = knh_NameSpace_getcid(ctx, ns, knh_Token_tobytes(ctx, StmtCLASS_superclass(stmt)));
 		if(supcid == CLASS_unknown) {
-			knh_Asm_perror(ctx, abr, KMSG_UCLASSN, NULL);
 			supcid = CLASS_Object;
-			knh_Token_perrata(ctx, StmtCLASS_superclass(stmt), CLASSN(supcid));
+			knh_Asm_perror(ctx, abr, KERR_ERRATA, _("unknown class: %s ==> %C"), sToken(StmtCLASS_superclass(stmt)), supcid);
 		}
 		else if(knh_class_isFinal(supcid)) {
-			knh_Asm_perror(ctx, abr, KMSG_EEXTENDS, CLASSN(supcid));
+			knh_Asm_perror(ctx, abr, KERR_ERROR, _("cannot extends %C: this class is final"), supcid);
 			knh_Stmt_done(ctx, stmt);
 			return 0;
 		}
@@ -180,18 +180,13 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	TC->cflag  = knh_StmtCLASS_flag(ctx, stmt);
 	TC->oflag  = KNH_FLAG_CF2OF(TC->cflag);
 	if(SP(StmtCLASS_instmt(stmt))->stt == STT_DONE) {
-		TC->bcid   = CLASS_Any;
+		TC->bcid   = CLASS_Any;    /* Glue */
 	}
 	else {
 		TC->bcid   = CLASS_Object;
 	}
 	TC->supcid = supcid;
-	if(supcid == CLASS_Object) {
-		TC->offset = 0;
-	}else {
-		KNH_ASSERT(supcid < cid);
-		TC->offset = ctx->share->ClassTable[supcid].bsize;
-	}
+	TC->offset = 0; /* will be extended in CLASS_typing */
 	KNH_ASSERT(TC->class_cid == NULL);
 	konoha_setClassName(ctx, cid, new_String(ctx, B(bufn), NULL));
 	KNH_INITv(TC->cstruct, new_ClassStruct0(ctx, 0, 8));
@@ -206,11 +201,11 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 			Token *tk = DP(stmt)->tokens[i];
 			knh_class_t icid = knh_NameSpace_getcid(ctx, ns, knh_Token_tobytes(ctx, tk));
 			if(icid == CLASS_unknown) {
-				knh_Token_perror(ctx, tk, KMSG_UCLASSN);
+				knh_Token_perror(ctx, tk, KERR_EWARN, _("unknown class: %s"), sToken(tk));
 				continue;
 			}
 			if(!knh_class_isInterface(icid)) {
-				knh_Token_perror(ctx, tk, KMSG_EIMPLEMENTS);
+				knh_Token_perror(ctx, tk, KERR_EWARN, _("cannot implements %C: this class is not @Interfac"), icid);
 				continue;
 			}
 			knh_class_addInterface(ctx, cid, icid);
@@ -280,16 +275,16 @@ int knh_StmtUIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	Token *tk = DP(stmt)->tokens[0];
 	knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 	knh_index_t loc = knh_bytes_rindex(name, '.');
-	if(loc != -1) { /* using math.Math */
+	if(loc != -1 && isupper(name.buf[loc+1])) { /* using math.Math */
 		if(knh_System_loadPackage(ctx, knh_bytes_first(name, loc))) {
 			knh_class_t newcid = konoha_getcid(ctx, name);
 			if(newcid == CLASS_unknown) {
-				knh_Token_perror(ctx, tk, KMSG_UCLASSN);
+				knh_Token_perror(ctx, tk, KERR_ERROR, _("unknown class: %s"), sToken(tk));
 				return 0;
 			}
 			knh_class_t oldcid = knh_NameSpace_getcid(ctx, ns, knh_bytes_last(name, loc+1));
 			if(oldcid != CLASS_unknown && newcid != oldcid) {
-				KNH_WARNING(ctx, "ovrriding.. %s => %s", CLASSN(oldcid), CLASSN(newcid));
+				KNH_WARNING(ctx, _("ovrriding.. %s => %s"), CLASSN(oldcid), CLASSN(newcid));
 			}
 			knh_NameSpace_setLocalName(ctx, ns, newcid);
 			return 1;
@@ -302,7 +297,7 @@ int knh_StmtUIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 			return 1;
 		}
 	}
-	knh_Token_perror(ctx, tk, KMSG_UPACKAGE);
+	knh_Token_perror(ctx, tk, KERR_ERROR, _("package not found: %s"), sToken(tk));
 	return 0;
 }
 
@@ -333,15 +328,14 @@ void knh_NameSpace_setTagName(Ctx *ctx, NameSpace *o, String *name, knh_class_t
 	KNH_ASSERT(IS_NameSpace(o));
 	KNH_ASSERT_cid(cid);
 	knh_DictSet_set(ctx, DP(o)->name2cidDictSet, name, (knh_uintptr_t)(cid+1));
-	knh_bytes_t n = knh_String_tobytes(name);
-	knh_index_t loc = knh_bytes_index(n, ':');
-	if(loc != -1) {
-		n = knh_bytes_last(n, loc+1);
-		if(isupper(n.buf[0])) {
-			TODO();
-			knh_DictSet_set(ctx, DP(o)->name2cidDictSet, new_String(ctx, n, name), (knh_uintptr_t)(cid+1));
-		}
-	}
+//	knh_bytes_t n = knh_String_tobytes(name);
+//	knh_index_t loc = knh_bytes_index(n, ':');
+//	if(loc != -1) {
+//		n = knh_bytes_last(n, loc+1);
+//		if(isupper(n.buf[0])) {
+//			knh_DictSet_set(ctx, DP(o)->name2cidDictSet, new_String(ctx, n, name), (knh_uintptr_t)(cid+1));
+//		}
+//	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -353,20 +347,21 @@ int knh_StmtXCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_class
 	Token *tkurn = DP(stmt)->tokens[1];
 
 	char bufcu[CLASSNAME_BUFSIZ];
-	knh_format_classurn(ctx, bufcu, sizeof(bufcu), bcid, knh_Token_tobytes(ctx, tkurn));
+	knh_format_classurn(ctx, bufcu, sizeof(bufcu), bcid, konoha_getURNAlias(ctx, knh_Token_tobytes(ctx, tkurn)));
+
 	knh_class_t cid = konoha_findcid(ctx, B(bufcu));
 	if(cid == bcid || cid == CLASS_unknown) {
-		knh_Token_perror(ctx, tkurn, KMSG_UPATH);
+		knh_Token_perror(ctx, tkurn, KERR_EWARN, _("resource not found: %s"), sToken(tkurn));
 		cid = bcid;
 	}
 
 	knh_class_t oldcid = knh_NameSpace_getcid(ctx, ns, knh_Token_tobytes(ctx, tkclassn));
 	if(oldcid != CLASS_unknown && cid != oldcid) {
-		KNH_WARNING(ctx, "Overriding %s", knh_Token_tobytes(ctx, tkclassn));
+		KNH_WARNING(ctx, _("overriding %s"), knh_Token_tobytes(ctx, tkclassn));
 	}
 
 	KNH_ASSERT(IS_String(DP(tkclassn)->data));
-	knh_NameSpace_setTagName(ctx, ns, (String*)DP(tkclassn)->data, cid);
+	knh_NameSpace_setTagName(ctx, ns, DP(tkclassn)->text, cid);
 	return 1;
 }
 
@@ -405,12 +400,12 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 		knh_index_t loc = knh_bytes_rindex(name, '.');
 		knh_class_t cid = knh_NameSpace_getcid(ctx, ns, knh_bytes_first(name, loc));
 		if(cid == CLASS_unknown) {
-			knh_Token_perror(ctx, tk, KMSG_UCLASSN);
+			knh_Token_perror(ctx, tk, KERR_ERROR, _("unknown class: %s"), sToken(tk));
 			return 0;
 		}
-		knh_methodn_t mn = knh_tName_getMethodn(ctx, knh_bytes_last(name, loc+1), METHODN_NONAME);
+		knh_methodn_t mn = konoha_getMethodName(ctx, knh_bytes_last(name, loc+1), METHODN_NONAME);
 		if(mn == METHODN_NONAME) {
-			knh_Token_perror(ctx, tk, KMSG_UMETHODN);
+			knh_Token_perror(ctx, tk, KERR_ERROR, _("unknown method: %C.%s"), cid, sToken(tk));
 			return 0;
 		}
 		knh_NameSpace_setFuncClass(ctx, ns, mn, cid);
@@ -418,7 +413,7 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	else {
 		knh_class_t cid = knh_NameSpace_getcid(ctx, ns, name);
 		if(cid == CLASS_unknown) {
-			knh_Token_perror(ctx, tk, KMSG_UCLASSN);
+			knh_Token_perror(ctx, tk, KERR_ERROR, _("unknown class: %s"), sToken(tk));
 			return 0;
 		}
 		else {
@@ -431,9 +426,7 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 				if(!knh_Method_isStatic(mtd)) continue;
 				knh_NameSpace_setFuncClass(ctx, ns, DP(mtd)->mn, cid);
 				if(knh_Context_isVerbose(ctx)) {
-					char bufcm[CLASSNAME_BUFSIZ];
-					knh_format_cmethodn(ctx, bufcm, sizeof(bufcm), cid, DP(mtd)->mn);
-					KNH_NOTICE(ctx, "using %s", bufcm);
+					KNH_NOTICE(ctx, _("using %C.%M"), cid, DP(mtd)->mn);
 				}
 			}
 		}
@@ -451,41 +444,6 @@ int knh_StmtUMAPMAP_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	return 1;
 }
 
-///* ------------------------------------------------------------------------ */
-//
-//void knh_stmtvisit_declc__weave(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int level)
-//{
-//	//DEBUG_STMT(ctx, stmt);
-//	//Asm *mc = (Asm*)v;
-//	/* ['weave', 'ANY', 'METHODFN'] */
-//	fprintf(stdout, "weaving %s into %s..\n", sToken((Token*)stmt->terms[1]), sToken((Token*)stmt->terms[2]));
-//	Method *mtd = knh_Class_getMethod(ctx, CLASS_Amazon, METHODN_placeOrder);
-//	if(IS_Method(mtd)) {
-//		String *aspect = ((Token*)stmt->terms[1])->data;
-//		if(IS_NULL(aspect)) {
-//			knh_weave(ctx, NULL, mtd);
-//		}
-//		else if(knh_String_equals(aspect, STEXT("null"))) {
-//			knh_weave(ctx, NULL, mtd);
-//		}
-//		else if(knh_String_equals(aspect, STEXT("RBAC"))) {
-//			knh_weave(ctx, knh_security_RBAC, mtd);
-//		}
-//		else if(knh_String_equals(aspect, STEXT("Audit"))) {
-//			knh_weave(ctx, knh_security_Audit, mtd);
-//		}
-//		else {
-//			KNH_THROWf(ctx, "NoSuchAspect!!: %s", knh_String_tochar(aspect));
-//		}
-//	}
-//	else{
-//		DEBUG("what?");
-//	}
-//	knh_Stmt_done(ctx, stmt);
-//}
-
-/* ------------------------------------------------------------------------ */
-/* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 
 static
@@ -496,9 +454,28 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 
 	int isExpr = knh_stmt_isExpr(SP(stmt)->stt);
-	knh_methodn_t mt = SP(stmt)->stt == STT_MT ? METHODN__s : METHODN__k ;
+
+	knh_methodn_t mt = METHODN__k;
+	if(SP(stmt)->stt == STT_MT) {
+		Token *tk0 = DP(stmt)->tokens[0];
+		knh_methodn_t mn = konoha_getMethodName(ctx, knh_Token_tobytes(ctx, tk0), METHODN_NEWID);
+		char *name = FIELDN(METHODN_TOFIELDN(mn));
+		if(name[1] != 0) mt = METHODN__s;
+	}
+
 	if(isExpr) {
 		if(SP(stmt)->stt == STT_CALL1) {
+			Token *tk0 = DP(stmt)->tokens[0];
+			if(SP(tk0)->tt == TT_STR) {
+				if(isEval) {
+					knh_write(ctx, KNH_STDOUT, knh_String_tobytes(DP(tk0)->text));
+					knh_write_EOL(ctx, KNH_STDOUT);
+				}
+				return 1;
+			}
+			if(SP(tk0)->tt == TT_ESTR) {
+				mt = METHODN__s;
+			}
 			SP(stmt)->stt = STT_RETURN;
 		}
 		else if(SP(stmt)->stt == STT_LET) {
@@ -508,9 +485,10 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 			Stmt *newstmt = new_Stmt(ctx, 0, STT_RETURN);
 			knh_Stmt_add(ctx, newstmt, UP(stmt));
 			stmt = newstmt;
+			KNH_LPUSH(ctx, stmt);
 		}
 	}
-	KNH_LPUSH(ctx, stmt);
+
 	KNH_ASM_METHOD(ctx, abr, mtd, NULL, stmt, 0 /* isIteration */);
 	if(knh_Method_isAbstract(mtd) || SP(stmt)->stt == STT_ERR) {
 		KNH_LOCALBACK(ctx, lsfp);
@@ -527,7 +505,7 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 			KNH_MOV(ctx, lsfp[3].o, scr);
 			KNH_SCALL(ctx, lsfp, 2, mtd, 0/*args*/);
 			if(isExpr && !isVOID) {
-				//DBG_P("returning %s %lld", CLASSN(knh_Object_cid(lsfp[2].o)), lsfp[2].ivalue);
+				//DBG2_P("returning %s %lld", CLASSNo(lsfp[2].o), lsfp[2].ivalue);
 				((Context*)ctx)->esp = lsfp+1;
 				knh_esp1_format(ctx, mt, KNH_STDOUT, KNH_NULL);
 				knh_write_EOL(ctx, KNH_STDOUT);
@@ -540,17 +518,20 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 		L_CATCH:;
 		KNH_PRINT_STACKTRACE(ctx, lsfp, 0);
 	}
+	KNH_LOCALBACK(ctx, lsfp);
 	return 1;
 }
 
 /* ------------------------------------------------------------------------ */
 
-int knh_Stmt_complie(Ctx *ctx, Stmt *stmt, String *nsname, int isEval)
+int knh_Stmt_compile(Ctx *ctx, Stmt *stmt, String *nsname, int isEval)
 {
 	Asm *abr = knh_Context_getAsm(ctx);
 	NameSpace *ns = knh_Context_setNameSpace(ctx, nsname);
-	Stmt *cur = stmt;
+	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+	KNH_LPUSH(ctx, stmt);
 
+	Stmt *cur = stmt;
 	DP(abr)->fileid = SP(stmt)->fileid;
 	while(IS_Stmt(cur)) {
 		knh_stmt_t stt = SP(cur)->stt;
@@ -595,7 +576,7 @@ int knh_Stmt_complie(Ctx *ctx, Stmt *stmt, String *nsname, int isEval)
 			break;
 		}
 		if(res == 0) {
-			return 0;
+			goto L_FAILED;
 		}
 		cur = DP(cur)->next;
 	}
@@ -620,16 +601,16 @@ int knh_Stmt_complie(Ctx *ctx, Stmt *stmt, String *nsname, int isEval)
 			knh_Asm_initThisScript(ctx, abr);
 			tm = knh_StmtDECL_typing(ctx, cur, abr, ns);
 			break;
-		}
+			}
 		case STT_LET: {
 			knh_Asm_initThisScript(ctx, abr);
 			tm = knh_StmtLET_typing(ctx, cur, abr, ns, TYPE_void);
 			break;
-		}
+			}
 		}
 		if(tm == NULL) {
 			knh_Stmt_done(ctx, stmt);
-			return 0;
+			goto L_FAILED;
 		}
 		cur = DP(cur)->next;
 	}
@@ -659,13 +640,19 @@ int knh_Stmt_complie(Ctx *ctx, Stmt *stmt, String *nsname, int isEval)
 			break;
 		default:
 			if(!knh_Stmt_eval(ctx, cur, abr, ns, isEval)) {
-				return 0;
+				goto L_FAILED;
 			}
 		}
 		knh_Stmt_done(ctx, cur);
 		cur = DP(cur)->next;
 	}
+
+	KNH_LOCALBACK(ctx, lsfp);
 	return 1;
+
+	L_FAILED:;
+	KNH_LOCALBACK(ctx, lsfp);
+	return 0;
 }
 
 /* ======================================================================== */
@@ -679,21 +666,20 @@ Stmt *knh_InputStream_parseStmt(Ctx *ctx, InputStream *in)
 	KNH_LPUSH(ctx, in);
 	knh_Token_parse(ctx, tk, in);
 	DBG2_DUMP(ctx, tk, KNH_NULL, "tokens");
-	Stmt *stmt = new_StmtINSTMT(ctx, tk); // new_StmtDONE(ctx);
+	Stmt *stmt = new_StmtINSTMT(ctx, tk); // new_Stmt(ctx, 0, STT_DONE)(ctx);
 	return stmt;
 }
 
-
 /* ------------------------------------------------------------------------ */
 
-Stmt *knh_bytes_parseStmt(Ctx *ctx, knh_bytes_t kscript)
+Stmt *knh_bytes_parseStmt(Ctx *ctx, knh_bytes_t kscript, int fileid, int line)
 {
 	knh_cwb_t cwb = new_cwb(ctx);
 	knh_Bytes_write(ctx, cwb.ba, kscript);
 	knh_Bytes_putc(ctx, cwb.ba, ';');
 	InputStream *in = new_BytesInputStream(ctx, cwb.ba, cwb.pos, knh_Bytes_size(cwb.ba));
-	DP(in)->fileid = 0;
-	DP(in)->line = 0;
+	DP(in)->fileid = (knh_fileid_t)fileid;
+	DP(in)->line = line;
 	Stmt *stmt = knh_InputStream_parseStmt(ctx, in);
 	knh_cwb_clear(cwb);
 	return stmt;
@@ -709,8 +695,8 @@ void konohac_eval(Ctx *ctx, String *nsname, InputStream *in)
 	{
 		Stmt *stmt = knh_InputStream_parseStmt(ctx, in);
 		KNH_LPUSH(ctx, stmt);
-		DBG2_DUMP(ctx, stmt, KNH_NULL, "stmt");
-		knh_Stmt_complie(ctx, stmt, nsname, !knh_Context_isCompiling(ctx) /* isrun 1 */);
+		DBG_DUMP(ctx, stmt, KNH_NULL, "stmt");
+		knh_Stmt_compile(ctx, stmt, nsname, !knh_Context_isCompiling(ctx) /* isrun 1 */);
 	}
 	KNH_LOCALBACK(ctx, lsfp);
 	return;
@@ -799,7 +785,7 @@ int knh_NameSpace_loadScript(Ctx *ctx, NameSpace *ns, knh_bytes_t fpath, int isE
 	if(isEval) {
 		knh_Asm_openlib(ctx, knh_Context_getAsm(ctx), fpath);
 	}
-	knh_Stmt_complie(ctx, stmt, DP(ns)->nsname, isEval);
+	knh_Stmt_compile(ctx, stmt, DP(ns)->nsname, isEval);
 	KNH_LOCALBACK(ctx, lsfp);
 	return 1;
 }
