@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -48,20 +48,6 @@ Stmt* new_Stmt(Ctx *ctx, knh_flag_t flag, knh_stmt_t stt)
 	return o;
 }
 
-/* ======================================================================== */
-/* [DONE] */
-
-Stmt* new_StmtDONE(Ctx *ctx)
-{
-	return new_Stmt(ctx, 0, STT_DONE);
-	//	if(tglobalStmtDONE == NULL) {
-//		tglobalStmtDONE = (Object*)new_Stmt(ctx, 0, STT_DONE);
-//		KNH_TSTATICOBJ(&tglobalStmtDONE);
-//	}
-//	return (Stmt*)tglobalStmtDONE;
-}
-
-
 /* ======================================================================== */
 /* [ERR] */
 
@@ -80,7 +66,7 @@ void knh_Stmt_toERR(Ctx *ctx, Stmt *stmt, Term *tm)
 		line =  SP((Stmt*)tm)->line;
 	}
 	{
-		char buf[512];
+		char buf[256];
 		knh_snprintf(buf, sizeof(buf), "Script!!: running errors at %s:%d", FILEIDN(SP(stmt)->fileid), SP(stmt)->line);
 		KNH_SETv(ctx, DP(stmt)->errMsg, new_String(ctx, B(buf), NULL));
 		KNH_SETv(ctx, DP(stmt)->next, KNH_NULL);
@@ -116,7 +102,7 @@ void knh_Stmt_add(Ctx *ctx, Stmt *o, Term *tm)
 	}
 	KNH_SETv(ctx, DP(o)->terms[DP(o)->size], tm);
 	DP(o)->size++;
-	if(IS_Stmt((Stmt*)tm)) {
+	if(IS_Stmt(tm)) {
 		Stmt *stmt = (Stmt*)tm;
 		DP(o)->line_end = SP(stmt)->line;
 		if(SP(o)->line == 0) {
@@ -147,6 +133,7 @@ Stmt *knh_Stmt_tail(Stmt *o)
 {
 	Stmt *tail = o;
 	while(IS_NOTNULL(DP(tail)->next)) {
+		//DBG_P("stt=%s", knh_stmt_tochar(SP(tail)->stt));
 		tail = DP(tail)->next;
 	}
 	return tail;
@@ -157,16 +144,11 @@ Stmt *knh_Stmt_tail(Stmt *o)
 Stmt* knh_StmtNULL_tail_append(Ctx *ctx, Stmt *o, Stmt *stmt)
 {
 	if(o == NULL) {
-		if(SP(stmt)->stt != STT_DONE) {
-			return stmt;
-		}
-		return NULL;
+		return stmt;
 	}
 	else {
-		if(SP(stmt)->stt != STT_DONE) {
-			Stmt *tail = knh_Stmt_tail(o);
-			KNH_SETv(ctx, DP(tail)->next, stmt);
-		}
+		Stmt *tail = knh_Stmt_tail(o);
+		KNH_SETv(ctx, DP(tail)->next, stmt);
 		return o;
 	}
 }
@@ -179,8 +161,6 @@ knh_bool_t knh_Stmt_hasMeta(Stmt *o)
 	return (IS_bDictMap(DP(o)->metaDictMap));
 }
 
-
-
 /* ------------------------------------------------------------------------ */
 
 knh_flag_t knh_StmtMETHOD_flag(Ctx *ctx, Stmt *o)
@@ -214,19 +194,6 @@ knh_flag_t knh_Stmt_metaflag__field(Ctx *ctx, Stmt *b)
 	return 0;
 }
 
-///* ------------------------------------------------------------------------ */
-//
-//knh_bool_t knh_StmtMETA_istime(Stmt *o)
-//{
-//	if(IS_NOTNULL(DP(o)->metaDictMap)) {
-//		Object *v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Time"));
-//		if(IS_NOTNULL(v)) {
-//			return 1;
-//		}
-//	}
-//	return 0;
-//}
-
 /* ------------------------------------------------------------------------ */
 
 knh_flag_t knh_StmtPRINT_flag(Ctx *ctx, Stmt *o)
@@ -244,19 +211,6 @@ knh_flag_t knh_StmtPRINT_flag(Ctx *ctx, Stmt *o)
 /* ======================================================================== */
 /* [Annotation] */
 
-//knh_bool_t knh_StmtMETA_isHistoric(Stmt *o)
-//{
-//	if(IS_NOTNULL(DP(o)->metaDictMap)) {
-//		Object *v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("Historic"));
-//		if(IS_NOTNULL(v)) return 1;
-////		v = knh_DictMap_get__b(ctx, DP(o)->metaDictMap, STEXT("historic"));
-////		if(IS_NOTNULL(v)) return 1;
-//	}
-//	return 0;  /* @Historic */
-//}
-
-/* ------------------------------------------------------------------------ */
-
 knh_bool_t knh_StmtMETA_isOverride(Ctx *ctx, Stmt *o)
 {
 	if(IS_DictMap(DP(o)->metaDictMap)) {
@@ -266,6 +220,17 @@ knh_bool_t knh_StmtMETA_isOverride(Ctx *ctx, Stmt *o)
 	return 0;  /* @Historic */
 }
 
+/* ------------------------------------------------------------------------ */
+
+int knh_StmtMETA_is(Ctx *ctx, Stmt *stmt, knh_bytes_t name)
+{
+	if(IS_DictMap(DP(stmt)->metaDictMap)) {
+		Object *v = knh_DictMap_get__b(ctx, DP(stmt)->metaDictMap, name);
+		if(IS_NOTNULL(v)) return 1;
+	}
+	return 0;
+}
+
 /* ======================================================================== */
 /* [movabletext] */
 
