@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -35,9 +35,6 @@
 extern "C" {
 #endif
 
-#undef DEBUG3
-#define DEBUG3(fmt, ...)
-
 /* ======================================================================== */
 /* [data] */
 
@@ -47,29 +44,13 @@ extern "C" {
 /* ======================================================================== */
 /* [util] */
 
-#ifndef KMSG_EINDENT
-#define KMSG_EINDENT KMSG_ESYNTAX
-#endif
-
-#ifndef KMSG_UTOKEN
-#define KMSG_UTOKEN KMSG_ESYNTAX
-#endif
-
-#ifndef KMSG_WCHAR
-#define KMSG_WCHAR KMSG_ESYNTAX
-#endif
-
-#ifndef KMSG_IGBLOCK
-#define KMSG_IGBLOCK KMSG_ESYNTAX
-#endif
-
-/* ======================================================================== */
-/* [util] */
-
 static
-void knh_InputStream_perror(Ctx *ctx, InputStream *in, int pe, char *msg)
+void knh_InputStream_perror(Ctx *ctx, InputStream *in, int pe, char *fmt, ...)
 {
-	knh_perror(ctx, DP(in)->fileid, DP(in)->line, pe, msg);
+	va_list ap;
+	va_start(ap, fmt);
+	knh_vperror(ctx, DP(in)->fileid, DP(in)->line, pe, fmt, ap);
+	va_end(ap);
 }
 
 /* ======================================================================== */
@@ -117,7 +98,7 @@ Token *new_Token__NAME(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t t
 		goto NAME_PART;
 	}
 	else {
-		knh_InputStream_perror(ctx, in, KMSG_UTOKEN, (char*)t.buf);
+		knh_InputStream_perror(ctx, in, KERR_ERROR, _("unknown token: %B"), t);
 		return new_Token(ctx, flag, DP(in)->fileid, DP(in)->line, TT_ERR);
 	}
 
@@ -153,8 +134,7 @@ Token *new_Token__NAME(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t t
 				goto TOKEN_PART;
 			}
 			if(!isalnum(t.buf[i]) && t.buf[i] != ':') {
-				char cb[2] = {t.buf[i], 0};
-				knh_InputStream_perror(ctx, in, KMSG_WCHAR, cb);
+				knh_InputStream_perror(ctx, in, KERR_EWARN, _("used strange character: %B"), t);
 				goto TOKEN_PART;
 			}
 			if(islower(t.buf[i])) has_lower = 1;
@@ -178,8 +158,7 @@ Token *new_Token__NAME(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t t
 				goto NAME_PART;
 			}
 			else {
-				char cb[2] = {t.buf[i], 0};
-				knh_InputStream_perror(ctx, in, KMSG_WCHAR, cb);
+				knh_InputStream_perror(ctx, in, KERR_EWARN, _("used strange character: %B"), t);
 				goto TOKEN_PART;
 			}
 		}
@@ -214,8 +193,7 @@ Token *new_Token__NAME(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t t
 					continue;
 				}
 				else {
-					char cb[2] = {t.buf[i], 0};
-					knh_InputStream_perror(ctx, in, KMSG_WCHAR, cb);
+					knh_InputStream_perror(ctx, in, KERR_EWARN, _("used strange character: %B"), t);
 					goto TOKEN_PART;
 				}
 			}
@@ -238,8 +216,7 @@ Token *new_Token__NAME(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t t
 
 	WLENGTH_PART: {
 		*p = 0;
-		knh_InputStream_perror(ctx, in, KMSG_WLENGTH, (char*)t.buf);
-		knh_perrata(ctx, DP(in)->fileid, DP(in)->line, (char*)t.buf, name);
+		knh_InputStream_perror(ctx, in, KERR_ERRATA, "too long name: '%B' => '%s'", t, name);
 	}
 
 	TOKEN_PART: {
@@ -308,8 +285,6 @@ Token *new_Token__buffer(Ctx *ctx, knh_token_t tt, knh_cwb_t tbuf, InputStream *
 	return o;
 }
 
-
-
 /* ------------------------------------------------------------------------ */
 #define knh_token_isNested(tt)  (tt == TT_BRACE || tt == TT_PARENTHESIS || tt == TT_BRANCET)
 
@@ -506,57 +481,90 @@ static
 void knh_Token_join(Ctx *ctx, Token *o)
 {
 	Array *a = (Array*)DP(o)->data;
-	if(IS_Array(a)) {
-		int i, size = knh_Array_size(a) - 1;
-		for(i = 0; i < size ; i++) {
-			Token *tk = (Token*)knh_Array_n(a, i);
-			if(SP(tk)->tt == TT_TYPEN) {
-				Token *tkn = (Token*)knh_Array_n(a, i+1);
-				if(SP(tkn)->tt == TT_LT) {  /* T<T> */
-					knh_Token_addTypeParam(ctx, tk, a, i+1);
-				}
-				if(SP(tkn)->tt == TT_PARENTHESIS) {
-					if(knh_Token_toClosureType(ctx, tk, tkn)) {
-						knh_Array_remove(ctx, a, i+1);
-					}
-				}
-				if(SP(tkn)->tt == TT_BRANCET && IS_NULL(DP(tkn)->data)) {
-					knh_Token_setArrayType(tk, 1);
-					DP(tk)->tt_next = DP(tkn)->tt_next;
-					if(DP(tkn)->tt_next == TT_NOT) {
-						knh_Token_setNotNullType(tk, 1);
-						DP(tk)->tt_next = DP((Token*)knh_Array_n(a, i+2))->tt_next;
-						knh_Array_remove(ctx, a, i+1);
-						knh_Array_remove(ctx, a, i+1);
-					}
-					else {
-						knh_Array_remove(ctx, a, i+1);
-					}
+	if(!IS_Array(a)) return;
+	int i, size = knh_Array_size(a) - 1;
+	for(i = 0; i < size ; i++) {
+		Token *tk =  (Token*)knh_Array_n(a, i);
+		Token *tkn = (Token*)knh_Array_n(a, i+1);
+		//DBG2_P("i=%d,%d tk=%s, tkn=%s", i, size, knh_token_tochar(tk->tt), knh_token_tochar(tkn->tt));
+		if(SP(tk)->tt == TT_TYPEN) {
+			if(SP(tkn)->tt == TT_LT) {  /* T<T> */
+				knh_Token_addTypeParam(ctx, tk, a, i+1);
+				i--; size = knh_Array_size(a) - 1;
+				continue;
+			}
+			else if(SP(tkn)->tt == TT_PARENTHESIS) { /* T(void) */
+				if(knh_Token_toClosureType(ctx, tk, tkn)) {
+					knh_Array_remove(ctx, a, i+1);
+					i--; size = knh_Array_size(a) - 1;
+					continue;
 				}
 			}
-			if(SP(tk)->tt == TT_NAME && ISB(knh_Token_tobytes(ctx, tk), "byte")) {
-				Token *tkn = (Token*)knh_Array_n(a, i+1);
-				if(SP(tkn)->tt == TT_BRANCET && IS_NULL(DP(tkn)->data)) {
-					SP(tk)->tt = TT_TYPEN;
-					KNH_SETv(ctx, DP(tk)->data, ctx->share->ClassTable[CLASS_Bytes].sname);
-					if(i>0) {
-						Token *tkp = (Token*)knh_Array_n(a, i-1);
-						DP(tkp)->tt_next = TT_TYPEN;
-					}
-					DP(tk)->tt_next = DP(tkn)->tt_next;
-					if(DP(tkn)->tt_next == TT_NOT) {
-						knh_Token_setNotNullType(tk, 1);
-						DP(tk)->tt_next = DP((Token*)knh_Array_n(a, i+2))->tt_next;
-						knh_Array_remove(ctx, a, i+1);
-						knh_Array_remove(ctx, a, i+1);
-					}
-					else {
-						knh_Array_remove(ctx, a, i+1);
-					}
-				}
+			else if(SP(tkn)->tt == TT_BRANCET && IS_NULL(DP(tkn)->data)) {
+				/* T[] */
+				knh_Token_setArrayType(tk, 1);
+				DP(tk)->tt_next = DP(tkn)->tt_next;
+				knh_Array_remove(ctx, a, i+1);
+				i--; size = knh_Array_size(a) - 1;
+				DBG2_P("T[]");
+				continue;
+			}
+			else if(SP(tkn)->tt == TT_NOT) {  /* T! */
+				knh_Token_setNotNullType(tk, 1);
+				DP(tk)->tt_next = DP(tkn)->tt_next;
+				knh_Array_remove(ctx, a, i+1);
+				i--; size = knh_Array_size(a) - 1;
+			}
+			else if(SP(tkn)->tt == TT_QUESTION) {  /* T? */
+				knh_Token_setNullableType(tk, 1);
+				DP(tk)->tt_next = DP(tkn)->tt_next;
+				knh_Array_remove(ctx, a, i+1);
+				i--; size = knh_Array_size(a) - 1;
 			}
 		}
+		else if(SP(tk)->tt == TT_NAME && ISB(knh_Token_tobytes(ctx, tk), "byte")) {
+			if(SP(tkn)->tt == TT_BRANCET && IS_NULL(DP(tkn)->data)) {
+				SP(tk)->tt = TT_TYPEN;
+				KNH_SETv(ctx, DP(tk)->data, ctx->share->ClassTable[CLASS_Bytes].sname);
+				if(i>0) {
+					Token *tkp = (Token*)knh_Array_n(a, i-1);
+					DP(tkp)->tt_next = TT_TYPEN;
+				}
+				DP(tk)->tt_next = DP(tkn)->tt_next;
+				knh_Array_remove(ctx, a, i+1);
+				i--; size = knh_Array_size(a) - 1;
+				continue;
+			}
+		}
+		else if(SP(tk)->tt == TT_STR || SP(tk)->tt == TT_TSTR || SP(tk)->tt == TT_TSTR) {
+			size_t next = i + 1;
+			if(SP(tkn)->tt != TT_STR && SP(tkn)->tt != TT_TSTR && SP(tk)->tt != TT_ESTR) {
+				continue;
+			}
+			knh_cwb_t cwb = new_cwb(ctx);
+			Token *tkp = tk;
+			knh_Bytes_write(ctx, cwb.ba, knh_String_tobytes(DP(tk)->text));
+			do {
+				tkn = (Token*)knh_Array_n(a, next);
+				if(SP(tkn)->tt != TT_STR && SP(tkn)->tt != TT_TSTR && SP(tk)->tt != TT_ESTR) {
+					break;
+				}
+				if(SP(tkp)->line < SP(tkn)->line) {
+					knh_Bytes_putc(ctx, cwb.ba, '\n');
+				}
+				knh_Bytes_write(ctx, cwb.ba, knh_String_tobytes(DP(tkn)->text));
+				tkp = tkn;
+				knh_Array_remove(ctx, a, next);
+			}while(next < knh_Array_size(a));
+			KNH_SETv(ctx, DP(tk)->data, new_String__cwb(ctx, cwb));
+			size = knh_Array_size(a) - 1;
+		} /* "a" "b" "c" */
+		else if(SP(tk)->tt == TT_FORMAT && DP(tk)->tt_next == TT_PARENTHESIS) {
+			SP(tk)->tt = TT_NAME;
+			KNH_SETv(ctx, DP(tk)->data, new_String__NAME(ctx, STEXT("format")));
+		}
 	}
+	//DBG2_DUMP(ctx, o, KNH_NULL, "joined");
 }
 
 /* ------------------------------------------------------------------------ */
@@ -574,7 +582,7 @@ knh_token_t knh_char_totoken(int ch)
 		case ',': return TT_COMMA;
 		case '"': return TT_STR;
 		case '\'': return TT_TSTR;
-		case '`': return TT_EVAL;
+		case '`': return TT_ESTR;
 	}
 	KNH_ASSERT(ch == -1);
 	return TT_ERR;
@@ -625,9 +633,7 @@ void knh_InputStream_skipBLOCK(Ctx *ctx, InputStream *in, int prev, Token *tk)
 	MAIN_PART:
 	while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
 		if(prev == '\n' && ch == endch) {
-			char buf[40];
-			knh_snprintf(buf, sizeof(buf), "lines: %d..%d", (int)SP(tk)->line, (int)DP(in)->line);
-			knh_InputStream_perror(ctx, in, KMSG_IGBLOCK, buf);
+			knh_InputStream_perror(ctx, in, KERR_DWARN, _("ignored this block from %d to %d lines"), (int)SP(tk)->line, (int)DP(in)->line);
 			return ;
 		}
 		if(prev == '/' && ch == '*') {
@@ -654,6 +660,19 @@ void knh_InputStream_skipBLOCK(Ctx *ctx, InputStream *in, int prev, Token *tk)
 	return;
 }
 
+/* ======================================================================== */
+
+static knh_token_t knh_Token_lastTT(Token *tk)
+{
+	if(IS_Array(DP(tk)->list)) {
+		return SP((Token*)knh_Array_last(DP(tk)->list))->tt;
+	}
+	else if(IS_Token(DP(tk)->data)) {
+		return SP((Token*)DP(tk)->data)->tt;
+	}
+	return TT_EOT;
+}
+
 /* ======================================================================== */
 /* [parse] */
 
@@ -718,7 +737,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				}
 			}
 			else {
-				knh_InputStream_perror(ctx, in, KMSG_ESYNTAX, "TOO MANY NESTED");
+				knh_InputStream_perror(ctx, in, KERR_ERROR, _("too many of tested blocks"));
 				knh_InputStream_skipBLOCK(ctx, in, prev, blocktk);
 				blocktk = NULL;
 			}
@@ -732,7 +751,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				tkl--;
 			}
 			else {
-				knh_InputStream_perror(ctx, in, KMSG_EMISMATCH, " ...}");
+				knh_InputStream_perror(ctx, in, KERR_ERROR, _("mismatched ...}"));
 				knh_InputStream_skipBLOCK(ctx, in, prev, blocktk);
 				while(tkl > 0) {
 					tks[tkl]->tt = TT_ERR;
@@ -751,7 +770,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				tkl--;
 			}
 			else {
-				knh_InputStream_perror(ctx, in, KMSG_EMISMATCH, " ...)");
+				knh_InputStream_perror(ctx, in, KERR_ERROR, _("mismatched ...)"));
 				knh_InputStream_skipBLOCK(ctx, in, prev, blocktk);
 				while(tkl > 0) {
 					tks[tkl]->tt = TT_ERR;
@@ -770,7 +789,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				tkl--;
 			}
 			else {
-				knh_InputStream_perror(ctx, in, KMSG_EMISMATCH, " ...]");
+				knh_InputStream_perror(ctx, in, KERR_ERROR, _("mismatched ...]"));
 				knh_InputStream_skipBLOCK(ctx, in, prev, blocktk);
 				while(tkl > 0) {
 					tks[tkl]->tt = TT_ERR;
@@ -821,22 +840,28 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 
 		case '%':
 			knh_Token_add_space(ctx, tks[tkl], &BOL, tbuf, in);
-			ch = knh_InputStream_getc(ctx, in);
-			if(isalnum(ch)) {
-				prev = '%';
-				knh_Bytes_putc(ctx, tbuf.ba, ch);
-				goto QNAME_PART;
+			//DBG2_P("IS EXTENDED QUOTE: %s", (equote) ? "yes" : "no");
+			if(equote == 0 && knh_Token_lastTT(tks[tkl]) < TT_NUM) {
+				equote = 1;
 			}
-			else if(ch == '%') {
-				knh_Bytes_putc(ctx, tbuf.ba, '%');
+
+			ch = knh_InputStream_getc(ctx, in);
+			prev = '%';
+			if(equote == 0 || ch == ' ' || ch == '\t') {
 				knh_Bytes_putc(ctx, tbuf.ba, '%');
+				equote = 1;
+				goto OP_PART_INLOOP;
+			}
+			else if(ch == '(') {  /* %(..) => format(..) */
+				knh_Bytes_write(ctx, tbuf.ba, STEXT("format"));
 				knh_Token_add_space(ctx, tks[tkl], &BOL, tbuf, in);
-				break;
+				equote = 0;
+				goto MAIN_PART_INLOOP;
 			}
 			else {
-				knh_Bytes_putc(ctx, tbuf.ba, '%');
-				knh_Token_add_space(ctx, tks[tkl], &BOL, tbuf, in);
-				goto MAIN_PART_INLOOP;
+				equote = 0;
+				knh_Bytes_putc(ctx, tbuf.ba, ch);
+				goto QNAME_PART;
 			}
 
 		case '|':
@@ -851,7 +876,10 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				goto MAIN_PART;
 			}
 
-			DBG2_P("IS EXTENDED QUOTE: %s", (equote) ? "yes" : "no");
+			//DBG2_P("IS EXTENDED QUOTE: %s", (equote) ? "yes" : "no");
+			if(equote == 0 && knh_Token_lastTT(tks[tkl]) < TT_NUM) {
+				equote = 1;
+			}
 
 			if(equote == 1) {
 				equote = 2; /*very dirty orz */
@@ -906,7 +934,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 
 		case '#':
 			knh_Token_add_space(ctx, tks[tkl], &BOL, tbuf, in);
-			goto LINE_COMMENT;
+			goto LINE_STRING;
 
 		case '.':
 			equote = 0;
@@ -976,7 +1004,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 		default:
 			equote = 0;
 			if(ch > 127) {
-				knh_InputStream_perror(ctx, in, KMSG_WASCII, NULL);
+				knh_InputStream_perror(ctx, in, KERR_DWARN, _("unexpected multi-byte character"));
 				ch = ' ';
 			}
 			knh_Bytes_putc(ctx, tbuf.ba, ch);
@@ -988,7 +1016,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 	NUM_PART:;  /* 0x0000 */
 	{
 		int unit = 0;
-		//DEBUG3("=> NUM_PART");
+		//DBG2_P("=> NUM_PART");
 		while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
 			NUM_PART_INLOOP:;
 			switch(ch) {
@@ -1050,7 +1078,6 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			}
 		}
 		knh_Token_padd(ctx, tks[tkl], &BOL, new_Token__buffer(ctx, TT_NUM, tbuf, in));
-		//knh_InputStream_perror(ctx, in, KMSG_WEOF, NULL);
 	}
 	goto MAIN_PART_INLOOP; /* EOF */
 
@@ -1090,7 +1117,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 
 	URN_PART:;
 	{
-		DEBUG3("=> URN_PART");
+		DBG2_P("=> URN_PART");
 		while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
 			URN_PART_INLOOP:;
 			switch(ch) {
@@ -1139,7 +1166,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				continue;
 			}
 			if(ch == '\n' || ch == '\r' || ch == EOF) {
-				knh_InputStream_perror(ctx, in, KMSG_WCHAR, "\\n");
+				knh_InputStream_perror(ctx, in, KERR_EWARN, _("suddenly ending?"));
 				knh_Token_padd(ctx, tks[tkl], &BOL, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 				goto MAIN_PART_INLOOP;
 			}
@@ -1208,7 +1235,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				continue;
 			}
 			if(ch == '\n' || ch == '\r' || ch == EOF) {
-				knh_InputStream_perror(ctx, in, KMSG_WCHAR, "\\n");
+				knh_InputStream_perror(ctx, in, KERR_EWARN, _("suddenly ending?"));
 				knh_Token_padd(ctx, tks[tkl], &BOL, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 				goto MAIN_PART_INLOOP;
 			}
@@ -1265,6 +1292,23 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 	}
 	goto L_EOF; /* EOF */
 
+	/** COMMENT **/
+	LINE_STRING:
+	{
+		ch = knh_InputStream_getc(ctx, in);
+		if(ch == '!') goto LINE_COMMENT; /* #! /usr/local/bin */
+		if(ch != ' ') goto LINE_STRING_INLOOP;
+		while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
+			LINE_STRING_INLOOP:
+			if(ch == '\n') {
+				knh_Token_padd(ctx, tks[tkl], &BOL, new_Token__buffer(ctx, TT_ESTR, tbuf, in));
+				goto MAIN_PART_INLOOP;
+			}
+			knh_Bytes_putc(ctx, tbuf.ba, ch);
+		}
+	}
+	goto L_EOF; /* EOF */
+
 	/** COMMENT **/
 	LINE_COMMENT:
 	{
@@ -1284,7 +1328,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				nest--;
 				if(nest == 0) goto MAIN_PART;
 			}else if(prev == '/' && ch == '*') {
-				DBG2_P("nesting /* ")
+				DBG2_P("nesting /* ");
 				nest++;
 			}
 			prev = ch;
@@ -1293,7 +1337,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 	goto L_EOF; /* EOF */
 
 	L_EOF:;
-	knh_InputStream_perror(ctx, in, KMSG_WEOF, NULL);
+	knh_InputStream_perror(ctx, in, KERR_EWARN, _("suddenly comes to end of file"));
 
 	L_JOIN:
 	knh_Token_add_space(ctx, tks[tkl], &BOL, tbuf, in);
