@@ -84,11 +84,11 @@ static char* knh_token_array_tochar[] = {
     ">>=",  /* rshifte */ 
     "+=",  /* adde */ 
     "-=",  /* sube */ 
-    "*=",  /* mule */ 
     "/=",  /* dive */ 
     "%=",  /* mode */ 
-    "|=",  /* bitore */ 
+    "*=",  /* mule */ 
     "&=",  /* bitande */ 
+    "|=",  /* bitore */ 
     "?\?=",  /* ALTLET */ 
     "?",  /* QUESTION */ 
     "..",  /* Itr */ 
@@ -112,8 +112,6 @@ static char* knh_token_array_tochar[] = {
     ">",  /* Gt */ 
     ">=",  /* Gte */ 
     "=~",  /* Match */ 
-    "|",  /* Lor */ 
-    "%%",  /* Fmt */ 
     "<<",  /* Lshift */ 
     ">>",  /* Rshift */ 
     "+",  /* Add */ 
@@ -122,6 +120,7 @@ static char* knh_token_array_tochar[] = {
     "%",  /* Mod */ 
     "*",  /* Mul */ 
     "&",  /* Land */ 
+    "|",  /* Lor */ 
     "~",  /* Lnot */ 
     "^",  /* Xor */ 
     "++",  /* Next */ 
@@ -130,7 +129,7 @@ static char* knh_token_array_tochar[] = {
     "NUM",  /* NUM */ 
     "STR",  /* STR */ 
     "TSTR",  /* TSTR */ 
-    "EVAL",  /* EVAL */ 
+    "ESTR",  /* ESTR */ 
     "LABEL",  /* LABEL */ 
     "METAN",  /* METAN */ 
     "PROPN",  /* PROPN */ 
@@ -214,11 +213,11 @@ static knh_methodn_t knh_token_array_tomethodn[] = {
     METHODN_NONAME,  /* rshifte */ 
     METHODN_NONAME,  /* adde */ 
     METHODN_NONAME,  /* sube */ 
-    METHODN_NONAME,  /* mule */ 
     METHODN_NONAME,  /* dive */ 
     METHODN_NONAME,  /* mode */ 
-    METHODN_NONAME,  /* bitore */ 
+    METHODN_NONAME,  /* mule */ 
     METHODN_NONAME,  /* bitande */ 
+    METHODN_NONAME,  /* bitore */ 
     METHODN_NONAME,  /* ALTLET */ 
     METHODN_NONAME,  /* QUESTION */ 
     METHODN_opItr,  /* Itr */ 
@@ -242,8 +241,6 @@ static knh_methodn_t knh_token_array_tomethodn[] = {
     METHODN_opGt,  /* Gt */ 
     METHODN_opGte,  /* Gte */ 
     METHODN_opMatch,  /* Match */ 
-    METHODN_opLor,  /* Lor */ 
-    METHODN_opFmt,  /* Fmt */ 
     METHODN_opLshift,  /* Lshift */ 
     METHODN_opRshift,  /* Rshift */ 
     METHODN_opAdd,  /* Add */ 
@@ -252,6 +249,7 @@ static knh_methodn_t knh_token_array_tomethodn[] = {
     METHODN_opMod,  /* Mod */ 
     METHODN_opMul,  /* Mul */ 
     METHODN_opLand,  /* Land */ 
+    METHODN_opLor,  /* Lor */ 
     METHODN_opLnot,  /* Lnot */ 
     METHODN_opXor,  /* Xor */ 
     METHODN_opNext,  /* Next */ 
@@ -260,7 +258,7 @@ static knh_methodn_t knh_token_array_tomethodn[] = {
     METHODN_NONAME,  /* NUM */ 
     METHODN_NONAME,  /* STR */ 
     METHODN_NONAME,  /* TSTR */ 
-    METHODN_NONAME,  /* EVAL */ 
+    METHODN_NONAME,  /* ESTR */ 
     METHODN_NONAME,  /* LABEL */ 
     METHODN_NONAME,  /* METAN */ 
     METHODN_NONAME,  /* PROPN */ 
@@ -344,11 +342,11 @@ static int knh_token_array_getOpPriority[] = {
     0,  /* rshifte */ 
     0,  /* adde */ 
     0,  /* sube */ 
-    0,  /* mule */ 
     0,  /* dive */ 
     0,  /* mode */ 
-    0,  /* bitore */ 
+    0,  /* mule */ 
     0,  /* bitande */ 
+    0,  /* bitore */ 
     0,  /* ALTLET */ 
     2,  /* QUESTION */ 
     2,  /* Itr */ 
@@ -372,8 +370,6 @@ static int knh_token_array_getOpPriority[] = {
     9,  /* Gt */ 
     9,  /* Gte */ 
     9,  /* Match */ 
-    11,  /* Lor */ 
-    12,  /* Fmt */ 
     15,  /* Lshift */ 
     15,  /* Rshift */ 
     20,  /* Add */ 
@@ -382,6 +378,7 @@ static int knh_token_array_getOpPriority[] = {
     24,  /* Mod */ 
     24,  /* Mul */ 
     27,  /* Land */ 
+    27,  /* Lor */ 
     27,  /* Lnot */ 
     27,  /* Xor */ 
     28,  /* Next */ 
@@ -390,7 +387,7 @@ static int knh_token_array_getOpPriority[] = {
     99,  /* NUM */ 
     99,  /* STR */ 
     99,  /* TSTR */ 
-    99,  /* EVAL */ 
+    99,  /* ESTR */ 
     0,  /* LABEL */ 
     0,  /* METAN */ 
     99,  /* PROPN */ 
@@ -474,11 +471,11 @@ static int knh_token_array_getOpSize[] = {
     0,  /* rshifte */ 
     0,  /* adde */ 
     0,  /* sube */ 
-    0,  /* mule */ 
     0,  /* dive */ 
     0,  /* mode */ 
-    0,  /* bitore */ 
+    0,  /* mule */ 
     0,  /* bitande */ 
+    0,  /* bitore */ 
     0,  /* ALTLET */ 
     0,  /* QUESTION */ 
     1,  /* Itr */ 
@@ -502,8 +499,6 @@ static int knh_token_array_getOpSize[] = {
     2,  /* Gt */ 
     2,  /* Gte */ 
     2,  /* Match */ 
-    0,  /* Lor */ 
-    0,  /* Fmt */ 
     0,  /* Lshift */ 
     0,  /* Rshift */ 
     0,  /* Add */ 
@@ -512,6 +507,7 @@ static int knh_token_array_getOpSize[] = {
     2,  /* Mod */ 
     2,  /* Mul */ 
     0,  /* Land */ 
+    0,  /* Lor */ 
     1,  /* Lnot */ 
     2,  /* Xor */ 
     1,  /* Next */ 
@@ -520,7 +516,7 @@ static int knh_token_array_getOpSize[] = {
     0,  /* NUM */ 
     0,  /* STR */ 
     0,  /* TSTR */ 
-    0,  /* EVAL */ 
+    0,  /* ESTR */ 
     0,  /* LABEL */ 
     0,  /* METAN */ 
     0,  /* PROPN */ 
@@ -604,11 +600,11 @@ static knh_bool_t knh_token_array_isEndOfStmt[] = {
     0,  /* rshifte */ 
     0,  /* adde */ 
     0,  /* sube */ 
-    0,  /* mule */ 
     0,  /* dive */ 
     0,  /* mode */ 
-    0,  /* bitore */ 
+    0,  /* mule */ 
     0,  /* bitande */ 
+    0,  /* bitore */ 
     0,  /* ALTLET */ 
     0,  /* QUESTION */ 
     0,  /* Itr */ 
@@ -632,8 +628,6 @@ static knh_bool_t knh_token_array_isEndOfStmt[] = {
     0,  /* Gt */ 
     0,  /* Gte */ 
     0,  /* Match */ 
-    0,  /* Lor */ 
-    0,  /* Fmt */ 
     0,  /* Lshift */ 
     0,  /* Rshift */ 
     0,  /* Add */ 
@@ -642,6 +636,7 @@ static knh_bool_t knh_token_array_isEndOfStmt[] = {
     0,  /* Mod */ 
     0,  /* Mul */ 
     0,  /* Land */ 
+    0,  /* Lor */ 
     0,  /* Lnot */ 
     0,  /* Xor */ 
     0,  /* Next */ 
@@ -650,7 +645,7 @@ static knh_bool_t knh_token_array_isEndOfStmt[] = {
     0,  /* NUM */ 
     0,  /* STR */ 
     0,  /* TSTR */ 
-    0,  /* EVAL */ 
+    0,  /* ESTR */ 
     0,  /* LABEL */ 
     1,  /* METAN */ 
     0,  /* PROPN */ 
@@ -726,7 +721,6 @@ Token *new_Token__parse(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t
     break;
     case '%':
         if(ISB(t, "%=")) { tt = TT_MODE; break; }
-        if(ISB(t, "%%")) { tt = TT_FMT; break; }
         if(ISB(t, "%")) { tt = TT_MOD; break; }
     break;
     case '&':
@@ -947,9 +941,6 @@ int knh_bytes_istoken(knh_bytes_t t, int ch)
 {
     switch(ch) {
 
-    case '%':
-        if(ISB(t, "%")) { return 1; }
-        return 0; 
     case '&':
         if(ISB(t, "&")) { return 1; }
         return 0; 
@@ -974,11 +965,11 @@ int knh_bytes_istoken(knh_bytes_t t, int ch)
         if(ISB(t, ">>")) { return 1; }
         if(ISB(t, "+")) { return 1; }
         if(ISB(t, "-")) { return 1; }
-        if(ISB(t, "*")) { return 1; }
         if(ISB(t, "/")) { return 1; }
         if(ISB(t, "%")) { return 1; }
-        if(ISB(t, "|")) { return 1; }
+        if(ISB(t, "*")) { return 1; }
         if(ISB(t, "&")) { return 1; }
+        if(ISB(t, "|")) { return 1; }
         if(ISB(t, "??")) { return 1; }
         if(ISB(t, "==")) { return 1; }
         if(ISB(t, "=")) { return 1; }
