@@ -1,14 +1,14 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -38,38 +38,8 @@ extern "C" {
 
 /* ======================================================================== */
 
-//static Object *fiboMethod = NULL;
-//
-///* ------------------------------------------------------------------------ */
-///* @method Int! Script.initfibo() */
-//
-//METHOD knh__Script_initfibo(Ctx *ctx, knh_sfp_t *sfp)
-//{
-//	fiboMethod = (Object*)knh_Class_getMethod(ctx, CLASS_Script, METHODN_fibo);
-//	KNH_ASSERT(IS_Method(fiboMethod));
-//}
-//
-//
-///* ------------------------------------------------------------------------ */
-///* @method Int! Script.fibo(Int! n) */
-//
-//METHOD knh__Script_fibo(Ctx *ctx, knh_sfp_t *sfp)
-//{
-//	JIT_SETESP(ctx, 2);
-//	JIT_iLTn(ctx, 2, 1, 3);
-//	JIT_bJIFF(ctx, NPC, L_JUMP, 2);
-//	JIT_RETo(ctx, -1, ctx->share->tInt[1  - KNH_TINT_MIN]);
-//	L_JUMP:;
-//	JIT_iSUBn(ctx, 5, 1, 1);
-//	JIT_FCALL(ctx, 3, 3, 0, fiboMethod);
-//	JIT_iSUBn(ctx, 6, 1, 2);
-//	JIT_FCALL(ctx, 4, 3, 0, fiboMethod);
-//	JIT_iADD(ctx, -1, 3, 4);
-//	JIT_RET(ctx);
-//}
-
 /* ------------------------------------------------------------------------ */
-/* @method void Script.changeChannel(Int! ch) */
+/* @ method void Script.changeChannel(Int! ch) */
 
 METHOD knh__Script_changeChannel(Ctx *ctx, knh_sfp_t *sfp)
 {
@@ -78,7 +48,7 @@ METHOD knh__Script_changeChannel(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void Script.hook(String! funcname) */
+/* @ method void Script.hook(String! funcname) */
 
 METHOD knh__Script_hook(Ctx *ctx, knh_sfp_t *sfp)
 {
@@ -88,6 +58,53 @@ METHOD knh__Script_hook(Ctx *ctx, knh_sfp_t *sfp)
 
 /* ------------------------------------------------------------------------ */
 
+#if defined(KNHX_API)
+///**
+//  regist C function to Method.
+//  @param method  method name,types... like this -> "void System.echo(String str)"
+//  @param func    function pointer.  that func must have normal library API
+//                 METHOD funcname (Ctx *ctx, knh_sfp_t* ){}
+//  @return true/false
+// */
+//KNHAPI(int) knh_registLocalFunction(konoha_t konoha, char* method, knh_fmethod func)
+//{
+//    Ctx *ctx = konoha.ctx;
+//    knh_setCurrentContext(ctx);
+//    knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+//
+//    knh_cwb_t *cwb = new_cwb(ctx);
+//    knh_Bytes_write(ctx, cwb->ba, B(method));
+//    InputStream *in = new_BytesInputStream(ctx, cwb->ba, cwb->pos, knh_Bytes_size(cwb->ba));
+//
+//    KNH_LPUSH(ctx, in);
+//    knh_InputStream_setEncoding(ctx, in, KNH_ENC);
+//
+//    Stmt *stmt = knh_InputStream_parseStmt(ctx, in, 0/*isData*/);
+//    KNH_LPUSH(ctx, stmt);
+//
+//    NameSpace *ns = knh_getCurrentNameSpace(ctx);
+//    knh_Stmt_compile(ctx, stmt, DP(ns)->nsname, 1);
+//
+//    knh_methodn_t mn = knh_Method_fromName(ctx, method);
+//    knh_Method_syncfunc(mtd, func);
+//    KNH_LOCALBACK(ctx, lsfp);
+//    return;
+//}
+
+#endif
+
+/* ------------------------------------------------------------------------ */
+/* [pacage test] */
+
+/* ------------------------------------------------------------------------ */
+
+int package_init(Ctx *ctx)
+{
+    return 1;
+}
+
+/* ------------------------------------------------------------------------ */
+
 #ifdef __cplusplus
 }
 #endif
