@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -62,6 +62,7 @@ void konoha_init(void)
 		knh_thread_key_create(&ctxkey);
 	}
 #endif
+	knh_srand(0);
 	isInit = 1;
 }
 
@@ -122,19 +123,18 @@ KNHAPI(Ctx*) konoha_getThreadContext(Ctx *ctx)
 /* ======================================================================== */
 /* [option] */
 
-static int debugMode = 0;
+static int debugMode2 = 0;    // this is -d2 debug mode for konoha itself
 
 /* ----------------------------------------------------------------------- */
 
-KNHAPI(int) konoha_isDebugMode()
+KNHAPI(int) konoha_isDebugMode2()
 {
-	return debugMode;
+	return debugMode2;
 }
 
 /* ----------------------------------------------------------------------- */
 
-static
-void konoha_checksize()
+static void konoha_dumpInit()
 {
 	fprintf(stderr, "sizeof(knh_int_t)=%lu, sizeof(void*)=%lu\n", sizeof(knh_intptr_t), sizeof(void*));
 	KNH_ASSERT(sizeof(knh_intptr_t) == sizeof(void*));
@@ -156,7 +156,7 @@ KNHAPI(int) konoha_parseopt(konoha_t konoha, int argc, char **argv)
 		char *t = argv[n];
 		if(t[0] != '-') return n;
 		if(t[1] == 's' && t[2] == 0) {
-			knh_Context_setAdmin(konoha.ctx, 1);
+			knh_Context_setTrusted(konoha.ctx, 1);
 		}
 		if(t[1] == 'c' && t[2] == 0) {
 			knh_Context_setCompiling(konoha.ctx, 1);
@@ -164,9 +164,12 @@ KNHAPI(int) konoha_parseopt(konoha_t konoha, int argc, char **argv)
 		if(t[1] == 'v') {
 			knh_Context_setVerbose(konoha.ctx, 1);
 		}
+		if(t[1] == 'g') {
+			knh_Context_setDebug(konoha.ctx, 1);
+		}
 		if(t[1] == 'd' && t[2] == '2' && t[3] == 0) {
-			debugMode = 1;
-			konoha_checksize();
+			debugMode2 = 1;
+			konoha_dumpInit();
 		}
 	}
 	return n;
@@ -402,6 +405,7 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 	Ctx *ctx = konoha.ctx;
 	konoha_setCurrenContext(ctx);
 	knh_Context_setInteractive(ctx, 1);
+	knh_Context_setDebug(ctx, 1);
 	{
 		knh_System__dump(ctx, ctx->sys, KNH_STDOUT, (String*)KNH_NULL);
 		int linenum, linecnt = 0;
@@ -472,7 +476,7 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 				knh_cwb_clear(cwb);
 				knh_Bytes_write(ctx, cwb.ba, B(bufc));
 			}
-			knh_Bytes_putc(ctx, cwb.ba, ';');
+			//knh_Bytes_putc(ctx, cwb.ba, ';');
 			knh_Bytes_putc(ctx, cwb.ba, '\n');
 			{
 				InputStream *in = new_BytesInputStream(ctx, cwb.ba, cwb.pos, knh_Bytes_size(cwb.ba));
