@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -67,16 +67,6 @@ void knh_srand(knh_uint_t seed)
 #endif
 }
 
-/* ------------------------------------------------------------------------ */
-/* void System.setRandomSeed(Int seed) */
-
-METHOD knh__System_setRandomSeed(Ctx *ctx, knh_sfp_t *sfp)
-{
-	knh_uint_t seed = IS_NULL(sfp[1].o) ? 0 : p_uinteger(sfp[1]);
-	knh_srand(seed);
-	KNH_RETURN_void(ctx, sfp);
-}
-
 /* ------------------------------------------------------------------------ */
 
 knh_uint_t knh_rand()
