@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -134,10 +134,15 @@ knh_Object_t *new_UnusedObject(Ctx *ctx)
 {
 	KNH_LOCK(ctx, LOCK_MEMORY, NULL);
 	int tindex = ctx->share->ObjectPageTableSize;
-	if(unlikely(!(tindex < KNH_TOBJECTPAGE_SIZE))) {
-		KNH_EXIT("Enlarge %s x %s", KNH_OBJECTPAGE_SIZE, KNH_TOBJECTPAGE_SIZE);
-		KNH_UNLOCK(ctx, LOCK_MEMORY, NULL);
-		return NULL;
+	if(unlikely(!(tindex < ctx->share->ObjectPageTableMaxSize))) {
+		size_t newsize = ctx->share->ObjectPageTableMaxSize * 2;
+		knh_ObjectPageTable_t *newpage
+			= (knh_ObjectPageTable_t*)KNH_MALLOC(ctx, sizeof(knh_ObjectPageTable_t) * newsize);
+		knh_bzero(newpage, sizeof(knh_ObjectPageTable_t) * newsize);
+		knh_memcpy(newpage, ctx->share->ObjectPageTable, ctx->share->ObjectPageTableMaxSize);
+		KNH_FREE(ctx, ctx->share->ObjectPageTable, ctx->share->ObjectPageTableMaxSize);
+		ctx->share->ObjectPageTable = newpage;
+		ctx->share->ObjectPageTableMaxSize = newsize;
 	}
 	ctx->share->ObjectPageTableSize += 1;
 	KNH_UNLOCK(ctx, LOCK_MEMORY, NULL);
@@ -171,7 +176,7 @@ knh_Object_t *new_UnusedObject(Ctx *ctx)
 	}
 	((Context*)ctx)->unusedObject = ((knh_Object_t*)h) + 1;
 	((Context*)ctx)->unusedObjectSize += cnt;
-	DBG2_P("*** new page tindex=%d, ctxid=%d unused=%p,%d ***",
+	DBG_P("*** new object page tindex=%d, ctxid=%d unused=%p,%d ***",
 			tindex, ctx->ctxid, ctx->unusedObject, (int)ctx->unusedObjectSize);
 	return ctx->unusedObject;
 }
