@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -110,8 +110,6 @@ METHOD knh__System_stackdump(Ctx *ctx, knh_sfp_t *sfp)
 /* ======================================================================== */
 /* [call] */
 
-char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int n);
-
 void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, knh_code_t *pc)
 {
 	DBG2_ASSERT(IS_Method(sfp[-1].mtd));
@@ -119,16 +117,16 @@ void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, knh_code_t *pc)
 	knh_class_t this_cid = knh_Object_cid(sfp[0].o);
 	int i, argc = knh_Method_isVarArgs(mtd) ? (ctx->esp - sfp) : knh_Method_psize(mtd);
 	for(i = 1; i < argc; i++) {
-		knh_type_t type = knh_Method_ptype(ctx, this_cid, mtd, i);
+		knh_type_t type = knh_Method_ptype(ctx, mtd, this_cid, i);
 		if(IS_NULL(sfp[i].o)) {
 			if(IS_NNTYPE(type)) {
-				emsg = "Null!!: "; goto L_THROWERR;
+				emsg = "Null!!: the parameter %d of %M"; goto L_THROWERR;
 			}
 		}
 		else {
 			knh_class_t reqc = CLASS_type(type);
 			if(!knh_class_instanceof(ctx, knh_Object_cid(sfp[i].o), reqc)) {
-				emsg = "Type!!: "; goto L_THROWERR;
+				emsg = "Null!!: the parameter %d of %M"; goto L_THROWERR;
 			}
 		}
 	}
@@ -136,10 +134,10 @@ void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, knh_code_t *pc)
 
 	L_THROWERR:
 	{
-		char buf[CLASSNAME_BUFSIZ];
-		knh_snprintf(buf, sizeof(buf), emsg);
-		knh_format_mtdparam(ctx, buf+9, sizeof(buf)-9, mtd, i);
-		knh_throw__s(ctx, buf, knh_Method_file(ctx, sfp[-1].mtd), knh_Method_pctoline(sfp[-1].mtd, pc));
+		knh_cwb_t cwb = new_cwb(ctx);
+		knh_printf(ctx, cwb.w, emsg, i, DP(mtd)->mn);
+		String *s = new_String__cwb(ctx, cwb);
+		knh_throw(ctx, (Object*)s, knh_Method_file(ctx, sfp[-1].mtd), knh_Method_pctoline(sfp[-1].mtd, pc));
 	}
 }
 
@@ -200,42 +198,6 @@ void knh_Exception_addStackTrace(Ctx *ctx, Exception *o, String *msg)
 	}
 }
 
-///* ------------------------------------------------------------------------ */
-///* @method void Context.%s(OutputStream w, String m) */
-//
-//void knh_Context__s(Ctx *ctx, Context *o, OutputStream *w, String *m)
-//{
-//	char *fn = NULL;
-//	int  line = 0;
-//	knh_sfp_t *mtd_sfp = NULL, *prev_sfp = o->esp;
-//	while(o->stack <= prev_sfp) {
-//		if(IS_Method(prev_sfp[0].o) && prev_sfp[0].op != 0) {
-//			if(mtd_sfp != NULL) {
-//				fn = knh_Method_file(ctx, prev_sfp[0].mtd);
-//				line = knh_Method_pctoline(prev_sfp[0].mtd, (knh_code_t*)mtd_sfp[-1].op);
-//				break;
-//			}
-//			mtd_sfp = prev_sfp;
-//		}
-//		prev_sfp--;
-//	}
-//	knh_putc(ctx, w, '[');
-//	if(o->doing == NULL) {
-//		knh_write__s(ctx, w, KONOHA_NAME);
-//	}
-//	else {
-//		knh_write__s(ctx, w, o->doing);
-//	}
-//	knh_putc(ctx, w, ']');
-//	if(fn != NULL) {
-//		knh_putc(ctx, w, '(');
-//		knh_write__s(ctx, w, KNH_SAFEFILE(fn));
-//		knh_putc(ctx, w, ':');
-//		knh_write__i(ctx, w, (knh_int_t)line);
-//		knh_putc(ctx, w, ')');
-//	}
-//}
-
 /* ------------------------------------------------------------------------ */
 
 static
@@ -251,7 +213,9 @@ String *knh_stackf_getStackTraceMsg(Ctx *ctx, knh_sfp_t *sfp)
 		line = knh_Method_pctoline(mtd, pc);
 	}
 	knh_cwb_t cwb = new_cwb(ctx);
-	knh_write_cidmn(ctx, cwb.w, DP(mtd)->cid, DP(mtd)->mn);
+	knh_write_cid(ctx, cwb.w, DP(mtd)->cid);
+	knh_putc(ctx, cwb.w, '.');
+	knh_write_mn(ctx, cwb.w, DP(mtd)->mn);
 	knh_putc(ctx, cwb.w, ':');
 	knh_write_fline(ctx, cwb.w, fn, line);
 	return new_String__cwb(ctx, cwb);
