@@ -8,7 +8,7 @@
  * You may choose one of the following two licenses when you use konoha.
  * See www.konohaware.org/license.html for further information.
  *
- * (1) GNU General Public License 2.0      (with    KONOHA_UNDER_GPL2)
+ * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
  * (2) Konoha Software Foundation License 1.0
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -141,7 +141,8 @@ void knh_Const__man(Ctx *ctx, knh_class_t cid, OutputStream *w)
 	int hasCaption = 0;
 	for(i = 0; i < size; i++) {
 		if(!hasCaption) {
-			knh_printf(ctx, w, "%s\n", knh_message_text(KMSG_CONST));
+			knh_write__s(ctx, w, _("Const"));
+			knh_write_EOL(ctx, w);
 			hasCaption = 1;
 		}
 		knh_printf(ctx, w, "\t%s.%s: %O\n", CLASSN(cid),
@@ -239,7 +240,7 @@ knh_fieldn_t knh_tfieldn_common(Ctx *ctx, knh_bytes_t n, knh_fieldn_t def)
 
 int knh_bytes_isOptionalMT(knh_bytes_t t)
 {
-	KNH_ASSERT(t.buf[0] != '%');
+	DBG2_ASSERT(t.buf[0] != '%');
 	return (isdigit(t.buf[0]) || knh_bytes_index(t, '%') > 0);
 }
 
@@ -386,7 +387,7 @@ String *new_String__mn(Ctx *ctx, knh_methodn_t mn)
 
 /* ------------------------------------------------------------------------ */
 
-knh_methodn_t knh_tName_getMethodn(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def)
+knh_methodn_t konoha_getMethodName(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def)
 {
 	if(tname.len == 0 || tname.len > 64) {
 		DBG2_P("invalid method name '%s'", tname.buf);
@@ -563,31 +564,61 @@ knh_drvapi_t *konoha_getDriverAPI(Ctx *ctx, int type, knh_bytes_t name)
 static
 char *knh_lookup_packageScript(Ctx *ctx, char *buf, size_t bufsiz, knh_bytes_t pkgname)
 {
-	char pbuf[40];
+	char pbuf[40], fbuf[40], *file1 = NULL, *file2 = NULL, *rootdir;
+	knh_index_t loc = knh_bytes_rindex(pkgname, '.');
 	knh_format_bytes(pbuf, sizeof(pbuf), pkgname);
-	char *p = knh_getenv("KONOHA_PACKAGE");
-	if(p != NULL) {
-		knh_snprintf(buf, bufsiz, "%s/%s/%s.k", p, pbuf, pbuf);
+	if(loc == -1) {
+		file1 = pbuf;
+	}
+	else {
+		file1 = pbuf + loc + 1;
+		knh_format_bytes(fbuf, sizeof(fbuf), knh_bytes_first(pkgname, loc));
+		file2 = fbuf;
+	}
+
+	rootdir = knh_getenv("KONOHA_PACKAGE");
+	if(rootdir != NULL) {
+		knh_snprintf(buf, bufsiz, "%s/%s/%s.k", rootdir, pbuf, file1);
+		if(knh_isfile(ctx, B(buf))) return buf;
+		if(file2 != NULL) {
+			knh_snprintf(buf, bufsiz, "%s/%s/%s.k", rootdir, pbuf, file2);
+		}
 		if(knh_isfile(ctx, B(buf))) return buf;
 	}
 #ifdef KNH_PREFIX
-	knh_snprintf(buf, bufsiz, "%s/lib/konoha/%s/%s.k", KNH_PREFIX, pbuf, pbuf);
-#else
-	knh_snprintf(buf, bufsiz, "%s/package/%s/%s.k", knh_String_tochar(knh_homeDir), pbuf, pbuf);
-#endif
+	knh_snprintf(buf, bufsiz, "%s/lib/konoha/%s/%s.k", KNH_PREFIX, pbuf, file1);
 	if(knh_isfile(ctx, B(buf))) return buf;
-	p = knh_getenv("HOME");
-	if(p != NULL) {
-		knh_snprintf(buf, bufsiz, "%s/.konoha/%s/%s.k", p, pbuf, pbuf);
+	if(file2 != NULL) {
+		knh_snprintf(buf, bufsiz, "%s/lib/konoha/%s/%s.k", KNH_PREFIX, pbuf, file2);
 		if(knh_isfile(ctx, B(buf))) return buf;
 	}
-	if(ctx->cwd == NULL) {
-		knh_snprintf(buf, bufsiz, "./.konoha/%s/%s.k", pbuf, pbuf);
-	}
-	else {
-		knh_snprintf(buf, bufsiz, "%s/.konoha/%s/%s.k", ctx->cwd, pbuf, pbuf);
-	}
+#else
+	knh_snprintf(buf, bufsiz, "%s/package/%s/%s.k", knh_String_tochar(knh_homeDir), pbuf, file1);
 	if(knh_isfile(ctx, B(buf))) return buf;
+	if(file2 != NULL) {
+		knh_snprintf(buf, bufsiz, "%s/package/%s/%s.k", knh_String_tochar(knh_homeDir), pbuf, file2);
+		if(knh_isfile(ctx, B(buf))) return buf;
+	}
+#endif
+
+	rootdir = knh_getenv("HOME");
+	if(rootdir != NULL) {
+		knh_snprintf(buf, bufsiz, "%s/.konoha/%s/%s.k", rootdir, pbuf, file1);
+		if(knh_isfile(ctx, B(buf))) return buf;
+		if(file2 != NULL) {
+			knh_snprintf(buf, bufsiz, "%s/.konoha/%s/%s.k", rootdir, pbuf, file2);
+			if(knh_isfile(ctx, B(buf))) return buf;
+		}
+	}
+
+	if(ctx->cwd == NULL) rootdir = ".";
+	else rootdir = ctx->cwd;
+	knh_snprintf(buf, bufsiz, "%s/.konoha/%s/%s.k", rootdir, pbuf, file1);
+	if(knh_isfile(ctx, B(buf))) return buf;
+	if(file2 != NULL) {
+		knh_snprintf(buf, bufsiz, "%s/.konoha/%s/%s.k", rootdir, pbuf, file2);
+		if(knh_isfile(ctx, B(buf))) return buf;
+	}
 	return NULL;
 }
 
@@ -661,32 +692,6 @@ NameSpace *knh_System_getNameSpace(Ctx *ctx, knh_bytes_t name)
 	}
 }
 
-/* ======================================================================== */
-/* [makesepc] */
-
-KNHAPI(void) konoha_addClassSpecFunc(Ctx *ctx, char *urn, knh_fspec func)
-{
-	if(func != (knh_fspec)NULL) {
-		DBG2_P("adding.. '%s'", urn);
-		KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-		knh_DictSet_set(ctx, DP(ctx->sys)->SpecFuncDictSet, new_String__T(ctx, urn), (knh_uintptr_t)func);
-		KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
-	}
-}
-/* ------------------------------------------------------------------------ */
-
-ClassSpec *konoha_getClassSpec(Ctx *ctx, knh_class_t bcid, knh_bytes_t urn)
-{
-	urn = knh_Runtime_aliasURN(ctx, urn);
-	DBG2_P("finding.. '%s'", urn.buf);
-	KNH_LOCK(ctx, LOCK_SYSTBL, NULL);
-	knh_fspec func = (knh_fspec)knh_DictSet_get__b(DP(ctx->sys)->SpecFuncDictSet, urn);
-	KNH_UNLOCK(ctx, LOCK_SYSTBL, NULL);
-	if(func != NULL) {
-		return func(ctx, bcid, NULL);
-	}
-	return (ClassSpec*)(KNH_NULL);
-}
 
 /* ------------------------------------------------------------------------ */
 
